/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.compare.impl;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.compare.MutableDocumentAssociation;
import org.xmloperator.framework.document.compare.CancelException;
import org.xmloperator.framework.document.compare.BuildingDocumentAssociation;
import org.xmloperator.framework.document.compare.DocumentComparator;
import org.xmloperator.framework.view.status.MessageDisplay;
import org.xmloperator.compare.impl.ElementAssociationImpl;
import org.xmloperator.compare.impl.DocumentAssociationImpl;

/**
 * This is a document comparator whose result (the document association)
 *   doesn'nt depend on the order of the elements in the two documents.
 */
public final class DocumentComparatorImpl implements DocumentComparator {
  private MessageDisplay messageDisplay;
  private DocumentAssociationImpl documentAssociation = null;

  public DocumentComparatorImpl(MessageDisplay messageDisplay) {
    this.messageDisplay = messageDisplay;
  }

  ////////////////////////////////////
  // DocumentComparator implementation
  ////////////////////////////////////

  public DocumentAssociation compare(
      Document document1, Document document2) {
    this.documentAssociation =
        new DocumentAssociationImpl(document1, document2);
    this.buildRootAssociation(documentAssociation);
    try {
      if (messageDisplay != null)
        messageDisplay.displayMessage(
            "Computting element characterizations ...");
      this.documentAssociation.computeElementCharacterizations();
      if (messageDisplay != null)
        messageDisplay.displayMessage(
            "Associating elements by characteristic ...");
      this.documentAssociation.associateEquallyCharacterizedElements();
      if (messageDisplay != null)
        messageDisplay.displayMessage("Associating ancestor elements (1)...");
      this.documentAssociation.associateAncestors();
      if (messageDisplay != null)
        messageDisplay.displayMessage("Associating ancestor elements (2)...");
      this.documentAssociation.associateAncestors();
      return this.documentAssociation;
    }
    catch (CancelException exception) {
      this.documentAssociation.clear();
      return null;
    }
    finally {
      this.documentAssociation = null;
    }
  }

  private void buildRootAssociation(
      MutableDocumentAssociation documentAssociation) {
    Element rootElement1 =
        documentAssociation.getDocument1().getDocumentElement();
    Element rootElement2 =
        documentAssociation.getDocument2().getDocumentElement();
    String namespaceURI = rootElement1.getNamespaceURI();
    String localName = rootElement2.getLocalName();
    if (localName.equals(rootElement2.getNodeName())
        && namespaceURI != null ?
        namespaceURI.equals(rootElement2.getNamespaceURI()) :
        rootElement2.getNamespaceURI() == null) {
      documentAssociation.addElementAssociation(rootElement1, rootElement2);
    }
  }

  public void cancel() {
    if (messageDisplay != null)
      messageDisplay.displayMessage("Canceling ...");
    DocumentAssociationImpl documentAssociation = this.documentAssociation;
    if (documentAssociation != null)
      documentAssociation.cancel();
  }
}
