/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.compare.impl;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.compare.ElementCharacterization;
import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;
import org.xmloperator.framework.document.compare.ElementAssociation;

public final class ElementAssociationImpl implements ElementAssociation {
  private String namespaceURI;
  private String localName;
  private Element element1;
  private Element element2;
  private Boolean areElementsEqual = null;
  private int descendantCount = 1;

  public ElementAssociationImpl(Element element1, Element element2) {
    if (element1 == null || element2 == null)
      throw new IllegalArgumentException();
    this.namespaceURI = element1.getNamespaceURI();
    this.localName = element1.getLocalName();
    if (!element2.getLocalName().equals(this.localName))
      throw new IllegalArgumentException("Different local names");
    {
      String namespaceURI2 = element2.getNamespaceURI();
      if (this.namespaceURI != null ?
          !namespaceURI2.equals(this.namespaceURI) : namespaceURI2 != null)
        throw new IllegalArgumentException("Different namespaces");
    }
    this.element1 = element1;
    this.element2 = element2;
  }

  ////////////////////////////////////
  // ElementAssociation implementation
  ////////////////////////////////////

  public String getNamespaceURI() {
    return this.namespaceURI;
  }

  public String getLocalName() {
    return this.localName;
  }

  public Element getElement1() {
    return element1;
  }

  public Element getElement2() {
    return element2;
  }

  public Element getAssociatedElement(Element element) {
    if (element == element1)
      return element2;
    if (element == element2)
      return element1;
    return null;
  }

  public boolean areElementsEqual(
      ElementCharacterizationAccess elementCharacterizationAccess) {
    if (this.areElementsEqual == null) {
      this.areElementsEqual =
          elementCharacterizationAccess.
          getElementCharacterization(this.element1).
          equals(elementCharacterizationAccess.
                 getElementCharacterization(this.element2)) ?
          Boolean.TRUE : Boolean.FALSE;
    }
    return this.areElementsEqual.booleanValue();
  }

  public int getDescendantCount() {
    return this.descendantCount;
  }

  public void addDescendantCount(int addedCount) {
    this.descendantCount+= addedCount;
  }
}
