/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.global;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.List;
import javax.swing.ImageIcon;

import org.apache.xerces.util.XMLChar;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.framework.view.global.DocumentTypeInfoView;
import org.xmloperator.framework.view.global.TransformerInfoView;
import org.xmloperator.framework.view.global.FileChooser;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.dialog.Dialog;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.dialog.DialogListSelection;
import org.xmloperator.framework.view.dialog.DialogNewDocument;
import org.xmloperator.framework.view.dialog.DialogElementKey;
import org.xmloperator.framework.view.dialog.DialogLocator;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.global.GlobalFileControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.ViewObserverImpl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.TextFileEditorImpl;
import org.xmloperator.util.view.DialogValidText;
import org.xmloperator.util.view.DialogSelectionImpl;
import org.xmloperator.util.view.DialogListSelectionImpl;
import org.xmloperator.view.frame.DocumentTypeInfoFrame;
import org.xmloperator.view.frame.TransformerInfoFrame;
import org.xmloperator.view.dialog.DialogTransformerSelection;
import org.xmloperator.view.dialog.DialogDocumentTypeSelection;
import org.xmloperator.view.dialog.DialogDocumentTypeInfo;
import org.xmloperator.view.dialog.DialogTransformerInfo;
import org.xmloperator.view.dialog.FileChooserImpl;
import org.xmloperator.view.dialog.DialogNewDocumentImpl;
import org.xmloperator.view.dialog.DialogElementKeyImpl;
import org.xmloperator.view.dialog.DialogLocatorImpl;

public final class GlobalViewControlImpl implements GlobalViewControl {
  private static ImageIcon toolImageIcon =
    new ImageIcon(Constants.TOOL + ".gif");
  // Resources
  private Options options;
  // Controls
  private GlobalControl globalControl = null;
  private GlobalFileControl globalFileControl = null;
  private GlobalOptionsControl globalOptionsControl = null;
  // Views
  private TextFileEditorImpl traceEditor = null;
  private DocumentTypeInfoFrame documentTypeInfoView = null;
  private TransformerInfoFrame transformerInfoView = null;
  private FileChooser fileChooser = null;
  private DialogListSelectionImpl dialogReopen = null;
  private DialogValidText dialogIdentifier = null;
  private DialogValidText dialogLocalName = null;
  private DialogListSelectionImpl dialogSelectionName = null;
  private DialogSelectionImpl dialogSelectionPrefix = null;
  private DialogSelectionImpl dialogSelectionInstance = null;
  private DialogSelectionImpl dialogSelectionOperator = null;
  private DialogSelectionImpl dialogSelectionTransformer = null;
  private DialogDocumentTypeSelection dialogSelectionDocumentType = null;
  private DialogNewDocumentImpl dialogNewDocument = null;
  private DialogDocumentTypeInfo dialogDocumentTypeInfo = null;
  private DialogTransformerInfo dialogTransformerInfo = null;
  private DialogElementKeyImpl dialogElementKey = null;
  private DialogLocatorImpl dialogLocator = null;
  private ViewObserverImpl globalViewObserver = null;

  public GlobalViewControlImpl(Options options,
    ViewObserverImpl globalViewObserver) {
    this.options = options;
    this.globalViewObserver = globalViewObserver;
  }

  public void setControleurs(
    GlobalControl globalControl,
    GlobalFileControl globalFileControl,
    GlobalOptionsControl globalOptionsControl) {
    this.globalControl = globalControl;
    this.globalFileControl = globalFileControl;
    this.globalOptionsControl = globalOptionsControl;
  }

  private void setIconImage(Frame frame) {
    if (toolImageIcon.getIconHeight() > 0)
      frame.setIconImage(toolImageIcon.getImage());
  }

  private void verifyControlers() {
    if (globalFileControl == null || globalOptionsControl == null)
      throw new RuntimeException("No controlers");
  }

  private String getString(String resourceKey) {
    return ResourceBundle.getBundle(Label.CLASS).getString(resourceKey);
  }

  private Font getCurrentFont() {
    return options.getThreeStateOptionValue(Options.FONT)
        == ThreeStateOption.TRUE ?
        Constants.FONT_LARGE : Constants.FONT_NORMAL;
  }

  //////////////////////////////////
  // GlobalViewAccess implementation
  //////////////////////////////////

  public DocumentTypeInfoView createDocumentTypeInfoView() {
    if (documentTypeInfoView == null) {
      verifyControlers();
      documentTypeInfoView = new DocumentTypeInfoFrame(
        options.getOptionList(Options.DOCUMENTTYPE),
        globalControl, globalFileControl, globalOptionsControl);
      Frame frame = documentTypeInfoView.getFrame();
      setIconImage(frame);
      ViewUtils.moveComponentToCentre(frame);
    }
    return documentTypeInfoView;
  }

  public TransformerInfoView createTransformerInfoView() {
    if (transformerInfoView == null) {
      verifyControlers();
      transformerInfoView = new TransformerInfoFrame(
        options.getOptionList(Options.TRANSFORMER), globalOptionsControl);
      Frame frame = transformerInfoView.getFrame();
      setIconImage(frame);
      ViewUtils.moveComponentToCentre(frame);
    }
    return transformerInfoView;
  }

  public void updateGlobalViews() {
    if (documentTypeInfoView != null)
      documentTypeInfoView.update();
    if (transformerInfoView != null)
      transformerInfoView.update();
  }

  public FileChooser getFileChooser() {
    if (this.fileChooser == null)
      this.fileChooser = new FileChooserImpl(this.options);
    return this.fileChooser;
  }

  public DialogListSelection getDialogReopen() {
    if (this.dialogReopen == null) {
      this.dialogReopen = new DialogListSelectionImpl(null);
      this.dialogReopen.setContentFont(this.getCurrentFont());
      this.dialogReopen.setTitle(getString(Label.REOUVRIR_DOCUMENT));
      this.dialogReopen.setContentPreferredSize(new Dimension(500, 200));
      this.dialogReopen.pack();
      ViewUtils.moveComponentToCentre(this.dialogReopen);
    }
    return this.dialogReopen;
  }

  public DialogText getDialogIdentifier(String invite) {
    if (this.dialogIdentifier == null) {
      this.dialogIdentifier = new DialogValidText(null);
      this.dialogIdentifier.setContentFont(this.getCurrentFont());
      if (invite != null) {
        if (this.dialogIdentifier.getTitle() == null)
          this.dialogIdentifier.setTitle(invite);
        this.dialogIdentifier.setInvite(invite);
      }
      ViewUtils.moveComponentToCentre(this.dialogIdentifier);
    }
    else if (invite != null)
      this.dialogIdentifier.setInvite(invite);
    return this.dialogIdentifier;
  }

  public DialogText getDialogLocalName(String invite) {
    if (invite == null)
      invite = Label.LOCAL_NAME;
    if (this.dialogLocalName == null) {
      this.dialogLocalName = new DialogValidText(null) {
        protected boolean isTextValid() {
          return super.isTextValid() && XMLChar.isValidNCName(getText());
        }
      };
      this.dialogLocalName.setColums(15);
      this.dialogLocalName.setContentFont(this.getCurrentFont());
      this.dialogLocalName.setTitle(invite);
      this.dialogLocalName.setInvite(invite);
      ViewUtils.moveComponentToCentre(this.dialogLocalName);
    }
    else
      this.dialogLocalName.setInvite(invite);
    return this.dialogLocalName;
  }

  public DialogListSelection getDialogSelectionName() {
    if (this.dialogSelectionName == null) {
      this.dialogSelectionName = new DialogListSelectionImpl(null);
      this.dialogSelectionName.setContentFont(this.getCurrentFont());
      this.dialogSelectionName.setContentPreferredSize(new Dimension(300, 200));
      this.dialogSelectionName.pack();
      ViewUtils.moveComponentToCentre(this.dialogSelectionName);
    }
    return this.dialogSelectionName;
  }

  public DialogSelection getDialogSelectionPrefix() {
    if (this.dialogSelectionPrefix == null) {
      this.dialogSelectionPrefix = new DialogSelectionImpl(null);
      this.dialogSelectionPrefix.setContentFont(this.getCurrentFont());
      this.dialogSelectionPrefix.setTitle(getString(Label.PREFIX));
      this.dialogSelectionPrefix.setInvite(Label.PREFIX);
      ViewUtils.moveComponentToCentre(this.dialogSelectionPrefix);
    }
    return this.dialogSelectionPrefix;
  }

  public DialogSelection getDialogSelectionInstance() {
    if (this.dialogSelectionInstance == null) {
      this.dialogSelectionInstance = new DialogSelectionImpl(null);
      this.dialogSelectionInstance.setContentFont(this.getCurrentFont());
      this.dialogSelectionInstance.setTitle(
          getString(Label.SELECTION_D_INSTANCE));
      this.dialogSelectionInstance.setInvite(Label.INSTANCE_OUTIL);
      ViewUtils.moveComponentToCentre(this.dialogSelectionInstance);
    }
    return this.dialogSelectionInstance;
  }

  public DialogSelection getDialogSelectionOperator() {
    if (this.dialogSelectionOperator == null) {
      this.dialogSelectionOperator = new DialogSelectionImpl(null);
      this.dialogSelectionOperator.setContentFont(this.getCurrentFont());
      this.dialogSelectionOperator.setTitle(getString(Label.OPERATOR));
      this.dialogSelectionOperator.setInvite(Label.OPERATOR);
      ViewUtils.moveComponentToCentre(this.dialogSelectionOperator);
    }
    return this.dialogSelectionOperator;
  }

  public DialogSelection getDialogSelectionTransformer() {
    if (this.dialogSelectionTransformer == null) {
      this.dialogSelectionTransformer = new DialogTransformerSelection(
          null, this.globalOptionsControl);
      this.dialogSelectionTransformer.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogSelectionTransformer);
    }
    return this.dialogSelectionTransformer;
  }

  public DialogSelection getDialogSelectionDocumentType(
      boolean isCancelEnabled, boolean isUserRelativeDTDSystemIdRequired) {
    if (this.dialogSelectionDocumentType == null) {
      this.dialogSelectionDocumentType = new DialogDocumentTypeSelection(null);
      this.dialogSelectionDocumentType.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogSelectionDocumentType);
    }
    this.dialogSelectionDocumentType.setCancelEnabled(isCancelEnabled);
    this.dialogSelectionDocumentType.setUserRelativeDTDSystemIdRequired(
        isUserRelativeDTDSystemIdRequired);
    return this.dialogSelectionDocumentType;
  }

  public DialogNewDocument getDialogNewDocument() {
    if (this.dialogNewDocument == null) {
      this.dialogNewDocument = new DialogNewDocumentImpl(
          null, this.globalOptionsControl, this.options, this.getFileChooser());
      this.dialogNewDocument.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogNewDocument);
    }
    return this.dialogNewDocument;
  }

  public Dialog getDialogDocumentTypeInfo(
      DocumentTypeInfo documentTypeInfo, List transformerInfoList,
      boolean isNameReadOnly, boolean isAddKeyReadOnly) {
    if (this.dialogDocumentTypeInfo == null) {
      this.dialogDocumentTypeInfo = new DialogDocumentTypeInfo(
          null, this.options, this.globalOptionsControl);
      this.dialogDocumentTypeInfo.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogDocumentTypeInfo);
    }
    this.dialogDocumentTypeInfo.setNameReadOnly(isNameReadOnly);
    this.dialogDocumentTypeInfo.setAddKeyReadOnly(isAddKeyReadOnly);
    this.dialogDocumentTypeInfo.setTransformerInfoList(transformerInfoList);
    this.dialogDocumentTypeInfo.setDocumentTypeInfo(documentTypeInfo);
    return this.dialogDocumentTypeInfo;
  }

  public Dialog getDialogTransformerInfo(
      TransformerInfo transformerInfo, boolean isKeyReadOnly) {
    if (this.dialogTransformerInfo == null) {
      this.dialogTransformerInfo = new DialogTransformerInfo(null);
      this.dialogTransformerInfo.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogTransformerInfo);
    }
    this.dialogTransformerInfo.setTransformerInfo(transformerInfo);
    this.dialogTransformerInfo.setKeyReadOnly(isKeyReadOnly);
    return this.dialogTransformerInfo;
  }

  public DialogElementKey getDialogElementKey() {
    if (this.dialogElementKey == null) {
      this.dialogElementKey = new DialogElementKeyImpl(null);
      this.dialogElementKey.setContentFont(this.getCurrentFont());
      this.dialogElementKey.pack();
      ViewUtils.moveComponentToCentre(this.dialogElementKey);
    }
    return this.dialogElementKey;
  }

  public DialogLocator getDialogLocator() {
    if (this.dialogLocator == null) {
      this.dialogLocator = new DialogLocatorImpl(null);
      this.dialogLocator.setContentFont(this.getCurrentFont());
      this.dialogLocator.pack();
      ViewUtils.moveComponentToCentre(this.dialogLocator);
    }
    return this.dialogLocator;
  }

  ///////////////////////////////////
  // GlobalViewControl implementation
  ///////////////////////////////////

  public void showExceptionTrace() {
    if (traceEditor == null) {
      traceEditor = new TextFileEditorImpl(new Dimension(760, 400));
      setIconImage(traceEditor);
      traceEditor.setTitle(getString(Label.EXCEPTION_TRACE));
      ViewUtils.moveComponentToCentre(traceEditor);
    }
    try {
      traceEditor.load(ExceptionControler.getTraceReader());
      traceEditor.show();
      ExceptionControler.acceptException();
    }
    catch (IOException exception) {
    }
  }

  public TextFileEditor createTextFileEditor() {
    TextFileEditorImpl editor = new TextFileEditorImpl();
    setIconImage(editor);
    ViewUtils.moveComponentToCentre(editor);
    globalViewObserver.addFrame(editor);
    editor.setViewObserver(globalViewObserver);
    return editor;
  }

  public void deleteGlobalView() {
    if (this.traceEditor != null) {
      this.traceEditor.dispose();
      this.traceEditor = null;
    }
    if (this.documentTypeInfoView != null) {
      this.documentTypeInfoView.dispose();
      this.documentTypeInfoView = null;
    }
    if (this.transformerInfoView != null) {
      this.transformerInfoView.dispose();
      this.transformerInfoView = null;
    }
    this.fileChooser = null;
    this.dialogReopen = null;
    this.dialogIdentifier = null;
    this.dialogLocalName = null;
    this.dialogSelectionName = null;
    this.dialogSelectionPrefix = null;
    this.dialogSelectionInstance = null;
    this.dialogSelectionOperator = null;
    this.dialogSelectionTransformer = null;
    this.dialogSelectionDocumentType = null;
    this.dialogNewDocument = null;
    this.dialogDocumentTypeInfo = null;
    this.dialogTransformerInfo = null;
    this.dialogElementKey = null;
    this.dialogLocator = null;
    this.globalViewObserver.deleteAllFrames();
  }
}
