/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.instance;

import java.awt.Frame;
import java.io.IOException;
import java.io.File;
import java.io.InputStream;
import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.Transformer;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.xerces.util.XMLChar;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.parse.ParseErrorList;
import org.xmloperator.framework.document.util.Parameter;
import org.xmloperator.framework.document.util.ResourceContext;
import org.xmloperator.framework.document.manage.DocumentManager;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.view.node.NodeViewer;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.dialog.DialogNewDocument;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.task.Task;
import org.xmloperator.framework.control.task.TaskControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.util.dom.NamespaceUtils;
import org.xmloperator.util.dom.Cleaner;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.FileDialogs;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.option.impl.DocumentTypeInfoImpl;
import org.xmloperator.option.impl.TransformerInfoImpl;
import org.xmloperator.option.impl.ParameterImpl;
import org.xmloperator.view.parameter.ParameterListDialog;
import org.xmloperator.operation.OperationPrinter;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.operation.OperationSaver;
import org.xmloperator.operation.OperationLoader;
import org.xmloperator.transform.AbstractTransformer;
import org.xmloperator.transform.XSLTTransformer;
import org.xmloperator.control.io.IOUtils;
import org.xmloperator.control.io.ToolEntityResolverImpl;
import org.xmloperator.control.task.DocumentLoadTask;
import org.xmloperator.control.task.SchemaLoadTask;
import org.xmloperator.control.task.SelectedNodeSchemaConformanceControlTask;
import org.xmloperator.control.task.DocumentSaveTask;
import org.xmloperator.control.task.DocumentTransformTask;
import org.xmloperator.control.task.TransformBatchTask;
import org.xmloperator.control.task.MessageDisplayTask;
import org.xmloperator.control.task.ToolOptionsReloadTask;

public final class FileControlImpl implements FileControl {
  private TaskControl taskControl;
  private GlobalOptionsControl globalOptionsControl;
  private GlobalViewControl globalViewControl;
  private ToolControl toolControl;
  private ViewControl viewControl;
  private OptionsControl optionsControl;
  private EditControl editControl;
  private DocumentViewControl documentViewControl;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private OperationStack operationStack;
  private OperationSaver operationSaver;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public FileControlImpl(TaskControl taskControl,
    GlobalOptionsControl globalOptionsControl,
    GlobalViewControl globalViewControl, ToolControl toolControl,
    ViewControl viewControl, OptionsControl optionsControl,
    EditControl editControl, DocumentViewControl documentViewControl,
    FullyFledgedDocument fullyFledgedDocument, Options options,
    OperationStack operationStack, OperationSaver operationSaver) {
    if (taskControl == null || globalOptionsControl == null
      || globalViewControl == null
      || toolControl == null || viewControl == null
      || optionsControl == null || editControl == null
      || documentViewControl == null || fullyFledgedDocument == null
      || options == null || operationStack == null
      || operationSaver == null)
      throw new IllegalArgumentException();
    this.taskControl = taskControl;
    this.globalOptionsControl = globalOptionsControl;
    this.globalViewControl = globalViewControl;
    this.toolControl = toolControl;
    this.viewControl = viewControl;
    this.optionsControl = optionsControl;
    this.editControl = editControl;
    this.documentViewControl = documentViewControl;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.operationStack = operationStack;
    this.operationSaver = operationSaver;
  }

  private void displayExceptionReadOnly() {
    ExceptionDialogs.displayErrorMessage("Exception",
      getString(Label.LE_DOCUMENT_EST_EN_LECTURE_SEULE));
  }

  private void displayMessage(String message) {
    ToolView toolView = viewControl.getToolView();
    if (toolView != null)
      toolView.getToolStatusBar().displayMessage(message);
  }

  private void clearMessage() {
    displayMessage(" ");
  }

  private void displayResourceMessage(String key) {
    displayMessage(getString(key));
  }

  private DocumentLoadTask createLoadDocumentTask(
      String documentSystemId, boolean isSchemaKept) {
    return new DocumentLoadTask(
        this.fullyFledgedDocument, this.options, this.operationStack,
        this.operationSaver, this.globalOptionsControl,
        (GlobalViewAccess)this.globalViewControl,
        (ViewAccess)this.viewControl, documentSystemId, isSchemaKept);
  }

  /////////////////////////////
  // FileControl implementation
  /////////////////////////////

  public boolean closeDocument() {
    clearMessage();
    if (fullyFledgedDocument.getDocument() != null
        && !isDocumentClearingAllowed())
      return false;
    DocumentLoadTask.closeDocument(
        fullyFledgedDocument, options,
        operationStack, (OperationPrinter)operationSaver,
        (ViewAccess)viewControl);
    toolControl.deleteCurrentDocumentAssociation();
    return true;
  }

  private boolean isDocumentClearingAllowed() {
    DocumentManager documentManager = fullyFledgedDocument.getDocumentManager();
    if (documentManager == null  || !documentManager.isDocumentModified()
      || options.getThreeStateOptionValue(Options.OPERATIONSSTORED)
      == ThreeStateOption.TRUE)
      return true;
    boolean isAbandonConfirme = JOptionPane.showConfirmDialog(null,
      getString(Label.DO_YOU_CONFIRM) + " "
      + getString(Label.L_ABANDON_DES_MODIFICATIONS) + " ?",
      getString(Label.DOCUMENT_NON_ENREGISTRE),
      JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION;
    return isAbandonConfirme;
  }

  public void newDocument(DocumentTypeInfo documentTypeInfo,
      String documentPathName) {
    if (!closeDocument())
      return;
    if (documentTypeInfo == null || documentPathName == null) {
      DialogNewDocument dialogNewDocument =
          this.globalViewControl.getDialogNewDocument();
      dialogNewDocument.setTitle(getString(Label.NOUVEAU_DOCUMENT));
      dialogNewDocument.setDocumentPathName(
          options.getStringOptionValue(Options.XMLPATH) + "noName.xml");
      dialogNewDocument.show();
      if (!dialogNewDocument.isOK())
        return;
      documentTypeInfo = dialogNewDocument.getDocumentTypeInfo();
      documentPathName = dialogNewDocument.getDocumentPathName();
      if (documentPathName.lastIndexOf('.') < 0)
        documentPathName += ".xml";
    }
    if (!FileDialogs.isFileWritable(documentPathName))
      return;
    String rootElementName = documentTypeInfo.getDocumentTypeName();
    if (rootElementName == null || rootElementName.length() == 0) {
      DialogText dialog =
          this.globalViewControl.getDialogLocalName(Label.ROOT_ELEMENT_NAME);
      dialog.show();
      if (!dialog.isOK())
        return;
      rootElementName = dialog.getText();
    }
    String prefix = NamespaceUtils.prefixFromQName(rootElementName);
    String namespaceId = documentTypeInfo.getNamespaceId();
    if (prefix.length() != 0
        && (namespaceId == null || namespaceId.length() == 0)) {
      DialogText dialog = this.globalViewControl.getDialogIdentifier(
          Label.NAMESPACE_IDENTIFIER);
      dialog.show();
      if (!dialog.isOK())
        return;
      namespaceId = dialog.getText();
    }
    try {
      DTDInfo dTDInfo = documentTypeInfo.getDTDInfo();
      IOUtils.createAndSaveAMinimalDocument(
          rootElementName, namespaceId,
          dTDInfo.getDTDPublicId(), dTDInfo.getDocumentRelativeDTDSystemId(),
          documentPathName, options);
      String normalisedSystemId = FileUtils.reducedSystemId(documentPathName);
      if (normalisedSystemId == null) {
        displayMessage(
            getString(Label.RESSOURCE_INVALIDE) + " : " + documentPathName);
        return;
      }
      {
        ResourceContext resourceContext =
            IOUtils.addResourceFirst(options, normalisedSystemId);
        resourceContext.setDocumentTypeAdditionalKey(
            documentTypeInfo.getDocumentTypeAdditionalKey());
      }
      this.createLoadDocumentTask(normalisedSystemId, false).loadDocument();
      Document document = fullyFledgedDocument.getDocument();
      if (document == null)
        return;
      new SchemaLoadTask(
          options,
          fullyFledgedDocument, documentViewControl, null, -1).loadSchema();
      if (!fullyFledgedDocument.isLazy()) {
        editControl.addChildsIfNecessary(document.getDocumentElement());
        if (fullyFledgedDocument.getDocumentManager().isDocumentModified())
          saveDocumentInternal(true);
      }
      documentViewControl.controlSelectedNodeSchemaConformance();
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
  }

  public void loadDocument(String documentSystemId) {
    loadDocument(documentSystemId, null);
  }

  public void loadDocument(String documentSystemId, String schemaSystemId) {
    if (documentSystemId == null)
      throw new IllegalArgumentException();
    if (!closeDocument())
      return;
    taskControl.appendTask(this.createLoadDocumentTask(
        documentSystemId, false));
    taskControl.appendTask(new SchemaLoadTask(options,
        fullyFledgedDocument, documentViewControl, schemaSystemId, -1));
    taskControl.appendTask(
        new SelectedNodeSchemaConformanceControlTask(documentViewControl));
  }

  public void loadLastDocument() {
    ResourceContext resource =
      (ResourceContext)options.getOption(Options.RESOURCE);
    if (resource != null)
      loadDocument(resource.getDocumentSystemId());
  }

  public final void setDocumentTypeAdditionalKey(
    final String documentTypeAdditionalKey) {
    if (fullyFledgedDocument.getDocument() == null)
      return;
    fullyFledgedDocument.setDocumentTypeAdditionalKey(
        documentTypeAdditionalKey);
    taskControl.appendTask(new SchemaLoadTask(options,
        fullyFledgedDocument, documentViewControl, null, -1));
    taskControl.appendTask(
        new SelectedNodeSchemaConformanceControlTask(documentViewControl));
  }

  public void saveDocument(boolean andReload) {
    saveDocument(null, andReload);
  }

  public void saveDocument(String pathName, boolean andReload) {
    if (fullyFledgedDocument.getDocument() == null) {
      displayResourceMessage(Label.THIS_ACTION_REQUIRES_AN_OPENED_DOCUMENT);
      return;
    }
    clearMessage();
    if (fullyFledgedDocument.isReadOnly()) {
      displayExceptionReadOnly();
      return;
    }
    boolean isNewName = true;
    boolean isFileNameAutoIncrement = options.getThreeStateOptionValue(
        Options.FILENAMEAUTOMATICINCREMENTING) == ThreeStateOption.TRUE;
    if (pathName != null) {
      if (!FileDialogs.isFileWritable(pathName))
        return;
      String systemId = FileUtils.reducedSystemId(pathName);
      if (systemId == null) {
        displayResourceMessage(Label.ILLEGAL_PATH_NAME);
        return;
      }
      fullyFledgedDocument.setDocumentSystemId(systemId);
    }
    else if (isFileNameAutoIncrement)
      incrementFileName();
    else
      isNewName = false;
    if (isNewName) {
      ResourceContext resourceContext =
          IOUtils.addResourceFirst(options,
          fullyFledgedDocument.getDocumentSystemId());
      resourceContext.setDocumentTypeAdditionalKey(
          fullyFledgedDocument.getDocumentTypeAdditionalKey());
    }
    renameFileIfExist();
    saveDocumentInternal(andReload);
  }

  private void incrementFileName() {
    String systemId = fullyFledgedDocument.getDocumentSystemId();
    int dotPos = systemId.lastIndexOf(".");
    int versionPos = dotPos;
    if (versionPos >= 0)
      while (versionPos > 0) {
        char ch = systemId.charAt(versionPos - 1);
        if (ch < '0' || ch > '9')
          break;
        versionPos--;
      }
    int version = 0;
    if (versionPos < dotPos)
      try {
        version = Integer.parseInt(systemId.substring(versionPos, dotPos));
      }
      catch (NumberFormatException exception) {
      }
    String versionString = new Integer(version + 1).toString();
    systemId = dotPos >= 0 ? systemId.substring(0, versionPos) + versionString
      + systemId.substring(dotPos) : systemId + versionString;
    fullyFledgedDocument.setDocumentSystemId(systemId);
  }

  private void renameFileIfExist() {
    String pathName = FileUtils.fromSystemIdToFilePathName(
      fullyFledgedDocument.getDocumentSystemId());
    if (pathName == null)
      return;
    File file = new File(pathName);
    if (!file.exists())
      return;
    int dotPos = pathName.lastIndexOf(".");
    String bakPathName =
      (dotPos >= 0 ? pathName.substring(0, dotPos) : pathName) + ".bak";
    File bakFile = new File(bakPathName);
    bakFile.delete();
    file.renameTo(bakFile);
  }

  private void saveDocumentInternal(boolean andReload) {
    this.saveDocumentInternal(false, andReload);
  }

  private void saveDocumentInternal(boolean haveElementsToBeIndented,
                                    boolean andReload) {
    try {
      operationSaver.rewriteIfOpen();
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
    boolean isOperationStackToBeClearedAfterSave =
      options.getThreeStateOptionValue(Options.CLEARUNDOAFTERSAVE)
      == ThreeStateOption.TRUE;
    taskControl.appendTask(new DocumentSaveTask(
      fullyFledgedDocument, haveElementsToBeIndented, (ViewAccess)viewControl,
      isOperationStackToBeClearedAfterSave ? operationStack : null));
    String documentSystemId = fullyFledgedDocument.getDocumentSystemId();
    if (andReload)
      taskControl.appendTask(this.createLoadDocumentTask(
          documentSystemId, true));
    if (documentSystemId.endsWith("." + Constants.TOOL_OPTIONS_FILE_EXTENSION))
      taskControl.appendTask(new ToolOptionsReloadTask(optionsControl));
    taskControl.appendTask(
      new SelectedNodeSchemaConformanceControlTask(documentViewControl));
  }

  public void transformDocument(boolean useLastDestinationPathName) {
    Document sourceDocument = fullyFledgedDocument.getDocument();
    if (sourceDocument == null) {
      displayResourceMessage(Label.THIS_ACTION_REQUIRES_AN_OPENED_DOCUMENT);
      return;
    }
    clearMessage();
    DocumentTypeInfo documentTypeInfo =
        fullyFledgedDocument.getDocumentTypeInfo();
    String transformerName = documentTypeInfo != null ?
                           documentTypeInfo.getDefaultTransformerName() : null;
    List transformerInfoList =
        TransformerInfoImpl.createTransformerInfoSubList(
        options.getOptionList(Options.TRANSFORMER),
        sourceDocument.getDocumentElement().getNodeName());
    if (transformerInfoList.isEmpty()) {
      ExceptionDialogs.displayErrorMessage(getString(Label.TRANSFORM),
          getString(Label.NO_TRANSFORMER_DEFINED));
      return;
    }
    TransformerInfo transformerInfo =
        transformerName != null && transformerName.length() != 0 ?
        TransformerInfoImpl.getTransformerInfo(
        transformerInfoList, transformerName) : null;
    if (transformerInfo == null) {
      Collections.sort(transformerInfoList);
      DialogSelection dialog =
          this.globalViewControl.getDialogSelectionTransformer();
      dialog.setItemList(transformerInfoList);
      dialog.show();
      if (!dialog.isOK())
        return;
      transformerInfo = (TransformerInfo)dialog.getSelectedItem();
    }
    String destinationPathName = useLastDestinationPathName ?
        transformerInfo.getLastDestinationPathname() : "";
    if (destinationPathName.length() == 0) {
      destinationPathName = this.globalViewControl.getFileChooser().
          chooseDocumentPathName(true, getString(Label.TRANSFORM_AS));
      if (destinationPathName == null
          || !FileDialogs.isFileWritable(destinationPathName))
        return;
    }
    transformerInfo.setLastDestinationPathname(destinationPathName);
    Transformer transformer =
        TransformBatchTask.createTransformer(transformerInfo, options);
    if (transformer == null)
      return;
    if (!copyParameterList(transformerInfo.getParamNames() ,transformer))
      return;
    if (transformer instanceof AbstractTransformer)
      ((AbstractTransformer)transformer).setIdBrowser(
          fullyFledgedDocument.getIdManager());
    taskControl.appendTask(new DocumentTransformTask(transformer,
        transformerInfo, new DOMSource(sourceDocument), destinationPathName,
        options, (ViewAccess)viewControl));
  }

  private boolean copyParameterList(String paramNames, Transformer transformer)
  {
    if (paramNames == null || paramNames.trim().length() == 0)
      return true;
    List parameterList = new ArrayList();
    StringTokenizer tokenizer = new StringTokenizer(paramNames);
    while (tokenizer.hasMoreTokens()) {
      String paramName = tokenizer.nextToken();
      parameterList.add(
          new ParameterImpl(paramName, getParameterValue(options, paramName)));
    }
    ParameterListDialog parameterListDialog =
      new ParameterListDialog(null, parameterList, options);
    parameterListDialog.setTitle(getString(Label.TRANSFORM_PARAMETERS));
    ViewUtils.moveComponentToCentre(parameterListDialog);
    parameterListDialog.show();
    if (!parameterListDialog.isOK())
      return false;
    List optionsParameterList = options.getOptionList(Options.PARAMETER);
    for (int k = 0; k < parameterList.size(); k++) {
      Parameter parameter = (Parameter)parameterList.get(k);
      transformer.setParameter(parameter.getName(), parameter.getValue());
      optionsParameterList.remove(parameter);
      optionsParameterList.add(parameter);
    }
    return true;
  }

  private static final String getParameterValue(
      Options options, String parameterName) {
    List optionsParameterList = options.getOptionList(Options.PARAMETER);
    for (int k = 0; k < optionsParameterList.size(); k++) {
      Parameter parameter = (Parameter)optionsParameterList.get(k);
      if (parameter.getName().equals(parameterName))
        return parameter.getValue();
    }
    return "";
  }

  public void processTransformBatch(String batchDocumentURI) {
    if (batchDocumentURI == null)
      throw new IllegalArgumentException();
    taskControl.appendTask(
        (Task)new TransformBatchTask(batchDocumentURI, options, viewControl));
  }

  public void loadSchema(String schemaSystemId, int schemaType) {
    taskControl.appendTask((Task)new SchemaLoadTask(options,
        fullyFledgedDocument, documentViewControl,
        schemaSystemId, schemaType));
  }

  public void closeDTD() {
    Document document = fullyFledgedDocument.getDocument();
    DTD originalDTD = fullyFledgedDocument.getOriginalDTD();
    if (document == null || originalDTD == null
      || fullyFledgedDocument.getDTD() == originalDTD)
      return;
    try {
      fullyFledgedDocument.setDTD(originalDTD);
    }
    catch (IdException exception) {
      ExceptionDialogs.displayException("ID exception", exception);
      Element locationElement = exception.getAttribute().getOwnerElement();
      ToolView toolView = documentViewControl.getToolView();
      if (locationElement != null && toolView != null)
        toolView.setSelectedNode(locationElement, true);
      fullyFledgedDocument.getIdManager().clearAndSetEnabled(false);
    }
    documentViewControl.displaySelectedNode();
  }

  public void editSource() {
    clearMessage();
    editSource(fullyFledgedDocument.getDocumentSystemId(),
               fullyFledgedDocument.isReadOnly());
  }

  public void editSource(String systemId, boolean isReadOnly) {
    if (systemId == null)
      return;
    TextFileEditor editor = globalViewControl.createTextFileEditor();
    String pathname = FileUtils.fromSystemIdToFilePathName(systemId);
    try {
      if (pathname != null)
        editor.load(pathname);
      else if (systemId.startsWith("http://")) {
        InputStream inputStream = new ToolEntityResolverImpl(options, null).
                                  resolveEntity(null, systemId).getByteStream();
        editor.load(inputStream);
        isReadOnly = true;
      }
      else
        throw new IOException("Cannot edit " + systemId);
      editor.setTitle(systemId);
      editor.setReadOnly(isReadOnly);
      editor.show();
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
      if (editor != null)
        editor.dispose();
    }
  }

  public void removeEmptyTexts() {
    clearMessage();
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    if (fullyFledgedDocument.isReadOnly()) {
      displayExceptionReadOnly();
      return;
    }
    DocumentManager documentManager = fullyFledgedDocument.getDocumentManager();
    if (documentManager.isDocumentModified()) {
      displayResourceMessage(Label.OPERATION_INTERDITE);
      return;
    }
    if (JOptionPane.showConfirmDialog(
      null, getString(Label.DO_YOU_CONFIRM) + " "
      + getString(Label.REMOVING_THE_EMPTY_TEXTS) + " ?",
      getString(Label.IRREVERSIBLE_ACTION),
      JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
      return;
    int removeCount = Cleaner.removeEmptyTexts(document);
    if (removeCount == 0)
      return;
    this.removingEmptyTextsConsequences();
    saveDocumentInternal(true);
    taskControl.appendTask(new MessageDisplayTask(
        viewControl.getToolView().getToolStatusBar(),
        new Integer(removeCount).toString() + " "
        + getString(Label.NODES_REMOVED)));
  }

  public void indent() {
    clearMessage();
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    if (fullyFledgedDocument.isReadOnly()) {
      displayExceptionReadOnly();
      return;
    }
    DocumentManager documentManager = fullyFledgedDocument.getDocumentManager();
    if (documentManager.isDocumentModified()) {
      displayResourceMessage(Label.OPERATION_INTERDITE);
      return;
    }
    {
      Element mixedContentElement =
          Cleaner.mixedContentElement(document.getDocumentElement());
      if (mixedContentElement != null) {
        viewControl.getToolView().setSelectedNode(mixedContentElement, true);
        displayResourceMessage(Label.NOT_ALLOWED_BECAUSE_MIXED_CONTENT);
        return;
      }
    }
    if (JOptionPane.showConfirmDialog(
      null, getString(Label.DO_YOU_CONFIRM) + " "
      + getString(Label.INDENTING_ELEMENTS) + " ?",
      getString(Label.IRREVERSIBLE_ACTION),
      JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
      return;
    Cleaner.removeEmptyTexts(document);
    this.removingEmptyTextsConsequences();
    saveDocumentInternal(true, true);
  }

  private void removingEmptyTextsConsequences() {
    fullyFledgedDocument.getDocumentManager().setDocumentModified(true);
    {
      ToolView toolView = viewControl.getToolView();
      toolView.getToolStatusBar().displayEditStatus(ToolStatusBar.EDIT_MODIFIED);
      toolView.getDocumentTreeView().getDomTree().nodeStructureChanged(
          fullyFledgedDocument.getDocument().getDocumentElement());
    }
    fullyFledgedDocument.getNodeList(FullyFledgedDocument.NODELIST_NODELIST).
        clear();
    fullyFledgedDocument.getNodeList(FullyFledgedDocument.NODELIST_NODESTACK).
        clear();
    operationStack.clear();
  }
}
