/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.operators;

import java.util.Hashtable;

import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.document.manage.AttributeManager;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.framework.control.operators.Element;
import org.xmloperator.framework.control.operators.TreeElement;
import org.xmloperator.framework.control.operators.Document;
import org.xmloperator.operation.core.TransitionOperation;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.control.operators.ElementImpl; // Reciprocal import

public final class DocumentImpl implements Document {
  private final IdManagerRead idManager;
  private final NodeManager nodeManager;
  private final AttributeManager attributeManager;
  private final ToolViewAccess toolViewAccess;
  private final OperationStack operationStack;
  private final org.w3c.dom.Document domDocument;
  private final ElementImpl rootElement;
  private final Hashtable fromDomElementToElement = new Hashtable();
  private boolean useTransitionOperation = false;

  public DocumentImpl(IdManagerRead idManager,
    NodeManager nodeManager, AttributeManager attributeManager,
    ToolViewAccess toolViewAccess, OperationStack operationStack,
    org.w3c.dom.Document domDocument) {
    if (idManager == null || nodeManager == null || attributeManager == null
        || toolViewAccess == null || operationStack == null
        || domDocument == null)
      throw new IllegalArgumentException();
    this.idManager = idManager;
    this.nodeManager = nodeManager;
    this.attributeManager = attributeManager;
    this.toolViewAccess = toolViewAccess;
    this.operationStack = operationStack;
    this.domDocument = domDocument;
    org.w3c.dom.Element documentElement = domDocument.getDocumentElement();
    this.rootElement = new ElementImpl(this, documentElement);
    this.fromDomElementToElement.put(documentElement, rootElement);
  }

  public NodeManager getNodeManager() {
    return this.nodeManager;
  }

  public AttributeManager getAttributeManager() {
    return this.attributeManager;
  }

  public ToolViewAccess getToolViewAccess() {
    return this.toolViewAccess;
  }

  public Element getElement(org.w3c.dom.Element domElement) {
    if (domElement == null)
      throw new IllegalArgumentException();
    Element element = (Element)fromDomElementToElement.get(domElement);
    if (element == null) {
      element = new ElementImpl(this, domElement);
      fromDomElementToElement.put(domElement, element);
    }
    return element;
  }

  public void executeAndStoreOperation(Operation operation)
    throws UndefinedNamespaceException, IdException, OperationException {
    if (useTransitionOperation)
      operationStack.executeAndAdd(new TransitionOperation());
    else
      useTransitionOperation = true;
    operationStack.executeAndAdd(operation);
  }

  public boolean useTransitionOperation() {
    return useTransitionOperation;
  }

  //////////////////////////
  // Document implementation
  //////////////////////////

  public TreeElement getRootElement() {
    return rootElement;
  }

  public Element getElement(String id) throws IdException {
    org.w3c.dom.Element domElement = idManager.getElementById(id);
    if (domElement == null)
      throw new IdException(IdException.CANNOT_FIND_ID, id);
    return getElement(domElement);
  }

  public Element createElement(String elementName) {
    return getElement(domDocument.createElement(elementName));
  }

  public String createId() throws IdException {
    return idManager.getNewVerifiedId();
  }

  public boolean isIdReferenced(String id) {
    return idManager.isIdReferenced(id);
  }
}
