/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.task.Task;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.util.io.MIME2Java;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.compare.DocumentsMerger;
import org.xmloperator.control.io.IOUtils;

public final class DocumentsMergeTask implements Task {
  private String documentPathName;
  private DTD dTD;
  private DocumentTypeInfo documentTypeInfo;
  private DocumentAssociation documentAssociation;
  private Options options;
  private ViewAccess viewAccess1;
  private ViewAccess viewAccess2;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public DocumentsMergeTask(String documentPathName,
    DTD dTD, DocumentTypeInfo documentTypeInfo,
    DocumentAssociation documentAssociation, Options options,
    ViewAccess viewAccess1, ViewAccess viewAccess2) {
    this.documentPathName = documentPathName;
    this.dTD = dTD;
    this.documentTypeInfo = documentTypeInfo;
    this.documentAssociation = documentAssociation;
    this.options = options;
    this.viewAccess1 = viewAccess1;
    this.viewAccess2 = viewAccess2;
  }

  public boolean mergeDocuments() {
    try {
      displayMessage(getString(Label.FUSION_EN_COURS));
      String encoding = options.getStringOptionValue(Options.DEFAULTENCODING);
      if (MIME2Java.convert(encoding) == null)
        encoding = Constants.DEFAULT_ENCODING;
      PrintWriter out =
        new PrintWriter(new OutputStreamWriter(
        new FileOutputStream(documentPathName), MIME2Java.convert(encoding)));
      final boolean isShortcutUsedForEmptyElement = false;
      final boolean addLineFeedAtStartTagEnd =
          options.getThreeStateOptionValue(Options.ADDLINEFEEDATSTARTTAGEND)
          == ThreeStateOption.TRUE;
      DocumentsMerger documentsMerger = new DocumentsMerger(out);
      documentsMerger.setShortcutUsedForEmptyElement(
          isShortcutUsedForEmptyElement);
      documentsMerger.setLineFeedAtStartTagEnd(addLineFeedAtStartTagEnd);
      int returnCode =
        documentsMerger.printMergedDocument(documentAssociation, dTD,
        documentTypeInfo.getDTDInfo().getDTDPublicId(),
        documentTypeInfo.getDTDInfo().getDocumentRelativeDTDSystemId(),
        null, encoding, false);
      out.close();
      String systemId = FileUtils.reducedSystemId(documentPathName);
      IOUtils.addResourceFirst(options, systemId);
      displayMessage(returnCode != 0 ?
        DomWriter.fromReportCodeToReplacedStringList(returnCode)
        + getString(Label.HAVE_BEEN_REPLACED_FOR_WRITING_OUT)
        : " ");
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
      return false;
    }
    return true;
  }

  private void displayMessage(String message) {
    ToolView toolView1 = viewAccess1.getToolView();
    if (toolView1 != null)
      toolView1.getToolStatusBar().displayMessage(message);
    ToolView toolView2 = viewAccess2.getToolView();
    if (toolView2 != null)
      toolView2.getToolStatusBar().displayMessage(message);
  }

  //////////////////////
  // Task implementation
  //////////////////////

  public boolean run() {
    return mergeDocuments();
  }
}
