/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.awt.Frame;
import java.util.ResourceBundle;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.relaxng.Schema;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.status.MessageDisplay;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.control.task.CancelableTask;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.document.relaxng.SchemaHandler;
import org.xmloperator.document.relaxng.SchemaToXMLTranslator;
import org.xmloperator.document.parse.ToolParser;
import org.xmloperator.control.io.ToolEntityResolverImpl;

public final class SchemaLoadTask implements CancelableTask {
  private static final boolean DEBUG_SCHEMA = false;
  private Options options;
  private FullyFledgedDocument fullyFledgedDocument;
  private DocumentViewControl documentViewControl;
  private String schemaSystemId;
  private int schemaType;
  private ToolEntityResolverImpl toolEntityResolver = null;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public SchemaLoadTask(Options options,
    FullyFledgedDocument fullyFledgedDocument,
    DocumentViewControl documentViewControl,
    String schemaSystemId, int schemaType) {
    if (fullyFledgedDocument == null || documentViewControl == null)
      throw new IllegalArgumentException();
    this.options = options;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.documentViewControl = documentViewControl;
    this.schemaSystemId = schemaSystemId;
    this.schemaType = schemaType;
  }

  public boolean loadSchema() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return false;
    if (schemaSystemId != null) {
      String normalisedSystemId = FileUtils.reducedSystemId(schemaSystemId);
      if (normalisedSystemId == null) {
        displayMessage(getString(Label.RESSOURCE_INVALIDE)
          + " : " + schemaSystemId);
        return false;
      }
      schemaSystemId = normalisedSystemId;
    }
    if (schemaSystemId == null || schemaSystemId.length() == 0) {
      DocumentTypeInfo documentTypeInfo =
        fullyFledgedDocument.getDocumentTypeInfo();
      if (documentTypeInfo == null)
        return false;
      if (options.getThreeStateOptionValue(Options.RNGAUTOMATICLOADING)
        == ThreeStateOption.TRUE) {
        schemaSystemId =
          documentTypeInfo.getRNGInfo().getRNGSimpleSyntaxSystemId();
        schemaType = DocumentTypeInfo.SCHEMA_TYPE_RNG;
      }
      if ((schemaSystemId == null || schemaSystemId.length() == 0)
        && options.getThreeStateOptionValue(Options.DTDAUTOMATICLOADING)
        == ThreeStateOption.TRUE
        && !fullyFledgedDocument.isThereDoctypeDecl()) {
        schemaSystemId =
          documentTypeInfo.getDTDInfo().getUserRelativeDTDSystemId();
        schemaType = DocumentTypeInfo.SCHEMA_TYPE_DTD;
      }
      if (schemaSystemId == null || schemaSystemId.length() == 0)
        return false;
    }
    ToolView toolView = documentViewControl.getToolView();
    MessageDisplay messageDisplay = toolView != null ?
      toolView.getToolStatusBar() : null;
    displayMessage(getString(Label.CHARGE) + " " + schemaSystemId);
    toolEntityResolver = new ToolEntityResolverImpl(options, messageDisplay);
    boolean canContinue = false;
    try {
      if (schemaType == DocumentTypeInfo.SCHEMA_TYPE_DTD
        || schemaSystemId.endsWith(".dtd")) {
        DTD dTD =
          ToolParser.parseDTD(schemaSystemId, toolEntityResolver);
        if (dTD.getElementNameList().isEmpty())
          throw new IOException("\"" + schemaSystemId
            + "\" is empty or non-existent");
        displayMessage(getString(Label.NOTE_IDS));
        fullyFledgedDocument.setDTD(dTD);
        canContinue = true;
      }
      else if (schemaType == DocumentTypeInfo.SCHEMA_TYPE_RNG
        || schemaSystemId.endsWith(".rng")) {
        Schema schema =
          SchemaHandler.loadSchema(schemaSystemId, toolEntityResolver);
        if (schema == null)
          throw new IOException("\"" + schemaSystemId
            + "\" is not a valid RELAX NG 1.0 simple syntax schema");
        if (DEBUG_SCHEMA)
          traceSchema(schema);
        displayMessage(getString(Label.NOTE_IDS));
        fullyFledgedDocument.setSchema(schema);
        canContinue = true;
      }
    }
    catch (SAXException exception) {
      ExceptionDialogs.displaySAXException(exception);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
    catch (IdException exception) {
      ExceptionDialogs.displayException("ID exception", exception);
      Element locationElement = exception.getAttribute().getOwnerElement();
      if (locationElement != null && toolView != null)
        toolView.setSelectedNode(locationElement, true);
      fullyFledgedDocument.getIdManager().clearAndSetEnabled(false);
    }
    /*
    catch (UndefinedNamespaceException exception) {
      ExceptionDialogs.displayException("Namespace exception", exception);
      Element locationElement = exception.getElement();
      if (locationElement != null && toolView != null)
        toolView.setSelectedNode(locationElement, true);
    }
    */
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    toolEntityResolver = null;
    documentViewControl.displaySelectedNode();
    clearMessage();
    return canContinue;
  }

  private void traceSchema(Schema schema) throws IOException {
    System.out.println("traceSchema");
    PrintWriter out = new PrintWriter(new OutputStreamWriter(
      new FileOutputStream("data/traceSchema.rng")));
    SchemaToXMLTranslator.printSchema(out, schema);
    out.close();
  }

  private void displayMessage(String message) {
    ToolView toolView = documentViewControl.getToolView();
    if (toolView != null)
      toolView.getToolStatusBar().displayMessage(message);
  }

  private void clearMessage() {
    displayMessage(" ");
  }

  ////////////////////////////////
  // CancelableTask implementation
  ////////////////////////////////

  public boolean run() {
    return loadSchema();
  }

  public void cancel() {
    ToolEntityResolverImpl toolEntityResolver = this.toolEntityResolver;
    if (toolEntityResolver != null)
      try {
        toolEntityResolver.closeAll();
      }
      catch (IOException exception) {
      }
  }
}
