/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.ErrorHandler;
import org.apache.xerces.xni.XNIException;
import org.apache.xerces.xni.XMLResourceIdentifier;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLEntityResolver;

import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.relaxng.Schema;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.framework.document.parse.ParseErrorList;
import org.xmloperator.framework.document.parse.DocumentParsingListener;
import org.xmloperator.framework.document.parse.ToolEntityResolver;
import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.manage.ElementOperationListener;
import org.xmloperator.framework.document.manage.TreeViewDocumentListener;
import org.xmloperator.framework.document.manage.DocumentModifiedListener;
import org.xmloperator.framework.document.manage.DocumentManager;
import org.xmloperator.framework.document.manage.AttributesContentAccess;
import org.xmloperator.framework.document.manage.AttributeChangeListener;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.util.PresseFragment;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.framework.document.util.NodeLinker;
import org.xmloperator.framework.document.elementKey.ElementKeyProvider;
import org.xmloperator.framework.document.DocumentAccess;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.io.MIME2Java;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.document.parse.ParseErrorListImpl;
import org.xmloperator.document.parse.ToolParser;
import org.xmloperator.document.id.IdManagerImpl;
import org.xmloperator.document.util.PresseFragmentImpl;
import org.xmloperator.document.util.VariableNodeListImpl;
import org.xmloperator.document.util.NodeIntervalImpl;
import org.xmloperator.document.util.NodeLinkerImpl;
import org.xmloperator.document.util.DocumentWriter;
import org.xmloperator.document.util.DocumentTypeInfoUtils;
import org.xmloperator.document.manage.DocumentManagerImpl;
import org.xmloperator.document.manage.AbstractElementManager;
import org.xmloperator.document.manage.DTDElementManager;
import org.xmloperator.document.manage.RNGElementManager;
import org.xmloperator.document.manage.DTDDocumentUtil;
import org.xmloperator.document.manage.RNGDocumentUtil;
import org.xmloperator.document.elementKey.ElementKeyProviderImpl;

public final class FullyFledgedDocumentImpl implements FullyFledgedDocument {
  private static final int PARSING_ID = 0;
  private static final int PARSING_FEATURE = 1;
  private static final String[][] PARSING_FEATURES = {
    {Options.PARSINGVALIDATION, "xml.org/sax/features/validation"},
    {Options.PARSINGVALIDATIONDYNAMIC,
      "apache.org/xml/features/validation/dynamic"},
    {Options.PARSINGVALIDATIONSCHEMA,
      "apache.org/xml/features/validation/schema"},
    {Options.PARSINGNONVALIDATINGLOADDTDGRAMMAR,
      "apache.org/xml/features/nonvalidating/load-dtd-grammar"},
    {Options.PARSINGNONVALIDATINGLOADEXTERNALDTD,
      "apache.org/xml/features/nonvalidating/load-external-dtd"},
    {Options.PARSINGDOMINCLUDEIGNORABLEWHITESPACE,
      "apache.org/xml/features/dom/include-ignorable-whitespace"}};
  private final Options options;
  private TreeViewDocumentListener treeViewDocumentListener = null;
  private final NodeLinkerImpl nodeLinker = new NodeLinkerImpl();
  private final ElementKeyProviderImpl elementKeyProvider;
  private String documentSystemId = null;
  private Document document = null;
  private String xMLVersion = null;
  private String encoding = null;
  private boolean isStandalone = false;
  private boolean isThereDoctypeDecl = false;
  private DTD originalDTD = null;
  private DTD usedDTD = null;
  private Schema schema = null;
  private int schemaType = SCHEMATYPE_NO;
  private String documentTypeAdditionalKey = null;
  private boolean isReadOnly = false;
  private boolean isLazy =false;
  private ParseErrorList parseErrorList = null;
  private IdManager idManager = null;
  private DocumentManagerImpl documentManager = null;
  private AbstractElementManager schemaElementManager = null;
  private DocumentUtil documentUtil = null;
  private PresseFragmentImpl presseFragment = null;
  private VariableNodeListImpl variableNodeList = null;
  private VariableNodeListImpl variableNodeStack = null;
  private NodeIntervalImpl nodeIntervalForMove = null;

  public FullyFledgedDocumentImpl(Options options) {
    this.options = options;
    this.elementKeyProvider =
        new ElementKeyProviderImpl((DocumentAccess)this, options);
  }

  public void setTreeViewDocumentListener(
      TreeViewDocumentListener treeViewDocumentListener) {
    this.treeViewDocumentListener = treeViewDocumentListener;
  }

  //////////////////////////////////////
  // FullyFledgedDocument implementation
  //////////////////////////////////////

  public void reset() {
    this.documentSystemId = null;
    this.document = null;
    this.xMLVersion = null;
    this.encoding = null;
    this.isStandalone = false;
    this.isThereDoctypeDecl = false;
    this.originalDTD = null;
    this.usedDTD = null;
    this.schema = null;
    this.schemaType = SCHEMATYPE_NO;
    this.documentTypeAdditionalKey = null;
    this.isReadOnly = false;
    this.isLazy = false;
    this.parseErrorList = null;
    this.documentManager = null;
    this.schemaElementManager = null;
    this.documentUtil = null;
    if (this.presseFragment != null) {
      this.presseFragment.done();
      this.presseFragment = null;
    }
    if (this.variableNodeList != null) {
      this.variableNodeList.done();
      this.variableNodeList = null;
    }
    if (this.variableNodeStack != null) {
      this.variableNodeStack.done();
      this.variableNodeStack = null;
    }
    this.nodeIntervalForMove = null;
    if (this.idManager != null) {
      this.idManager.clearAndSetEnabled(true);
      this.idManager = null;
    }
    this.nodeLinker.clear();
    this.elementKeyProvider.clear();
  }

  public void parse(String systemId, ToolEntityResolver toolEntityResolver)
    throws SAXException, XNIException, IOException, IdException {
    if (systemId == null)
      throw new IllegalArgumentException();
    this.reset();
    this.documentSystemId = systemId;
    this.parseErrorList = new ParseErrorListImpl();
    ToolParser parser = new ToolParser();
    for (int k = 0; k < PARSING_FEATURES.length; k++) {
      String[] parsingFeature = PARSING_FEATURES[k];
      String featureId = parsingFeature[PARSING_ID];
      int featureValue = options.getThreeStateOptionValue(featureId);
      if (featureValue != ThreeStateOption.UNDEFINED) {
        String featureName = parsingFeature[PARSING_FEATURE];
        parser.setFeature("http://" + featureName,
          featureValue == ThreeStateOption.TRUE);
      }
    }
    parser.setDocumentParsingListener((DocumentParsingListener)parseErrorList);
    parser.setErrorHandler((ErrorHandler)parseErrorList);
    toolEntityResolver.setParseErrorList(parseErrorList);
    parser.setXMLEntityResolver(toolEntityResolver);
    parser.parse(toolEntityResolver.resolveEntity(null, systemId));
    this.document = parser.getDocument();
    if (this.document == null)
      return;
    this.processToolProcessingInstructions();
    this.xMLVersion = parser.getXMLVersion();
    this.encoding = parser.getEncoding();
    String standalone =  parser.getStandalone();
    this.isStandalone = standalone != null && standalone.equals("yes");
    this.isThereDoctypeDecl = parser.isThereDoctypeDecl();
    this.originalDTD = parser.getDTD();
    try {
      parser.reset();
    }
    catch (Exception exception) {
    }
    parser = null;
    this.idManager = new IdManagerImpl(options);
    this.presseFragment = new PresseFragmentImpl(document);
    this.variableNodeList = new VariableNodeListImpl(false);
    this.variableNodeStack = new VariableNodeListImpl(true);
    this.nodeIntervalForMove = new NodeIntervalImpl();
    this.documentManager = new DocumentManagerImpl(this.idManager);
    this.documentManager.setTreeViewDocumentListener(
        this.treeViewDocumentListener);
    this.documentManager.addNodeChangeListener(variableNodeList);
    this.documentManager.addNodeChangeListener(variableNodeStack);
    this.setDTD(originalDTD);
  }

  private void processToolProcessingInstructions() {
    Node node = this.document.getFirstChild();
    while (node != null) {
      if (node.getNodeType() == Node.PROCESSING_INSTRUCTION_NODE
        && ((ProcessingInstruction)node).getTarget().equalsIgnoreCase(
        Constants.TOOL)) {
        StringTokenizer st = new StringTokenizer(node.getNodeValue());
        while (st.hasMoreTokens()) {
          String token = st.nextToken();
          if (token.equalsIgnoreCase("ReadOnly"))
            isReadOnly = true;
        }
      }
      node = node.getNextSibling();
    }
  }

  public String getDocumentSystemId() {
    return documentSystemId;
  }

  public void setDocumentSystemId(String documentSystemId) {
    if (this.documentSystemId == null)
      return;
    if (documentSystemId == null)
      throw new IllegalArgumentException();
    this.documentSystemId = documentSystemId;
  }

  public Document getDocument() {
    return this.document;
  }

  public ParseErrorList getParseErrorList() {
    return this.parseErrorList;
  }

  public String getXMLVersion() {
    return this.xMLVersion;
  }

  public void setXMLVersion(String xMLVersion) {
    if (documentManager == null)
      return;
    if (xMLVersion == null ? this.xMLVersion != null :
      !xMLVersion.equals(this.xMLVersion))
      documentManager.setDocumentModified(true);
    this.xMLVersion = xMLVersion;
  }

  public String getEncoding() {
    return this.encoding;
  }

  public void setEncoding(String encoding) {
    if (documentManager == null)
      return;
    if (encoding == null ? this.encoding != null :
      !encoding.equals(this.encoding))
      documentManager.setDocumentModified(true);
    this.encoding = encoding;
  }

  public boolean isStandalone() {
    return this.isStandalone;
  }

  public void setStandalone(boolean isStandalone) {
    if (documentManager == null)
      return;
    if (isStandalone != this.isStandalone)
      documentManager.setDocumentModified(true);
    this.isStandalone = isStandalone;
  }

  public boolean isThereDoctypeDecl() {
    return this.isThereDoctypeDecl;
  }

  public DTD getOriginalDTD() {
    return this.originalDTD;
  }

  public DTD getDTD() {
    return this.usedDTD;
  }

  public void setDTD(DTD dTD) throws IdException {
    if (this.document == null)
      return;
    if (dTD == null)
      throw new IllegalArgumentException();
    Element documentElement = this.document.getDocumentElement();
    if (this.schemaElementManager != null)
      this.schemaElementManager.notifyElementRemove(documentElement);
    this.usedDTD = dTD;
    sortDTD(usedDTD);
    DTDElementManager dTDElementManager =
        new DTDElementManager(idManager, usedDTD);
    this.schemaElementManager = (AbstractElementManager)dTDElementManager;
    this.documentManager.setAttributesContentAccess(
        (AttributesContentAccess)dTDElementManager);
    this.documentManager.setAttributeChangeListener(
        (AttributeChangeListener)dTDElementManager);
    this.documentManager.setElementOperationListener(
      (ElementOperationListener)dTDElementManager);
    this.documentUtil = new DTDDocumentUtil(
        (ElementDefinitionMapper)this.schemaElementManager);
    this.schemaType = SCHEMATYPE_DTD;
    this.schemaElementManager.notifyElementInsert(documentElement);
  }

  private static final void sortDTD(DTD dTD) {
    List elementNameList = dTD.getElementNameList();
    Collections.sort(elementNameList);
    for (int k = 0; k < elementNameList.size(); k++) {
      AttributeDefinitionList attributeDefinitionList =
        dTD.getAttributeDefinitionList((String)elementNameList.get(k));
      if (attributeDefinitionList != null)
        attributeDefinitionList.sort();
    }
    Collections.sort(dTD.getEntityNameList());
    Collections.sort(dTD.getNotationNameList());
  }

  public Schema getSchema() {
    return schema;
  }

  public void setSchema(Schema schema) throws IdException {
    if (this.document == null)
      return;
    if (schema == null)
      throw new IllegalArgumentException();
    Element documentElement = this.document.getDocumentElement();
    if (this.schemaElementManager != null)
      this.schemaElementManager.notifyElementRemove(documentElement);
    this.schema = schema;
    RNGElementManager rNGElementManager =
        new RNGElementManager(idManager, schema);
    this.schemaElementManager = (AbstractElementManager)rNGElementManager;
    this.documentManager.setAttributesContentAccess(
        (AttributesContentAccess)rNGElementManager);
    this.documentManager.setAttributeChangeListener(
      (AttributeChangeListener)rNGElementManager);
    this.documentManager.setElementOperationListener(
      (ElementOperationListener)rNGElementManager);
    this.documentUtil = new RNGDocumentUtil(
        (ElementDefinitionMapper)this.schemaElementManager);
    this.schemaType = SCHEMATYPE_RNG;
    this.schemaElementManager.notifyElementInsert(documentElement);
  }

  public int getSchemaType() {
    return this.schemaType;
  }

  public ElementDefinitionMapper getElementDefinitionMapper() {
    return (ElementDefinitionMapper)this.schemaElementManager;
  }

  public DocumentUtil getDocumentUtil() {
    return this.documentUtil;
  }

  public String getDocumentTypeName() {
    return this.document != null ?
      this.document.getDocumentElement().getNodeName() : null;
  }

  public String getDocumentTypeAdditionalKey() {
    return this.documentTypeAdditionalKey;
  }

  public void setDocumentTypeAdditionalKey(String documentTypeAdditionalKey) {
    if (this.document == null || this.schemaElementManager == null)
      return;
    this.documentTypeAdditionalKey = documentTypeAdditionalKey;
    ((ElementDefinitionMapper)this.schemaElementManager).removeElementFromCache(
        this.document.getDocumentElement());
  }

  public DocumentTypeInfo getDocumentTypeInfo() {
    return this.document != null ?
        DocumentTypeInfoUtils.getDocumentTypeInfoFromList(
        options.getOptionList(Options.DOCUMENTTYPE),
        getDocumentTypeName(), this.documentTypeAdditionalKey) : null;
  }

  public boolean isReadOnly() {
    return this.isReadOnly;
  }

  public void setReadOnly(boolean isReadOnly) {
    this.isReadOnly = isReadOnly;
  }

  public boolean isLazy() {
    return this.isLazy;
  }

  public void setLazy(boolean isLazy) {
    this.isLazy = isLazy;
  }

  public NodeLinker getNodeLinker() {
    return this.document != null ? this.nodeLinker : null;
  }

  public ElementKeyProvider getElementKeyProvider() {
    return this.elementKeyProvider;
  }

  public IdManager getIdManager() {
    return idManager;
  }

  public DocumentManager getDocumentManager() {
    return this.documentManager;
  }

  public PresseFragment getPresseFragment() {
    return presseFragment;
  }

  public VariableNodeList getNodeList(int nodeListType) {
    switch (nodeListType) {
      case NODELIST_NODELIST:
        return this.variableNodeList;
      case NODELIST_NODESTACK:
        return this.variableNodeStack;
      default:
        throw new IllegalArgumentException();
    }
  }

  public NodeInterval getNodeIntervalForMove() {
    return this.nodeIntervalForMove;
  }

  public int printDocument(PrintWriter out, boolean haveElementsToBeIndented)
      throws IOException {
    if (document == null)
      throw new RuntimeException("No document");
    DocumentTypeInfo documentTypeInfo = this.getDocumentTypeInfo();
    DTDInfo dTDInfo =
      documentTypeInfo != null ? documentTypeInfo.getDTDInfo() : null;
    DocumentType doctype = this.document.getDoctype();
    boolean isDTDIgnored =
      this.options.getThreeStateOptionValue(Options.DOCTYPEIFDTD)
        == ThreeStateOption.FALSE;
    DocumentWriter documentWriter = new DocumentWriter(out);
    documentWriter.setShortcutUsedForEmptyElement(
        documentTypeInfo.isShortcutUsedForEmptyElement());
    documentWriter.setLineFeedAtStartTagEnd(
        options.getThreeStateOptionValue(Options.ADDLINEFEEDATSTARTTAGEND)
        == ThreeStateOption.TRUE);
    documentWriter.setElementsToBeIndented(haveElementsToBeIndented);
    return documentWriter.printDocument(
        document, isDTDIgnored ? (DTD)null : (DTD)originalDTD,
        dTDInfo != null ? dTDInfo.getDTDPublicId() : doctype.getPublicId(),
        dTDInfo != null ?
        dTDInfo.getDocumentRelativeDTDSystemId() : doctype.getSystemId(),
        xMLVersion, encoding, isStandalone, isThereDoctypeDecl);
  }

  public int saveDocument(boolean haveElementsToBeIndented) throws IOException {
    if (this.document == null)
      throw new RuntimeException("No document");
    String documentPath =
      FileUtils.fromSystemIdToFilePathName(documentSystemId);
    if (documentPath == null)
      throw new IOException("Cannot save " + '"' + documentSystemId + '"');
    if (this.encoding == null || MIME2Java.convert(this.encoding) == null)
      this.encoding = Constants.DEFAULT_ENCODING;
    PrintWriter out =
      new PrintWriter(new OutputStreamWriter(new FileOutputStream(documentPath),
      MIME2Java.convert(encoding)));
    int returnCode = printDocument(out, haveElementsToBeIndented);
    out.close();
    this.documentManager.setDocumentModified(false);
    return returnCode;
  }
}
