/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd;

import java.util.List;
import java.io.PrintWriter;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.ExternalDefinition;
import org.xmloperator.framework.document.dtd.EntityDefinition;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.document.dtd.SpecNodeUtils;

public abstract class DTDWriter {

  public static void printDTD(PrintWriter out, DTD dTD) {
    if (out == null || dTD == null)
      throw new IllegalArgumentException();
    {
      List notationNameList = dTD.getNotationNameList();
      for (int k = 0; k < notationNameList.size(); k++)
        out.println(notationToString(dTD, (String)notationNameList.get(k)));
    }
    {
      List entityNameList = dTD.getEntityNameList();
      for (int k = 0; k < entityNameList.size(); k++)
        out.println(entityDefToString(dTD, (String)entityNameList.get(k)));
    }
    {
      List elementNameList = dTD.getElementNameList();
      for (int k = 0; k < elementNameList.size(); k++) {
        String elementName = (String)elementNameList.get(k);
        printElementDecl(out, dTD, elementName);
        printAttrListDecl(out, dTD, elementName);
      }
    }
  }

  public static String notationToString(DTD dTD, String notationName) {
    StringBuffer s = new StringBuffer();
    s.append("<!NOTATION ");
    s.append(notationName);
    s.append(' ');
    ExternalDefinition notationDef = dTD.getNotationDefinition(notationName);
    s.append(externalDefinitionToString(
      notationDef.getPublicId(), notationDef.getSystemId()));
    s.append('>');
    return s.toString();
  }

  public static String externalDefinitionToString(
    String publicId, String systemId) {
    StringBuffer s = new StringBuffer();
    if (publicId != null) {
      s.append("PUBLIC");
      s.append(' ');
      s.append('"');
      s.append(publicId);
      s.append('"');
    }
    else
      s.append("SYSTEM");
    if (systemId != null) {
      s.append(' ');
      s.append('"');
      s.append(systemId);
      s.append('"');
    }
    return s.toString();
  }

  public static String entityDefToString(DTD dTD, String entityName) {
    StringBuffer s = new StringBuffer();
    s.append("<!ENTITY ");
    s.append(entityName);
    s.append(' ');
    EntityDefinition entityDef = dTD.getEntityDefinition(entityName);
    int entityType = entityDef.getEntityType();
    if (entityType == EntityDefinition.INTERNAL_ENTITY) {
      s.append('"');
      s.append(exportValue(entityDef.getEntityValue()));
      s.append('"');
    }
    else {
      s.append(externalDefinitionToString(
        entityDef.getPublicId(), entityDef.getSystemId()));
      if (entityType == EntityDefinition.UNPARSED_ENTITY) {
        s.append(' ');
        s.append("NDATA");
        s.append(' ');
        s.append(entityDef.getNotationName());
      }
    }
    s.append('>');
    return s.toString();
  }

  private static String exportValue(String value) {
    StringBuffer s = new StringBuffer();
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      if (ch == '"')
        s.append("&#34;");
      else if (ch == '&')
        s.append("&#38;");
      else if (ch == '<')
        s.append("&#60;");
      else
        s.append(ch);
    }
    return s.toString();
  }

  private static void printElementDecl(PrintWriter out, DTD dTD,
    String elementName) {
    ElementSpecification elementSpec = dTD.getElementSpecification(elementName);
    if (elementSpec == null)
      return;
    out.print("<!ELEMENT ");
    out.print(elementName);
    out.print(' ');
    SpecificationNode specNode = elementSpec.getSpecNode();
    out.print(specNode != null ?
      SpecNodeUtils.specNodeToString(specNode) : elementSpec.getSpecType());
    out.println('>');
  }

  private static void printAttrListDecl(PrintWriter out, DTD dTD,
    String elementName) {
    AttributeDefinitionList attributeDefinitionList =
      dTD.getAttributeDefinitionList(elementName);
    if (attributeDefinitionList == null)
      return;
    out.print("<!ATTLIST ");
    out.print(elementName);
    out.println();
    for (int k = 0; k < attributeDefinitionList.size(); k++) {
      out.print("  ");
      out.println(attributeDefinitionToString(
        attributeDefinitionList.get(k)));
    }
    out.println('>');
  }

  public static String attributeDefinitionToString(
    AttributeDefinition attributeDefinition) {
    StringBuffer s = new StringBuffer();
    s.append(attributeDefinition.getAttrName());
    String attrType = attributeDefinition.getAttrType();
    boolean isNotation = attrType.equals(AttributeDefinition.NOTATION);
    boolean isEnumeration = attrType.equals(AttributeDefinition.ENUMERATION);
    if (!isEnumeration) {
      s.append(' ');
      s.append(attrType);
    }
    if (isNotation | isEnumeration) {
      s.append(' ');
      s.append(enumerationToString(attributeDefinition.getEnumeration()));
    }
    String attrDefaultType = attributeDefinition.getAttrDefaultType();
    boolean isFixedValue = attrDefaultType.equals(AttributeDefinition.FIXED);
    boolean isDefaultValue =
      attrDefaultType.equals(AttributeDefinition.DEFAULT);
    if (!isDefaultValue) {
      s.append(' ');
      s.append(attrDefaultType);
    }
    if (isFixedValue | isDefaultValue) {
      s.append(' ');
      s.append('"');
      s.append(attributeDefinition.getAttrDefaultValue());
      s.append('"');
    }
    return s.toString();
  }

  private static final String enumerationToString(String[] enumeration) {
    StringBuffer sb = new StringBuffer();
    sb.append('(');
    for (int k = 0; k < enumeration.length; k++) {
      if (k > 0)
        sb.append('|');
      sb.append(enumeration[k]);
    }
    sb.append(')');
    return sb.toString();
  }
}
