/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.swing.JOptionPane;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.ExternalDefinition;
import org.xmloperator.framework.document.dtd.EntityDefinition;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.document.dtd.impl.EntityDefinitionImpl;

public final class DTDImpl implements DTD {
  private static final int PREDEFINED_ENTITY_NAME = 0;
  private static final int PREDEFINED_ENTITY_DEFINITION = 1;
  private static final Object[][] predefinedEntities = {
    {"lt", EntityDefinitionImpl.LT},
    {"gt", EntityDefinitionImpl.GT},
    {"amp", EntityDefinitionImpl.AMP},
    {"apos", EntityDefinitionImpl.APOS},
    {"quot", EntityDefinitionImpl.QUOT}};
  private final ArrayList elementNameList = new ArrayList(64);
  private final Hashtable fromElementNameToElementSpecification =
    new Hashtable();
  private final Hashtable fromElementNameToAttrDefList = new Hashtable();
  private final ArrayList entityNameList = new ArrayList(16);
  private final Hashtable fromEntityNameToEntityDefinition = new Hashtable();
  private final ArrayList notationNameList = new ArrayList();
  private final Hashtable fromNotationNameToNotationDefinition =
    new Hashtable();
  private final ArrayList namespaceList = new ArrayList();
  private final ArrayList namespacePrefixList = new ArrayList();
  private boolean isThereAPrefixAttachedToDifferentNamespaces = false;
  private final Hashtable fromNamespacePrefixToNamespaceURI = new Hashtable();

  public DTDImpl() {
  }

  public void addPredefinedEntities() {
    for (int k = 0; k < predefinedEntities.length; k++) {
      Object[] predefinedEntity = predefinedEntities[k];
      String entityName = (String)predefinedEntity[PREDEFINED_ENTITY_NAME];
      if (!entityNameList.contains(entityName)) {
        EntityDefinition entityDefinition =
          (EntityDefinition)predefinedEntity[PREDEFINED_ENTITY_DEFINITION];
        entityNameList.add(entityName);
        fromEntityNameToEntityDefinition.put(entityName, entityDefinition);
      }
    }
  }

  public Hashtable fromElementNameToElementSpecification() {
    return fromElementNameToElementSpecification;
  }

  public Hashtable fromElementNameToAttrDefList() {
    return fromElementNameToAttrDefList;
  }

  public Hashtable fromEntityNameToEntityDefinition() {
    return fromEntityNameToEntityDefinition;
  }

  public Hashtable fromNotationNameToNotationDefinition() {
    return fromNotationNameToNotationDefinition;
  }

  public String addNamespace(String uRIReference) {
    if (uRIReference == null)
      throw new IllegalArgumentException();
    int index = namespaceList.indexOf(uRIReference);
    if (index >= 0)
      uRIReference = (String)namespaceList.get(index);
    else
      namespaceList.add(uRIReference);
    return uRIReference;
  }

  public String addNamespacePrefix(String prefix) {
    if (prefix == null)
      throw new IllegalArgumentException();
    if (prefix.equals("xmlns"))
      return null;
    int index = namespacePrefixList.indexOf(prefix);
    if (index >= 0)
      prefix = (String)namespacePrefixList.get(index);
    else
      namespacePrefixList.add(prefix);
    return prefix;
  }

  public void addNamespacePrefix(String prefix, String uRIReference) {
    if (prefix == null || uRIReference == null)
      throw new IllegalArgumentException();
    uRIReference = addNamespace(uRIReference);
    prefix = addNamespacePrefix(prefix);
    if (prefix == null)
      return;
    String oldURIReference =
      (String)fromNamespacePrefixToNamespaceURI.get(prefix);
    if (oldURIReference == null)
      fromNamespacePrefixToNamespaceURI.put(prefix, uRIReference);
    else
      isThereAPrefixAttachedToDifferentNamespaces |=
        !uRIReference.equals(oldURIReference);
  }

  /////////////////////
  // DTD implementation
  /////////////////////

  public List getElementNameList() {
    return elementNameList;
  }

  public ElementSpecification getElementSpecification(String elementName) {
    return (ElementSpecification)fromElementNameToElementSpecification.
      get(elementName);
  }

  public AttributeDefinitionList getAttributeDefinitionList(String elementName)
    {
    return (AttributeDefinitionList)fromElementNameToAttrDefList.
      get(elementName);
  }
  public List getEntityNameList() {
    return entityNameList;
  }

  public EntityDefinition getEntityDefinition(String entityName) {
    return (EntityDefinition)fromEntityNameToEntityDefinition.get(entityName);
  }

  public List getNotationNameList() {
    return notationNameList;
  }

  public ExternalDefinition getNotationDefinition(String notationName) {
    return (ExternalDefinition)fromNotationNameToNotationDefinition.
      get(notationName);
  }

  public List getNamespaceList() {
    return namespaceList;
  }

  public List getNamespacePrefixList() {
    return namespacePrefixList;
  }

  public boolean isThereAPrefixAttachedToDifferentNamespaces() {
    return isThereAPrefixAttachedToDifferentNamespaces;
  }

  public String getNamespaceUri(String namespacePrefix) {
    return (String)fromNamespacePrefixToNamespaceURI.get(namespacePrefix);
  }
}
