/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.impl;

import org.xmloperator.framework.document.dtd.SpecificationNode;

public final class SpecificationNodeImpl implements SpecificationNode {
  public static final SpecificationNode PCDATA_SPEC_NODE =
    new SpecificationNodeImpl();
  private char occurrence;
  private boolean isSequence;
  private String elementName;
  private SpecificationNode[] specNodes;

  // #PCDATA leaf in a MIXED content
  public SpecificationNodeImpl() {
    this.occurrence = '.';
    this.isSequence = false;
    this.elementName = null;
    this.specNodes = null;
  }

  // Element leaf
  public SpecificationNodeImpl(String elementName) {
    if (elementName == null)
      throw new IllegalArgumentException();
    this.occurrence = '.';
    this.isSequence = false;
    this.elementName = elementName;
    this.specNodes = null;
  }

  // Node
  public SpecificationNodeImpl(boolean isSequence,
    SpecificationNode[] specNodes) {
    if (specNodes == null || specNodes.length < 1)
      throw new IllegalArgumentException();
    this.occurrence = '.';
    this.isSequence = isSequence;
    this.elementName = null;
    this.specNodes = specNodes;
  }

  ///////////////////////////////////
  // SpecificationNode implementation
  ///////////////////////////////////

  public char getOccurrence() {
    return occurrence;
  }

  public void setOccurrence(char occurrence) {
    if (occurrence != '?' && occurrence != '+' && occurrence != '*')
      throw new IllegalArgumentException();
    this.occurrence = occurrence;
  }

  public boolean isSequence() {
    return isSequence;
  }

  public String getElementName() {
    return elementName;
  }

  public int getChildCount() {
    return specNodes != null ? specNodes.length : 0;
  }

  public SpecificationNode getChild(int index) {
    return specNodes[index];
  }
}
