/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.elementKey;

import org.xmloperator.framework.document.elementKey.ElementKey;

public final class ElementKeyImpl implements ElementKey {
  private int keyType;
  private String attributeOrChildElementName;

  public static final ElementKeyImpl ELEMENTKEY_TEXTIFLEAF =
      new ElementKeyImpl(KEYTYPE_TEXTIFLEAF);
  public static final ElementKeyImpl ELEMENTKEY_ALLATTRIBUTES =
      new ElementKeyImpl(KEYTYPE_ALLATTRIBUTES);
  public static final ElementKeyImpl ELEMENTKEY_ALLSINGLEELEMENTS =
      new ElementKeyImpl(KEYTYPE_ALLSINGLEELEMENTS);
  public static final ElementKeyImpl ELEMENTKEY_ALLATTRIBUTESANDSINGLEELEMENTS =
      new ElementKeyImpl(KEYTYPE_ALLATTRIBUTESANDSINGLEELEMENTS);

  public ElementKeyImpl(int keyType) {
    this(keyType, null);
  }

  public ElementKeyImpl(int keyType, String attributeOrChildElementName) {
    switch (keyType) {
      case KEYTYPE_TEXTIFLEAF:
      case KEYTYPE_ALLATTRIBUTES:
      case KEYTYPE_ALLSINGLEELEMENTS:
      case KEYTYPE_ALLATTRIBUTESANDSINGLEELEMENTS:
        if (attributeOrChildElementName != null)
          throw new IllegalArgumentException();
        break;
      case KEYTYPE_ONEATTRIBUTE:
      case KEYTYPE_ONESINGLEELEMENT:
        if (attributeOrChildElementName == null)
          throw new IllegalArgumentException();
        break;
      default:
        throw new IllegalArgumentException();
    }
    this.keyType = keyType;
    this.attributeOrChildElementName = attributeOrChildElementName;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    switch (this.keyType) {
      case KEYTYPE_TEXTIFLEAF:
        sb.append("TextIfLeaf");
        break;
      case KEYTYPE_ALLATTRIBUTES:
        sb.append("AllAttributes");
        break;
      case KEYTYPE_ALLSINGLEELEMENTS:
        sb.append("AllSingleElements");
        break;
      case KEYTYPE_ALLATTRIBUTESANDSINGLEELEMENTS:
        sb.append("AllAttributesAndSingleElements");
        break;
      case KEYTYPE_ONEATTRIBUTE:
        sb.append("Attribute(");
        sb.append(this.attributeOrChildElementName);
        sb.append(")");
        break;
      case KEYTYPE_ONESINGLEELEMENT:
        sb.append("SingleElement(");
        sb.append(this.attributeOrChildElementName);
        sb.append(")");
        break;
      default:
        sb.append("??");
    }
    return sb.toString();
  }

  ////////////////////////////
  // ElementKey implementation
  ////////////////////////////

  public int getKeyType() {
    return this.keyType;
  }

  public String getAttributeOrSingleElementName() {
    return this.attributeOrChildElementName;
  }
}
