/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.elementKey;

import java.util.Map;
import java.util.HashMap;

import org.w3c.dom.Document;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.elementKey.ElementKey;
import org.xmloperator.framework.document.elementKey.ElementKeyProvider;
import org.xmloperator.framework.document.DocumentAccess;
import org.xmloperator.document.elementKey.ElementKeyImpl;
import org.xmloperator.document.elementKey.ElementKeyStat;

public final class ElementKeyProviderImpl implements ElementKeyProvider {
  private DocumentAccess documentAccess;
  private Options options;
  private final Map fromElementNameToElementKey = new HashMap();

  public ElementKeyProviderImpl(
      DocumentAccess documentAccess, Options options) {
    this.documentAccess = documentAccess;
    this.options = options;
  }

  public void clear() {
    this.fromElementNameToElementKey.clear();
  }

  ////////////////////////////////////
  // ElementKeyProvider implementation
  ////////////////////////////////////

  public ElementKey getElementKey(String elementName) {
    ElementKey elementKey =
        (ElementKey)this.fromElementNameToElementKey.get(elementName);
    if (elementKey == null && this.documentAccess != null
        && this.options.getThreeStateOptionValue(Options.AUTOMATICLABELLING)
        == ThreeStateOption.TRUE) {
      Document document = documentAccess.getDocument();
      if (document != null) {
        elementKey = ElementKeyStat.getElementKey(document, elementName);
        if (elementKey != null)
          this.fromElementNameToElementKey.put(elementName, elementKey);
      }
    }
    return elementKey;
  }

  public void setElementKey(String elementName, ElementKey elementKey) {
    if (elementName == null || elementName.length() == 0)
      throw new IllegalArgumentException();
    if (elementKey == null)
      this.fromElementNameToElementKey.remove(elementName);
    else
      this.fromElementNameToElementKey.put(elementName, elementKey);
  }
}
