/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.Iterator;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;
import java.util.List;
import java.util.HashSet;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.util.dom.ElementIndexer;
import org.xmloperator.util.dom.ElementNamesUtils;
import org.xmloperator.util.dom.ElementSpecListUtils;
import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.schema.ElementContent;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.schema.
    NotConformantToContentModelException;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.document.dtd.SpecNodeUtils;
import org.xmloperator.document.dtd.ElementContentImpl;
import org.xmloperator.document.dtd.DTDUtils;
import org.xmloperator.document.manage.AbstractDocumentUtil;

public final class DTDDocumentUtil extends AbstractDocumentUtil {

  public DTDDocumentUtil(ElementDefinitionMapper elementDefinitionMapper) {
    super(elementDefinitionMapper);
  }

  private static final String[] toStringArray(Object[] objects) {
    String[] strings = new String[objects.length];
    System.arraycopy(objects, 0, strings, 0, strings.length);
    return strings;
  }

  public static final String[] entityReferenceNames(final DTD dTD) {
    if (dTD == null)
      return null;
    List entityNameList = dTD.getEntityNameList();
    if (entityNameList.size() == 0)
      return null;
    String[] entityNames = new String[entityNameList.size()];
    entityNameList.toArray(entityNames);
    return entityNames;
  }

  //////////////////////////////
  // DocumentUtil implementation
  //////////////////////////////

  public String contentModelToString(Element parentElement,
    boolean isSummarized) {
    return elementFromContentModelToString(parentElement, null, isSummarized);
  }

  public String elementFromContentModelToString(
    final Element element, boolean isSummarized) {
    Node parentNode = element.getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    return elementFromContentModelToString((Element)parentNode, element,
        isSummarized);
  }

  public String elementFromContentModelToString(
    final Element parentElement, final Element focusedElement,
    boolean isSummarized) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    String parentElementName = parentElement.getNodeName();
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return null;
    String specType = parentElementSpec.getSpecType();
    String attributeContentModel =
        attributeContentModelToString(parentElementName);
    if (specType.equals(ElementSpecification.EMPTY))
      return attributeContentModel.length() != 0 ?
        attributeContentModel : specType;
    SpecificationNode parentSpecNode = parentElementSpec.getSpecNode();
    String elementContentModel = specType;
    if (parentSpecNode != null) {
      if (focusedElement != null)
        try {
        ElementContent elementContent =
            elementDefinitionMapper.getElementContent(parentElement);
        if (elementContent == null)
          return null;
        int elementNameIndex =
            ElementIndexer.getFlatIndex(focusedElement);
        elementContentModel = elementContent.elementFromContentModelToString(
            elementNameIndex, isSummarized, parentElement);
      }
      catch (NotConformantToContentModelException exception) {
        return null;
      }
      else {
        elementContentModel =
            SpecNodeUtils.specNodeToString(parentSpecNode, null, isSummarized);
      }
    }
    return attributeContentModel.length() != 0 ?
        attributeContentModel + " , " + elementContentModel :
        elementContentModel;
  }

  private String attributeContentModelToString(String elementName) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    AttributeDefinitionList attributeDefinitionList =
        dTD.getAttributeDefinitionList(elementName);
    if (attributeDefinitionList == null)
      return "";
    StringBuffer sb = new StringBuffer();
    for (int k = 0; k < attributeDefinitionList.size(); k++) {
      AttributeDefinition attributeDefinition = attributeDefinitionList.get(k);
      String attributeName = attributeDefinition.getAttrName();
      if (attributeName.equals("xmlns") || attributeName.startsWith("xmlns:"))
        continue;
      if (k != 0)
        sb.append(" , ");
      sb.append('@');
      sb.append(attributeName);
      String defaultType = attributeDefinition.getAttrDefaultType();
      if (!defaultType.equals(AttributeDefinition.REQUIRED)
          && !defaultType.equals(AttributeDefinition.FIXED))
        sb.append('?');
    }
    return sb.toString();
  }

  public boolean areNodesMovable(final Node[] nodes, final int moveType) {
    switch (moveType) {
      case EditControl.MOVE_PREVIOUS:
      case EditControl.MOVE_NEXT:
        return areNodesMovableWithParentUnchanged(nodes, moveType);
      case EditControl.MOVE_CHILD:
      case EditControl.MOVE_PARENT: {
        if (!areNodesRemovable(nodes))
          return false;
        Node parentNode = moveType == EditControl.MOVE_PARENT ?
          nodes[0].getParentNode().getParentNode() :
          nodes[0].getPreviousSibling();
        if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
          return false;
        Node nextSibling = moveType == EditControl.MOVE_PARENT ?
          nodes[0].getParentNode().getNextSibling() : null;
        return areElementNamesInsertable(ElementNamesUtils.elementNames(nodes),
          (Element)parentNode, nextSibling);
      }
      default:
        throw new IllegalArgumentException();
    }
  }

  private boolean areNodesMovableWithParentUnchanged(
    final Node[] nodes, final int moveType) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE
      || !ElementIndexer.hasSignificantNodes(nodes))
      return true;
    Node movedNode = moveType == EditControl.MOVE_PREVIOUS ?
      nodes[0].getPreviousSibling() : nodes[nodes.length - 1].getNextSibling();
    if (movedNode == null)
      return false;
    if (!ElementIndexer.isSignificant(movedNode))
      return true;
    // Return true if the content is not valid BEFORE move
    if (!elementDefinitionMapper.isElementContentValid((Element)parentNode))
      return true;
    // Return false if the content is not valid AFTER move
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    SpecificationNode specNode =
      DTDUtils.getSpecificationNode(dTD, parentNode.getNodeName());
    if (specNode == null)
      return true;
    String[] elementNames =
      ElementNamesUtils.elementNames((Element)parentNode);
    int firstElementNameIndex = ElementIndexer.getFlatIndex(nodes[0]);
    int lastElementNameIndex = ElementIndexer.getLastFlatIndex(nodes);
    if (moveType == EditControl.MOVE_PREVIOUS)
      ElementSpecListUtils.moveElementSpecsPrevious(elementNames,
        firstElementNameIndex, lastElementNameIndex);
    else
      ElementSpecListUtils.moveElementSpecsNext(elementNames,
        firstElementNameIndex, lastElementNameIndex);
    return ElementContentImpl.isContentValid(specNode, elementNames);
  }

  public boolean areNodesRemovable(final Node[] nodes) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE
      || !ElementIndexer.hasSignificantNodes(nodes))
      return true;
    // Return true if the content is not valid BEFORE remove
    if (!elementDefinitionMapper.isElementContentValid((Element)parentNode))
      return true;
    // Return false if the content is not valid AFTER remove
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    SpecificationNode specNode =
      DTDUtils.getSpecificationNode(dTD, parentNode.getNodeName());
    if (specNode == null)
      return true;
    String[] elementNames =
      ElementNamesUtils.elementNames((Element)parentNode);
    int firstElementNameIndex = ElementIndexer.getFlatIndex(nodes[0]);
    int lastElementNameIndex = ElementIndexer.getLastFlatIndex(nodes);
    elementNames =
      toStringArray(ElementSpecListUtils.removeElementSpecsAt(elementNames,
      firstElementNameIndex, lastElementNameIndex - firstElementNameIndex + 1));
    return ElementContentImpl.isContentValid(specNode, elementNames);
  }

  public boolean canElementHaveChilds(Element element) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    return DTDUtils.canHaveChilds(dTD, element.getNodeName());
  }

  public boolean canElementHaveText(Element element) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    return DTDUtils.canHaveChildText(dTD, element.getNodeName());
  }

  public boolean shouldElementHaveText(Element element) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    return DTDUtils.shouldHaveChildText(dTD, element.getNodeName());
  }

  public boolean areElementNamesInsertable(final String[] insertedElementNames,
    final Element parentElement, final Node nextSibling) {
    // Return true if the content is not valid BEFORE insert
    if (!elementDefinitionMapper.isElementContentValid(parentElement))
      return true;
    // Return false if the content is not valid AFTER insert
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    SpecificationNode specNode =
      DTDUtils.getSpecificationNode(dTD, parentElement.getNodeName());
    if (specNode == null)
      return true;
    String[] elementNames = ElementNamesUtils.elementNames(parentElement);
    int elementNameIndex = nextSibling != null ?
      ElementIndexer.getFlatIndex(nextSibling) : elementNames.length;
    elementNames = toStringArray(ElementSpecListUtils.insertElementSpecsAt(
      insertedElementNames, elementNames, elementNameIndex));
    return ElementContentImpl.isContentValid(specNode, elementNames);
  }

  public boolean isElementUnpackable(final Element element) {
    Element parentElement = (Element)element.getParentNode();
    // Return true if the content is not valid BEFORE unpack
    if (!elementDefinitionMapper.isElementContentValid(parentElement))
      return true;
    // Return false if the content is not valid AFTER unpack
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    SpecificationNode specNode =
      DTDUtils.getSpecificationNode(dTD, parentElement.getNodeName());
    if (specNode == null)
      return true;
    String[] elementNames = ElementNamesUtils.elementNames(parentElement);
    int elementNameIndex = ElementIndexer.getFlatIndex(element);
    String[] newElementNames =
      toStringArray(ElementSpecListUtils.removeElementSpecsAt(
      elementNames, elementNameIndex, 1));
    String[] insertedElementNames = ElementNamesUtils.elementNames(element);
    newElementNames = toStringArray(ElementSpecListUtils.insertElementSpecsAt(
      insertedElementNames, newElementNames, elementNameIndex));
    return ElementContentImpl.isContentValid(specNode, newElementNames);
  }

  public boolean isElementNameReplacable(Element element,
    String newElementName) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    String[] childElementNames = ElementNamesUtils.elementNames(element);
    return DTDUtils.canElementNameBeParentOf(dTD, newElementName,
      childElementNames);
  }

  public Collection elementNamesForAppend(boolean isLazy,
    Element parentElement) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    Collection elementNames = DTDUtils.elementNamesForAppendChild(dTD,
      parentElement.getNodeName());
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  public Collection elementNamesForInsert(boolean isLazy, final Element element,
    final boolean isInsertedAfter) {
    Collection elementNames = elementNamesForInsert(element, isInsertedAfter);
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementNamesForInsert(final Element element,
    final boolean isInsertedAfter) {
    Node parentNode = element.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      throw new IllegalArgumentException();
    ElementContent elementContent;
    try {
      elementContent =
        elementDefinitionMapper.getElementContent((Element)parentNode);
      if (elementContent == null) // ANY
        return Collections.singleton(element.getNodeName());
    }
    catch (NotConformantToContentModelException exception) {
      return Collections.singleton(element.getNodeName());
    }
    int elementNameIndex = ElementIndexer.getFlatIndex(element);
    if (isInsertedAfter)
      elementNameIndex++;
    Collection elementNameSet =
      elementContent.elementDefinitionsInsertableBefore(elementNameIndex);
    return !elementNameSet.isEmpty() ? elementNameSet : null;
  }

  public Collection elementNamesForReplace(boolean isLazy, final Node node) {
    Collection elementNames = null;
    Node parentNode = node.getParentNode();
    if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE) {
      if (elementDefinitionMapper.isElementContentValid((Element)parentNode))
        elementNames = elementNamesForReplace(node);
      else
        isLazy = true;
    }
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementNamesForReplace(final Node node) {
    if (node.getNodeType() == Node.ELEMENT_NODE)
      return elementNamesForReplace((Element)node);
    Node parentNode = node.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    Element element = ElementBrowser.getPreviousElement(node);
    boolean isInsertedAfter = true;
    if (element == null) {
      element = ElementBrowser.getNextElement(node);
      isInsertedAfter = false;
    }
    return element != null ?
      elementNamesForInsert(element, isInsertedAfter) :
      DTDUtils.elementNamesForAppendChild(
      (DTD)elementDefinitionMapper.getSchema(), parentNode.getNodeName());
  }

  private Collection elementNamesForReplace(final Element element) {
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    Node parentNode = element.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentNode.getNodeName());
    if (parentElementSpec == null)
      return null;
    String parentSpecType = parentElementSpec.getSpecType();
    if (parentSpecType.equals(ElementSpecification.EMPTY))
      return null;
    List anyElementList = dTD.getElementNameList();
    if (parentSpecType.equals(ElementSpecification.ANY))
      return anyElementList;
    SpecificationNode specNode = parentElementSpec.getSpecNode();
    if (specNode == null)
      return null;
    String[] elementNames =
      ElementNamesUtils.elementNames((Element)parentNode);
    ElementContent elementContent;
    try {
      elementContent =
        elementDefinitionMapper.getElementContent((Element)parentNode);
    }
    catch (NotConformantToContentModelException exception) {
      return anyElementList;
    }
    int elementNameIndex = ElementIndexer.getFlatIndex(element);
    Set elementNameSet = new HashSet();
    elementContent.addElementDefinitionsForReplace(elementNameSet,
      elementNameIndex);
    elementNames =
      toStringArray(ElementSpecListUtils.removeElementSpecsAt(elementNames,
      elementNameIndex, 1));
    try {
      ElementContent subContent =
        new ElementContentImpl(specNode, elementNames);
      Set insertableElementNameSet =
        subContent.elementDefinitionsInsertableBefore(elementNameIndex);
      if (insertableElementNameSet != null)
        elementNameSet.addAll(insertableElementNameSet);
    }
    catch (NotConformantToContentModelException exception) {
    }
    return elementNameSet;
  }

  public Collection elementNamesForPack(
    final boolean isLazy, final Node[] nodes) {
    Collection elementNames = elementNamesForPackInternal(nodes);
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementNamesForPackInternal(final Node[] nodes) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    DTD dTD = (DTD)elementDefinitionMapper.getSchema();
    String parentElementName = parentNode.getNodeName();
    if (!ElementIndexer.hasSignificantChilds(parentNode))
      return DTDUtils.elementNamesForAppendChild(dTD, parentElementName);
    Collection elementNameSet =
      DTDUtils.childElementNameSet(dTD, parentElementName);
      // to be revisited ...
    return elementNameSet;
  }

  public Collection elementAndAttributeNamesForRequiredInsert(
    Element parentElement, Object[] unsatisfiedPatterns) {
    List elementAndAttributeNameList = new ArrayList();
    for (int k = 0; k < unsatisfiedPatterns.length; k++) {
      Object unsatisfiedPattern = unsatisfiedPatterns[k];
      if (!(unsatisfiedPattern instanceof String))
        throw new IllegalArgumentException();
      elementAndAttributeNameList.add(unsatisfiedPattern);
    }
    return elementAndAttributeNameList;
  }
}
