/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.parse;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.parse.ParseError;

public final class ParseErrorImpl implements ParseError {
  private int errorType;
  private String systemId;
  private int lineNumber;
  private int columnNumber;
  private String message;
  private int[] elementIndexStack;
  private String[] elementNameStack;

  public ParseErrorImpl(int[] elementIndexStack, String[] elementNameStack,
    int errorType, String systemId, int lineNumber, int columnNumber,
    String message) {
    if (elementIndexStack == null || elementNameStack == null
      || errorType < ERRORTYPE_WARNING || errorType > ERRORTYPE_FATALERROR
      || message == null)
      throw new IllegalArgumentException();
    this.errorType = errorType;
    this.systemId = systemId;
    this.lineNumber = lineNumber;
    this.columnNumber = columnNumber;
    this.message = message;
    this.elementIndexStack = elementIndexStack;
    this.elementNameStack = elementNameStack;
  }

  public void pushElement(int elementIndex, String elementName) {
    int[] indexStack = new int[elementIndexStack.length + 1];
    String[] nameStack = new String[elementIndexStack.length + 1];
    System.arraycopy(elementIndexStack, 0, indexStack, 0,
      elementIndexStack.length);
    System.arraycopy(elementNameStack, 0, nameStack, 0,
      elementNameStack.length);
    indexStack[elementIndexStack.length] = elementIndex;
    nameStack[elementNameStack.length] = elementName;
    elementIndexStack = indexStack;
    elementNameStack = nameStack;
  }

  ////////////////////////////
  // ParseError implementation
  ////////////////////////////

  public int getErrorType() {
    return errorType;
  }

  public String getSystemId() {
    return systemId;
  }

  public int getLineNumber() {
    return lineNumber;
  }

  public int getColumnNumber() {
    return columnNumber;
  }

  public String getMessage() {
    return message;
  }

  public String elementPathToString() {
    StringBuffer s = new StringBuffer();
    for (int k = 0; k < elementIndexStack.length; k++) {
      if (k != 0)
        s.append('/');
      s.append(elementIndexStack[k]);
      s.append(':');
      s.append(elementNameStack[k]);
    }
    return s.toString();
  }

  public Element elementPathToElement(Element documentElement) {
    if (documentElement == null)
      return null;
    if (elementIndexStack.length == 0)
      return null;
    if (!elementNameStack[0].equals(documentElement.getNodeName()))
      return null;
    Element element = documentElement;
    for (int n = 1; n < elementIndexStack.length; n++) {
      int expectedElementIndex = elementIndexStack[n];
      String expectedElementName = elementNameStack[n];
      int currentElementIndex = 0;
      boolean isChildFound = false;
      Node childNode = element.getFirstChild();
      while (childNode != null) {
        if (childNode.getNodeType() == Node.ELEMENT_NODE) {
          if (currentElementIndex == expectedElementIndex) {
            if (!childNode.getNodeName().equals(expectedElementName))
              return element;
            isChildFound = true;
            element = (Element)childNode;
            break;
          }
          currentElementIndex++;
        }
        childNode = childNode.getNextSibling();
      }
      if (!isChildFound)
        return element;
    }
    return element;
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    s.append(ERROR_TYPE_LABEL[errorType]);
    if (lineNumber > 0 && columnNumber > 0) {
      s.append(" in line ");
      s.append(lineNumber);
      s.append(", column ");
      s.append(columnNumber);
    }
    s.append(" : ");
    s.append(message);
    return s.toString();
  }
}
