/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.parse;

import java.util.Vector;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;

import org.xmloperator.framework.document.parse.ParseError;
import org.xmloperator.framework.document.parse.ParseErrorList;
import org.xmloperator.framework.document.parse.DocumentParsingListener;
import org.xmloperator.util.dom.IntStack;
import org.xmloperator.document.parse.ParseErrorImpl;

public final class ParseErrorListImpl
  implements ParseErrorList, ErrorHandler, DocumentParsingListener  {
  private final Vector list = new Vector();
  private int mostSeriousErrorType = ParseError.ERRORTYPE_NO;
  private final IntStack elementIndexStack = new IntStack();
  private final Vector elementNameStack = new Vector();
  private int lastEndedElementIndex = -1;
  private static final int PARSING_BEFOREDTD = 0;
  private static final int PARSING_INSIDEDTD = 1;
  private static final int PARSING_AFTERDTD = 2;
  private int parsingState = PARSING_BEFOREDTD;
  private boolean isItWaitingForStartElementToComplete = false;

  public ParseErrorListImpl() {
  }

  ////////////////////////////////
  // ParseErrorList implementation
  ////////////////////////////////

  public int size() {
    return list.size();
  }

  public ParseError get(int index) {
    return (ParseError)list.elementAt(index);
  }

  public void add(int parseErrorType,
    String systemId, int lineNumber, int columnNumber, String message) {
    int[] indexStack = new int[elementIndexStack.size()];
    elementIndexStack.copyInto(indexStack);
    String[] nameStack = new String[elementNameStack.size()];
    elementNameStack.copyInto(nameStack);
    ParseError parseError =
      new ParseErrorImpl(indexStack, nameStack,
      parseErrorType, systemId, lineNumber, columnNumber, message);
    list.addElement(parseError);
    if (parseErrorType > mostSeriousErrorType)
      mostSeriousErrorType = parseErrorType;
    isItWaitingForStartElementToComplete = (parsingState == PARSING_AFTERDTD);
  }

  public int getMostSeriousErrorType() {
    return mostSeriousErrorType;
  }

  //////////////////////////////
  // Implementation ErrorHandler
  //////////////////////////////

  public void warning(SAXParseException exception) throws SAXException {
    add(ParseError.ERRORTYPE_WARNING, exception);
  }

  public void error(SAXParseException exception) throws SAXException {
    add(ParseError.ERRORTYPE_ERROR, exception);
  }

  public void fatalError(SAXParseException exception) throws SAXException {
    add(ParseError.ERRORTYPE_FATALERROR, exception);
    throw exception;
  }

  private void add(int parseErrorType, SAXParseException exception) {
    add(parseErrorType, exception.getSystemId(), exception.getLineNumber(),
      exception.getColumnNumber(), exception.getMessage());
  }

  /////////////////////////////////////////
  // DocumentParsingListener implementation
  /////////////////////////////////////////

  public void notifyStartDTD() {
    parsingState = PARSING_INSIDEDTD;
  }

  public void notifyEndDTD() {
    parsingState = PARSING_AFTERDTD;
  }

  public void notifyStartElement(String elementName) {
    int elementIndex = lastEndedElementIndex + 1;
    lastEndedElementIndex = -1;
    elementIndexStack.push(elementIndex);
    elementNameStack.addElement(elementName);
    if (isItWaitingForStartElementToComplete) {
      ((ParseErrorImpl)list.lastElement()).
        pushElement(elementIndex, elementName);
      isItWaitingForStartElementToComplete = false;
    }
  }

  public void notifyEndElement() {
    int lastIndex = elementIndexStack.size() - 1;
    lastEndedElementIndex = elementIndexStack.get(lastIndex);
    elementIndexStack.pop();
    elementNameStack.removeElementAt(lastIndex);
    isItWaitingForStartElementToComplete = false;
  }
}
