/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import java.util.Map;
import java.util.List;
import java.util.ArrayList;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.framework.document.relaxng.Param;
import org.xmloperator.framework.document.relaxng.Data;
import org.xmloperator.document.relaxng.impl.NonEmptyPatternImpl;
import org.xmloperator.document.relaxng.impl.DataImpl;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.NonEmptyPatternHandler;
import org.xmloperator.document.relaxng.parse.ParamBuilder;
import org.xmloperator.document.relaxng.parse.ExceptPatternBuilder;

// dataPattern  ::= <data type="NCName" datatypeLibrary="anyURI">
//   param* [exceptPattern] </data>
public class DataBuilder implements NonEmptyPatternHandler {
  private static final int STATE_EXPECT_DATA = 0;
  private static final int STATE_EXPECT_PARAM_OR_EXCEPT = 1;
  private static final int STATE_INSIDE_PARAM = 2;
  private static final int STATE_INSIDE_EXCEPT = 3;
  private static final int STATE_EXPECT_END_DATA = 4;
  private static final int STATE_END = 5;

  private int state = STATE_EXPECT_DATA; // STATE_*
  private String type;
  private String datatypeLibraryURI;
  private final List paramBuilderList = new ArrayList();
  private ParamBuilder paramBuilder = null;
  private ExceptPatternBuilder exceptPatternBuilder = null;

  public static final boolean isStart(String elementName) {
    return elementName.equals("data");
  }

  public DataBuilder() {
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("data", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    switch (state) {
      case STATE_EXPECT_DATA:
        if (!name.equals("data"))
          error();
        type = atts.getValue("type");
        if (type == null)
          type = "";
        datatypeLibraryURI = atts.getValue("datatypeLibrary");
        if (datatypeLibraryURI == null)
          datatypeLibraryURI = "";
        state = STATE_EXPECT_PARAM_OR_EXCEPT;
        break;
      case STATE_EXPECT_PARAM_OR_EXCEPT:
        if (ParamBuilder.isStart(name)) {
          paramBuilder = new ParamBuilder();
          state = STATE_INSIDE_PARAM;
          paramBuilder.startElement(name, atts);
        }
        else {
          exceptPatternBuilder = new ExceptPatternBuilder();
          state = STATE_INSIDE_EXCEPT;
          exceptPatternBuilder.startElement(name, atts);
        }
        break;
      case STATE_INSIDE_PARAM:
        paramBuilder.startElement(name, atts);
        break;
      case STATE_INSIDE_EXCEPT:
        exceptPatternBuilder.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    switch (state) {
      case STATE_INSIDE_PARAM:
        if (paramBuilder.endElement()) {
          paramBuilderList.add(paramBuilder);
          paramBuilder = null;
          state = STATE_EXPECT_PARAM_OR_EXCEPT;
        }
        return false;
      case STATE_INSIDE_EXCEPT:
        if (exceptPatternBuilder.endElement())
          state = STATE_EXPECT_END_DATA;
        return false;
      case STATE_EXPECT_PARAM_OR_EXCEPT:
      case STATE_EXPECT_END_DATA:
        state = STATE_END;
        return true;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_PARAM:
        paramBuilder.characters(ch, start, length);
        break;
    }
  }

  ////////////////////////////////////////
  // NonEmptyPatternHandler implementation
  ////////////////////////////////////////

  public NonEmptyPatternImpl result(Map fromNameToElementDefinition) {
    if (state != STATE_END)
      return null;
    int paramCount = paramBuilderList.size();
    Param[] params = new Param[paramCount];
    for (int k = 0; k < paramCount; k++)
      params[k] = ((ParamBuilder)paramBuilderList.get(k)).result();
    DataImpl data = new DataImpl(type, datatypeLibraryURI, params);
    if (exceptPatternBuilder != null)
      data.setExcept(exceptPatternBuilder.result(fromNameToElementDefinition));
    return data;
  }
}
