/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import java.util.ArrayList;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.NameClass;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.NameClassHandler;
import org.xmloperator.document.relaxng.parse.AnyNameBuilder;
  // Reciprocal import
import org.xmloperator.document.relaxng.parse.NsNameBuilder;
  // Reciprocal import
import org.xmloperator.document.relaxng.parse.NameBuilder;
import org.xmloperator.document.relaxng.parse.ChoiceNameClassBuilder;
  // Reciprocal import

// nameClass  ::= <anyName> [exceptNameClass] </anyName>
//   | <nsName ns="string"> [exceptNameClass] </nsName>
//   | <name ns="string"> NCName </name>
//   | <choice> nameClass nameClass </choice>
public class NameClassBuilder implements NameClassHandler {
  private static final int STATE_EXPECT_NAMECLASS = 0;
  private static final int STATE_INSIDE_NAMECLASS = 1;
  private static final int STATE_END = 2;

  private int state = STATE_EXPECT_NAMECLASS; // STATE_*
  private NameClassHandler nameClassHandler = null;

  public static final boolean isStart(String elementName) {
    return elementName.equals("anyName")
      || elementName.equals("nsName")
      || elementName.equals("name")
      || elementName.equals("choice");
  }

  public NameClassBuilder() {
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("nameClass", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    switch (state) {
      case STATE_EXPECT_NAMECLASS:
        if (name.equals("anyName"))
          nameClassHandler = new AnyNameBuilder();
        else if (name.equals("nsName"))
          nameClassHandler = new NsNameBuilder();
        else if (name.equals("name"))
          nameClassHandler = new NameBuilder();
        else if (name.equals("choice"))
          nameClassHandler = new ChoiceNameClassBuilder(new ArrayList());
        else
          error();
        state = STATE_INSIDE_NAMECLASS;
      case STATE_INSIDE_NAMECLASS:
        nameClassHandler.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    switch (state) {
      case STATE_INSIDE_NAMECLASS:
        if (nameClassHandler.endElement()) {
          state = STATE_END;
          return true;
        }
        else
          return false;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_NAMECLASS:
        nameClassHandler.characters(ch, start, length);
        break;
    }
  }

  //////////////////////////////////
  // NameClassHandler implementation
  //////////////////////////////////

  public NameClass result() {
    return state == STATE_END ? nameClassHandler.result() : null;
  }
}
