/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.util;

import org.w3c.dom.Node;

import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.document.util.NodeIntervalImpl;

public abstract class NodeIntervalUtils {

  public static boolean isNodeInterval(Node firstNode, Node lastNode) {
    if (firstNode == null || lastNode == null)
      throw new IllegalArgumentException();
    Node node = firstNode;
    while (true) {
      if (node == lastNode)
        return true;
      node = node.getNextSibling();
      if (node == null)
        return false;
    }
  }

  public static int nodeIntervalSize(Node firstNode, Node lastNode) {
    if (firstNode == null || lastNode == null)
      throw new IllegalArgumentException();
    int count = 1;
    Node node = firstNode;
    while (true) {
      if (node == lastNode)
        return count;
      node = node.getNextSibling();
      if (node == null)
        throw new IllegalArgumentException();
      count++;
    }
  }

  public static void moveNodeInterval(Node firstSourceNode, Node lastSourceNode,
    Node destinationParentNode, Node destinationNextSibling) {
    if (!isNodeInterval(firstSourceNode, lastSourceNode)
      || destinationParentNode == null
      || (destinationNextSibling != null
      && destinationNextSibling.getParentNode() != destinationParentNode))
      throw new IllegalArgumentException();
    Node sourceNode = firstSourceNode;
    while (true) {
      Node nextSibling = sourceNode.getNextSibling();
      destinationParentNode.insertBefore(sourceNode, destinationNextSibling);
      if (sourceNode == lastSourceNode)
        break;
      sourceNode = nextSibling;
    }
  }

  public static Node[] nodesFromInterval(Node firstNode, Node lastNode) {
    int count = nodeIntervalSize(firstNode, lastNode);
    Node[] nodes = new Node[count];
    Node node = firstNode;
    for (int k = 0; k < count; k++) {
      nodes[k] = node;
      node = node.getNextSibling();
    }
    return nodes;
  }

  public static final NodeInterval nodeIntervalFromNodes(Node[] nodes) {
    if (nodes.length == 0)
      throw new IllegalArgumentException();
    Node parentNode = nodes[0].getParentNode();
    int firstIndex = Integer.MAX_VALUE;
    int lastIndex = 0;
    for (int k = 0; k < nodes.length; k++) {
      Node node = nodes[k];
      if (node.getParentNode() != parentNode)
        throw new IllegalArgumentException();
      int index = getNodeIndex(node);
      if (index < firstIndex)
        firstIndex = index;
      if (index > lastIndex)
        lastIndex = index;
    }
    if (lastIndex - firstIndex + 1 != nodes.length)
      throw new IllegalArgumentException();
    NodeInterval nodeInterval = new NodeIntervalImpl();
    nodeInterval.set(
      getChildNodeFromIndex(parentNode, firstIndex),
      getChildNodeFromIndex(parentNode, lastIndex));
    return nodeInterval;
  }

  private static final int getNodeIndex(Node node) {
    int index = 0;
    while (true) {
      node = node.getPreviousSibling();
      if (node == null)
        return index;
      index++;
    }
  }

  private static final Node getChildNodeFromIndex(Node parentNode,
    int childNodeIndex) {
    Node node = parentNode.getFirstChild();
    while (node != null) {
      if (childNodeIndex == 0)
        return node;
      node = node.getNextSibling();
      childNodeIndex--;
    }
    return null;
  }
}

