/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.util;

import java.util.Vector;

import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.manage.NodeChangeEvent;
import org.xmloperator.framework.document.manage.NodeChangeListener;

public final class VariableNodeListImpl
    implements VariableNodeList, NodeChangeListener {
  private final boolean isStack;
  private ObserverView observerView = null;
  private final Vector nodeList = new Vector();

  public VariableNodeListImpl(boolean isStack) {
    this.isStack = isStack;
  }

  public void done() {
    this.nodeList.removeAllElements();
    if (this.observerView != null)
      this.observerView.dispose();
  }

  //////////////////////////
  // NodeList implementation
  //////////////////////////

  public int getLength() {
    return this.nodeList.size();
  }

  public Node item(int index) {
    return (Node)this.nodeList.get(index);
  }

  //////////////////////////////////
  // VariableNodeList implementation
  //////////////////////////////////

  public void setObserverView(ObserverView observerView) {
    if (this.observerView != null && observerView != this.observerView)
      this.observerView.dispose();
    this.observerView = observerView;
  }

  public void add(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    if (this.isStack) {
      if (this.nodeList.indexOf(node) == 0)
        return;
      this.nodeList.remove(node);
      this.nodeList.add(0, node);
    }
    else {
      if (this.nodeList.contains(node))
        return;
      this.nodeList.add(node);
    }
    if (this.observerView != null)
      this.observerView.update(this, null);
  }

  public void remove(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    if (this.nodeList.remove(node)
        && this.observerView != null)
      this.observerView.update(this, null);
    {
      Node childNode = node.getFirstChild();
      while (childNode != null) {
        this.remove(childNode);
        childNode = childNode.getNextSibling();
      }
    }
  }

  public void clear() {
    this.nodeList.removeAllElements();
    if (this.observerView != null)
      this.observerView.update(this, null);
  }

  ////////////////////////////////////
  // NodeChangeListener implementation
  ////////////////////////////////////

  public void notifyNodeChangeEvent(NodeChangeEvent nodeChangeEvent) {
    switch (nodeChangeEvent.getEventType()) {
      case NodeChangeEvent.EVENTTYPE_NODEREMOVE:
        this.remove(nodeChangeEvent.getNode());
        break;
      case NodeChangeEvent.EVENTTYPE_NODEAPPEARANCEMAYHAVECHANGED:
        if (this.nodeList.contains(nodeChangeEvent.getNode())
            && this.observerView != null)
          this.observerView.update(this, null);
        break;
    }
  }
}
