/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.manage;

import java.util.Collection;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.manage.AttributesContentAccess;

public interface DocumentUtil extends AttributesContentAccess {

  /////////////////////////////////
  // Methods about children content
  /////////////////////////////////

  /**
   * Return a representation of the content model of parentElement.
   */
  public abstract String contentModelToString(Element parentElement,
    boolean isSummarized);

  /**
   * Return a representation of the content model which contains element.
   */
  public abstract String elementFromContentModelToString(Element element,
    boolean isSummarized);

  ////////////////////
  // Methods about IDs
  ////////////////////

  /**
   * Return the element ID if it exists, null elsewhere.
   */
  public abstract String getId(Element element);

  /**
   * Return the set of reference IDs.
   */
  public abstract String[] referencesFromElement(Element element);

  /////////////////////////////////////
  // Methods for authorizing operations
  /////////////////////////////////////

  /**
   * Return true if moving nodes as specified by moveType is valid.
   */
  public abstract boolean areNodesMovable(Node[] nodes, int moveType);

  /**
   * Return true if removing nodes is valid.
   */
  public abstract boolean areNodesRemovable(Node[] nodes);

  /**
   * Return true if element can have child nodes.
   */
  public abstract boolean canElementHaveChilds(Element element);

  /**
   * Return true if element can have a child text node.
   */
  public abstract boolean canElementHaveText(Element element);

  /**
   * Return true if element should have a child text node.
   */
  public abstract boolean shouldElementHaveText(Element element);

  /**
   * Return true if inserting elements specified by insertedElementNames
   *   before nextSibling is valid.
   */
  public abstract boolean areElementNamesInsertable(
    String[] insertedElementNames, Element parentElement, Node nextSibling);

  /**
   * Return true if unpacking element children is valid.
   */
  public abstract boolean isElementUnpackable(Element element);

  /**
   * Return true if the children content of element is yet valid
   *   after changing the element name to newElementName.
   */
  public abstract boolean isElementNameReplacable(Element element,
    String newElementName);

  ////////////////////////////////////////////////////////////////////
  // Methods for determining element names for insert, replace or pack
  ////////////////////////////////////////////////////////////////////
  // An NsName maping to a namespace prefix is representened by a string
  //   such as "prefix:".
  // An NsName mapping to no namespace is represented by an empty string.
  // An AnyName is represented by the ":" string.

  /**
   * Return a set of element name which can be used for apending
   *   a first child element to parentElement.
   */
  public abstract Collection elementNamesForAppend(boolean isLazy,
    Element parentElement);

  /**
   * Return a set of element name which can be used for inserting
   *   after of before element (upon isInsertedAfter value).
   */
  public abstract Collection elementNamesForInsert(boolean isLazy,
    Element element, boolean isInsertedAfter);

  /**
   * Return a set of element name which can be used for replacing the node.
   */
  public abstract Collection elementNamesForReplace(boolean isLazy, Node node);

  /**
   * Return a set of element name which can be used for packing nodes.
   */
  public abstract Collection elementNamesForPack(boolean isLazy, Node[] nodes);

  /**
   * Return a set of names of elements and attributes for inserting one as
   *   child of parentElement for begining to fit an unsatisfied pattern.
   * case DTD : each unsatisfiedPattern must be an element name.
   * case RNG : each unsatisfiedPattern must be a Ref or an Attribute pattern.
   */
  public abstract Collection elementAndAttributeNamesForRequiredInsert(
    Element parentElement, Object[] unsatisfiedPatterns);
}
