/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.manage;

import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Element;

/**
 * A Node manager assumes actions on nodes with consequences.
 */
public interface NodeManager {

  /**
   * Insert newNode before nextSibling if not null or as last child of
   *   parentNode if nextSibling is null.
   */
  public void insertNode(Node parentNode, Node nextSibling, Node newNode);

  /**
   * Insert newNodes before nextSibling if not null or as last child of
   *   parentNode if nextSibling is null.
   */
  public void insertNodes(
      Node parentNode, Node nextSibling, Node[] newNodes);

  /**
   * Remove the given node.
   */
  public void removeNode(Node node);

  /**
   * Remove the given nodes.
   */
  public void removeNodes(Node[] nodes);

  /**
   * Move the given node before destinationNextSibling if not null
   *   or as last child of destinationParentNode if destinationNextSibling
   *   is null.
   */
  public void moveNode(
      Node node, Node destinationParentNode, Node destinationNextSibling);

  /**
   * Move the given nodes before destinationNextSibling if not null
   *   or as last child of destinationParentNode if destinationNextSibling
   *   is null.
   */
  public void moveNodes(
      Node[] nodes, Node destinationParentNode, Node destinationNextSibling);

  /**
   * Replace the given old nodes by the new ones.
   */
  public void replaceNodes(Node[] oldNodes, Node[] newNodes);

  /**
   * Set the given new value to the given node.
   */
  public void setNodeValue(Node node, String newValue);

  /**
   * Join the next sibling text node to the given text node.
   */
  public void joinNextText(Text textNodeToJoinToNextOne);

  /**
   * Split the given text node at the given offset.
   */
  public void splitText(
      Text textNodeToSplit, int offset, Text recipientForSecondPart);

  /**
   * Replace oldElement by newElement and move the children from the old
   *   element to the new one.
   */
  public void replaceElementName(Element oldElement, Element newElement);

  /**
   * Replace the given nodes by the given element and move the nodes
   *   to children of the new element.
   */
  public void packNodes(Node[] nodes, Element newElement);

  /**
   * Replace the given element by its children.
   */
  public void unPackElement(Element element);

  /**
   * Reorder the given nodes.
   */
  public void reorderNodes(Node[] nodesInDestinationOrder);
}
