/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.manage;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

/**
 * A TreeViewDocumentListener is a listener to the set of document editing
 *   events that produce an effect on a tree view of the document.
 */
public interface TreeViewDocumentListener {

  /**
   * The notification occurs after insert.
   */
  public void notifyNodeInsert(Node node);

  /**
   * The notification occurs after insert.
   */
  public void notifyNodesInsert(Node[] nodes);

  /**
   * The notification occurs before remove.
   * Return a context object to pass after the operation.
   */
  public Object notifyBeforeNodeRemoveMoveOrReplace(Node node);

  /**
   * The notification occurs before remove.
   * Return a context object to pass after the operation.
   */
  public Object notifyBeforeNodesRemoveMoveOrReplace(Node[] nodes);

  /**
   * The notification occurs after remove.
   * Pass the context returned before the operation.
   */
  public void notifyAfterNodesRemove(Object context, Node nodeToSelect);

  /**
   * The notification occurs after move.
   * Pass the context returned before the operation.
   */
  public void notifyAfterNodeMoveOrReplace(Object context, Node node);

  /**
   * The notification occurs after move.
   * Pass the context returned before the operation.
   */
  public void notifyAfterNodesMoveOrReplace(Object context, Node[] nodes);

  /**
   * Notification occurs after set value.
   */
  public void notifyNodeSetValue(Node node);

  /**
   * Notification occurs before text join.
   * Return a context object to pass after the operation.
   */
  public Object notifyBeforeTextJoin(Node secondTextNode);

  /**
   * Notification occurs after text join.
   * Pass the context returned before the operation.
   */
  public void notifyAfterTextJoin(Object context, Node textNode);

  /**
   * Notification occurs after text split.
   */
  public void notifyTextSplit(Node textNode);

  /**
   * Notification occurs after set value.
   */
  public void notifyAttributeSetValue(Element element, String attributeName);

  /**
   * Notification occurs after replace.
   */
  public void notifyElementNameReplace(Element element);

  /**
   * Notification occurs before pack;
   * Return a context object to pass after the operation.
   */
  public Object notifyBeforeNodeIntervalPack(Node[] nodes);

  /**
   * Notification occurs after pack.
   * Pass the context returned before the operation.
   */
  public void notifyAfterNodeIntervalPack(Object context, Element element);

  /**
   * Notification occurs before element unpack.
   * Return a context object to pass after the operation.
   */
  public Object notifyBeforeElementUnPack(Element element);

  /**
   * Notification occurs after element unpack.
   * Pass the context returned before the operation.
   */
  public void notifyAfterElementUnpack(Object context, Node[] children);
}
