/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.schema;

import java.util.Set;

import org.w3c.dom.Element;

/**
 * Maps a content to a content model in order to support content editing.
 * A contentElementSpec represents an element of the content or an element
 *   inside an entity referenced in the content.
 */
public interface ElementContent {

  /**
   * Return the element specification count.
   */
  public abstract int getElementSpecCount();

  /**
   * Return the contentElementSpec at the contentElementSpecIndex position.
   * case DTD : it is an element name string.
   * case RNG : it is either a QualifiedName or an ElementDefinition.
   */
  public abstract Object getElementSpec(int contentElementSpecIndex);

  /**
   * Return the element definition associated to the element at the
   *   contentElementSpecIndex position.
   * case DTD : it is an element name string.
   * case RNG : it is an ElementDefinition.
   */
  public abstract Object getElementDefinition(int contentElementSpecIndex);

  /**
   * Return true if the element definition associated to the element at the
   *   contentElementSpecIndex position can be used repeatedly for
   *   inserting new elements.
   */
  public abstract boolean isElementDefinitionRepeatable(
      int contentElementSpecIndex);

  /**
   * Adds to a set all the element definitions of elements that could replace
   *   the element at the contentElementSpecIndex position.
   * Pre-condition : 0 <= contentElementSpecIndex < contentElementSpecCount
   */
  public abstract void addElementDefinitionsForReplace(
      Set elementDefinitionSet, int contentElementSpecIndex);

  /**
   * Return the set of element definitions of elements that could be inserted
   *   before the element at the contentElementSpecIndex position.
   * Pre-condition : 0 <= contentElementSpecIndex <= contentElementSpecCount
   * Doesn't take into account the element definitions of elements
   *   that could replace the element or the previous one.
   */
  public abstract Set elementDefinitionsInsertableBefore(
      int contentElementSpecIndex);

  /**
   * return a string that represents the content model focused on
   *   the element at the contentElementSpecIndex position.
   */
  public abstract String elementFromContentModelToString(
    int contentElementSpecIndex, boolean isSummarized, Element contextElement);
}
