/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.option;

import java.awt.Font;
import java.util.Locale;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;

public interface Constants {

  // Tool name and version
  public static final String TOOL = "xmloperator";
  public static final String PROJECT = "The_" + TOOL + "_project";
  public static final String COPYRIGHT = "Copyright (c) 2000 - 2003 " + PROJECT;
  public static final int VERSION_GENERALE = 2;
  public static final int VERSION_REVISION = 3;
  public static final int VERSION_CORRECTIVE = 0;
  public static final String VERSION =
    VERSION_GENERALE + "." + VERSION_REVISION + "." + VERSION_CORRECTIVE;

  // File names and extensions
  public static final String TOOL_OPTIONS_FILE_EXTENSION = "xin";
  public static final String TOOL_FILE =
    TOOL + '_' + VERSION_GENERALE + '_' + VERSION_REVISION;

  // Full names
  public static final String TOOL_PREFIX = "org";
  public static final String XSLT_TRANSFORMER =
    TOOL_PREFIX + "." + TOOL + ".transform.XSLTTransformer";

  // XML versions
  public static final String[] xMLVersions = new String[] {"1.0", "1.1"};

  // Encoding names
  public static final String DEFAULT_ENCODING = "UTF-8";
  public static final String[] encodingNames = {
    "UTF-8",
    "ISO-8859-1", // ISO Latin-1
    "ISO-8859-2", // ISO Latin-2 [Bosnian, Croatian, Czech, Hungarian, Polish,
      // Romanian, Serbian (in Latin transcription), Serbocroatian, Slovak,
      // Slovenian, Upper and Lower Sorbian]
    "ISO-8859-3", // ISO Latin-3 [Maltese, Esperanto]
    "ISO-8859-4", // ISO Latin-4
    "ISO-8859-5", // ISO Latin Cyrillic
    "ISO-8859-6", // ISO Latin Arabic
    "ISO-8859-7", // ISO Latin Greek
    "ISO-8859-8", // ISO Latin Hebrew
    "ISO-8859-9", // ISO Latin-5 [Turkish]
    "BIG5", // Chinese: Big5
    "ISO-2022-JP", // Japanese ISO-2022-JP
    "EUC-JP", // Extended Unix Code, packed for Japanese
    "GB2312", // Chinese for PRC (mixed 1/2 byte)
    "EUC-KR", // Extended Unix Code, packed for Korean
    "ISO-2022-KR", // Korean ISO-2022-KR
    "KOI8-R", // Russian Unix, Cyrillic
    "SHIFT_JIS", // Japanese: Shift JIS
    "UTF-16"};

  // Languages
  public static final int LANGUAGE_DEFAULT = 0;
  public static final int LANGUAGE_ENGLISH = 1;
  public static final int LANGUAGE_FRENCH = 2;
  public static final Locale[] locales = {
    Locale.getDefault(),
    Locale.ENGLISH,
    Locale.FRENCH};

  // Look and feel
  public static final int LOOKANDFEEL_SYSTEM = 0;
  public static final int LOOKANDFEEL_METAL = 1;
  public static final String[] lookAndFeelClassNames = {
    UIManager.getSystemLookAndFeelClassName(),
    "javax.swing.plaf.metal.MetalLookAndFeel"};

  // Fonts
  public static final Font FONT_NORMAL = new Font("Dialog", Font.PLAIN, 12);
  public static final Font FONT_LARGE = new Font("Dialog", Font.PLAIN, 14);
  public static final Font FONT_MONOSPACE_NORMAL =
      new Font("Monospaced", Font.PLAIN, 12);
  public static final Font FONT_MONOSPACE_LARGE =
      new Font("Monospaced", Font.PLAIN, 14);
}
