/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation;

import java.io.IOException;
import java.io.File;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.operation.OperationPrinter;

public class OperationSaver implements OperationPrinter {

  public static void saveOperationStack(OperationStack operationStack,
    String pathName) throws IOException {
    OperationSaver operationSaver = new OperationSaver();
    operationSaver.open(pathName, null);
    try {
      while (operationStack.getPossibleRedoCount() != 0)
        operationSaver.printElement(operationStack.redo(false, true));
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      operationSaver.close();
    }
  }

  protected String pathName = null;
  protected File tempFile = null;
  protected PrintWriter printWriter = null;
  protected int operationElementPrintCount = 0;

  public OperationSaver() {
  }

  public void open(String pathName, File tempFile) throws IOException {
    if (pathName == null)
      throw new IllegalArgumentException();
    this.pathName = pathName;
    this.tempFile = tempFile;
    if (tempFile != null)
      tempFile.createNewFile(); // le conserve tel quel s'il existe dja
    rewriteIfOpen();
  }

  public void rewriteIfOpen() throws IOException {
    if (pathName == null)
      return;
    if (printWriter != null) {
      flush();
      printWriter.close();
      printWriter = null;
    }
    FileOutputStream fileOutputStream = new FileOutputStream(pathName);
    printWriter = new PrintWriter(
      new OutputStreamWriter(fileOutputStream, "UTF8"));
    printWriter.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    printWriter.println("<operations>");
    printWriter.flush();
    operationElementPrintCount = 0;
  }

  //////////////////////////////////
  // OperationPrinter implementation
  //////////////////////////////////

  public boolean isReady() {
    return printWriter != null;
  }

  public void printOperation(Operation operation, Element operationElement) {
    if (operation == null || operationElement == null)
      throw new IllegalArgumentException();
    if (printWriter == null)
      return;
    printElement(operationElement);
    printWriter.flush();
  }

  public void flush() {
  }

  public void close() {
    if (printWriter != null) {
      flush();
      printWriter.println("</operations>");
      printWriter.close();
      printWriter = null;
      if (operationElementPrintCount == 0)
        new File(pathName).delete();
    }
    if (tempFile != null)
      tempFile.delete();
  }

  protected void printElement(Element element) {
    operationElementPrintCount++;
    String elementName = element.getNodeName();
    printWriter.print('<');
    printWriter.println(elementName);
    NamedNodeMap attrs = element.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++) {
      Attr attr = (Attr)attrs.item(k);
      DomWriter.printAttribute(
          printWriter, attr.getNodeName(), attr.getNodeValue());
      printWriter.println();
    }
    printWriter.print('>');
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      printNode(childNode);
      childNode = childNode.getNextSibling();
    }
    printWriter.print('<');
    printWriter.print('/');
    printWriter.print(elementName);
    printWriter.println('>');
  }

  protected void printNode(Node node) {
    switch (node.getNodeType()) {
      case Node.ELEMENT_NODE:
        printElement((Element)node);
        break;
      case Node.TEXT_NODE:
        DomWriter.printText(printWriter, node.getNodeValue());
        break;
    }
  }
}
