/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.Event;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.AbstractListModel;
import javax.swing.ListCellRenderer;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.xmloperator.framework.util.Observer;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.io.FileExtensionFilter;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.operation.OperationStack;

public final class OperationStackFrame extends JFrame implements Observer {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private Options options;
  private OperationStack operationStack;
  private EditControl editControl;
  private JList list;
  private OperationStackListModel listModel;
  private JMenuItem menuEffacerDefaire;
  private JMenuItem menuEffacerRefaire;
  private JMenuItem menuToutDefaire;
  private JMenuItem menuToutRefaire;
  private JMenuItem menuCharger;
  private JMenuItem menuEnregistrer;

  public OperationStackFrame(int instanceIdentifier,
    Options options, OperationStack operationStack,
    EditControl editControl, Shortcuts shortcuts) {
    setTitle(Constants.TOOL + '.' + instanceIdentifier + " - "
      + viewResource.getString(Label.OPERATIONS));
    this.options = options;
    this.operationStack = operationStack;
    this.editControl = editControl;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(300, 200));
    shortcuts.registerKeyboardAction(voletContenu);
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    menuToutDefaire =
      new JMenuItem(viewResource.getString(Label.TOUT_DEFAIRE));
    menuToutDefaire.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.TOUT_DEFAIRE).charAt(0));
    menuToutDefaire.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          toutDefaire_actionPerformed();
        }
      });
    menuBar.add(menuToutDefaire);
    menuToutRefaire =
      new JMenuItem(viewResource.getString(Label.TOUT_REFAIRE));
    menuToutRefaire.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.TOUT_REFAIRE).charAt(0));
    menuToutRefaire.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          toutRefaire_actionPerformed();
        }
      });
    menuBar.add(menuToutRefaire);
    menuEffacerDefaire =
      new JMenuItem(viewResource.getString(Label.EFFACER_DEFAIRE));
    menuEffacerDefaire.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.EFFACER_DEFAIRE).charAt(0));
    menuEffacerDefaire.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          effacerDefaire_actionPerformed();
        }
      });
    menuBar.add(menuEffacerDefaire);
    menuEffacerRefaire =
      new JMenuItem(viewResource.getString(Label.EFFACER_REFAIRE));
    menuEffacerRefaire.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.EFFACER_REFAIRE).charAt(0));
    menuEffacerRefaire.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          effacerRefaire_actionPerformed();
        }
      });
    menuBar.add(menuEffacerRefaire);
    menuCharger =
      new JMenuItem(viewResource.getString(Label.CHARGER));
    menuCharger.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.CHARGER).charAt(0));
    menuCharger.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          charger_actionPerformed();
        }
      });
    menuBar.add(menuCharger);
    menuEnregistrer = new JMenuItem(
      viewResource.getString(Label.ENREGISTRER));
    menuEnregistrer.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.ENREGISTRER).charAt(0));
    menuEnregistrer.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          enregistrer_actionPerformed();
        }
      });
    menuBar.add(menuEnregistrer);
    // Raccourcis
    voletContenu.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          defaire_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.CTRL_MASK),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    voletContenu.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          refaire_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.SHIFT_MASK + Event.CTRL_MASK),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    // Volet liste
    listModel =
      new OperationStackListModel((OperationStackReadOnly)operationStack);
    list = new JList(listModel);
    list.setCellRenderer(
      new OperationCellRenderer((OperationStackReadOnly)operationStack));
    JScrollPane scrollPane = new JScrollPane(list);
    voletContenu.add(scrollPane);
    // Mise aux dimensions
    this.pack();
    // Initialisation
    update(null, new Integer(OperationStackReadOnly.NOTIFY_NEW));
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private void defaire_actionPerformed() {
    editControl.undoOperation();
  }

  private void refaire_actionPerformed() {
    editControl.redoOperation();
  }

  private void toutDefaire_actionPerformed() {
    editControl.undoAllOperations();
  }

  private void toutRefaire_actionPerformed() {
    editControl.redoAllOperations();
  }

  private void effacerDefaire_actionPerformed() {
    editControl.clearUndoOperations();
  }

  private void effacerRefaire_actionPerformed() {
    editControl.clearRedoOperations();
  }

  private void charger_actionPerformed() {
    String pathname = choosePathName(false);
    if (pathname != null)
      editControl.loadOperationStack(pathname);
  }

  private void enregistrer_actionPerformed() {
    String pathname = choosePathName(true);
    if (pathname == null)
      return;
    if (pathname.lastIndexOf('.') < 0)
      pathname += ".ope";
    editControl.saveOperationStack(pathname);
  }

  private String choosePathName(boolean forSave) {
    JFileChooser fileChooser =
      new JFileChooser(options.getStringOptionValue(Options.OPERATIONSPATH));
    fileChooser.setDialogType(
      forSave ? JFileChooser.SAVE_DIALOG : JFileChooser.OPEN_DIALOG);
    fileChooser.setDialogTitle(forSave ?
      viewResource.getString(Label.ENREGISTRER_OPERATIONS_SOUS) :
      viewResource.getString(Label.CHARGER_OPERATIONS));
    final FileFilter opeFileFilter = new FileExtensionFilter("ope");
    fileChooser.setFileFilter(opeFileFilter);
    if (fileChooser.showDialog((Frame)this, null)
      != JFileChooser.APPROVE_OPTION)
      return null;
    String pathName = fileChooser.getSelectedFile().getPath();
    pathName = FileUtils.normalisePathName(pathName);
    {
      int index = pathName.lastIndexOf('/');
      if (index >= 0)
        options.setStringOptionValue(Options.OPERATIONSPATH,
          pathName.substring(0, index + 1));
    }
    return pathName;
  }

  public void setMenuEnabled(boolean isEnabled) {
    int undoCount = operationStack.getPossibleUndoCount();
    int redoCount = operationStack.getPossibleRedoCount();
    menuEffacerDefaire.setEnabled(isEnabled && undoCount != 0);
    menuEffacerRefaire.setEnabled(isEnabled && redoCount != 0);
    menuToutDefaire.setEnabled(isEnabled && undoCount != 0);
    menuToutRefaire.setEnabled(isEnabled && redoCount != 0);
    menuCharger.setEnabled(isEnabled);
    menuEnregistrer.setEnabled(isEnabled && undoCount != 0 && redoCount == 0);
  }

  public static class OperationStackListModel extends AbstractListModel {
    private OperationStackReadOnly operationStack;
    private int lastUndoCount;
    private int lastRedoCount;

    public OperationStackListModel(OperationStackReadOnly operationStack) {
      this.operationStack = operationStack;
      this.lastUndoCount = 0;
      this.lastRedoCount = 0;
    }

    public void processNotification(int notification) {
      int undoCount = operationStack.getPossibleUndoCount();
      int redoCount = operationStack.getPossibleRedoCount();
      int size = undoCount + 1 + redoCount;
      int lastSize = lastUndoCount + 1 + lastRedoCount;
      switch (notification) {
      case OperationStackReadOnly.NOTIFY_NEW:
        if (size != 0)
          fireIntervalAdded(this, 0, size - 1);
        break;
      case OperationStackReadOnly.NOTIFY_CLEAR_NEW:
        if (lastSize != 0)
          fireIntervalRemoved(this, 0, lastSize - 1);
        if (size != 0)
          fireIntervalAdded(this, 0, size - 1);
        break;
      case OperationStackReadOnly.NOTIFY_ADD:
        fireContentsChanged(this, lastUndoCount, lastUndoCount);
        fireIntervalAdded(this, undoCount, undoCount);
        break;
      case OperationStackReadOnly.NOTIFY_REPLACE:
        if (undoCount != 0)
          fireContentsChanged(this, undoCount - 1, undoCount - 1);
        break;
      case OperationStackReadOnly.NOTIFY_CLEAR_UNDO:
        if (lastUndoCount != 0)
          fireIntervalRemoved(this, 0, lastUndoCount - 1);
        break;
      case OperationStackReadOnly.NOTIFY_CLEAR_REDO:
        if (lastRedoCount != 0)
          fireIntervalRemoved(this,
            lastUndoCount + 1, lastUndoCount + lastRedoCount);
        break;
      case OperationStackReadOnly.NOTIFY_UNDO:
        fireContentsChanged(this, undoCount, undoCount);
        fireContentsChanged(this, lastUndoCount, lastUndoCount);
        break;
      case OperationStackReadOnly.NOTIFY_REDO:
        fireContentsChanged(this, undoCount, undoCount);
        fireContentsChanged(this, lastUndoCount, lastUndoCount);
        break;
      }
      lastUndoCount = undoCount;
      lastRedoCount = redoCount;
    }

    ///////////////////////////
    // ListModel implementation
    ///////////////////////////

    public int getSize() {
      int undoCount = operationStack.getPossibleUndoCount();
      int redoCount = operationStack.getPossibleRedoCount();
      return undoCount + 1 + redoCount;
    }

    public Object getElementAt(int index) {
      try {
        int undoCount = operationStack.getPossibleUndoCount();
        int redoCount = operationStack.getPossibleRedoCount();
        int size = undoCount + 1 + redoCount;
        if (index < 0 || index >= size)
          return "";
        if (index < undoCount)
          return operationStack.getUndoOperationName(undoCount - 1 - index);
        if (index > undoCount)
          return operationStack.getRedoOperationName(index - undoCount - 1);
        return "";
      }
      catch (Exception exception) {
        ExceptionControler.printStackTrace(exception);
        return "";
      }
    }
  }

  private static class OperationCellRenderer implements ListCellRenderer {
    private OperationStackReadOnly operationStack;
    private DefaultListCellRenderer defaultListCellRenderer;
    private JPanel blackSeparator;
    private JPanel whiteSeparator;

    public OperationCellRenderer(OperationStackReadOnly operationStack) {
      this.operationStack = operationStack;
      this.defaultListCellRenderer = new DefaultListCellRenderer();
      this.blackSeparator = createSeparator(Color.black);
      this.whiteSeparator = createSeparator(Color.white);
    }

    private JPanel createSeparator(Color color) {
      JPanel separator = new JPanel();
      separator.setOpaque(true);
      separator.setPreferredSize(new Dimension(200, 1));
      separator.setBackground(color);
      return separator;
    }

    //////////////////////////////////
    // ListCellRenderer implementation
    //////////////////////////////////

    public Component getListCellRendererComponent(JList list, Object value,
      int index, boolean isSelected,  boolean cellHasFocus) {
      int undoCount = operationStack.getPossibleUndoCount();
      int redoCount = operationStack.getPossibleRedoCount();
      return index == undoCount ?
        redoCount != 0 ? blackSeparator : whiteSeparator :
        defaultListCellRenderer.getListCellRendererComponent(list, value,
        index, isSelected, cellHasFocus);
    }
  }

  //////////////////////////
  // Observer implementation
  //////////////////////////

  public void update(Object observable, Object argument) {
    if (argument == null || !(argument instanceof Integer))
      throw new IllegalArgumentException();
    listModel.processNotification(((Integer)argument).intValue());
    list.clearSelection();
    int undoCount = operationStack.getPossibleUndoCount();
    int redoCount = operationStack.getPossibleRedoCount();
    list.ensureIndexIsVisible(undoCount);
    if (undoCount > 0)
      list.ensureIndexIsVisible(undoCount - 1);
    if (redoCount > 0)
      list.ensureIndexIsVisible(undoCount + 1);
    setMenuEnabled(true);
  }
}
