/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;

public abstract class AbstractOperation implements Operation {
  protected static final String NO_OPERATION = "No operation";
  protected boolean isExecuted = false;
  protected String operationName = NO_OPERATION;
  protected String reverseOperationName = NO_OPERATION;

  public AbstractOperation() {
  }

  public final String getOperationName() {
    return operationName;
  }

  public final String getReverseOperationName() {
    return reverseOperationName;
  }

  public boolean isNull() {
    return false;
  }

  public final boolean isExecuted() {
    return isExecuted;
  }

  public boolean isHidable() {
    return false;
  }

  public boolean canHide(Operation operation) {
    return false;
  }

  public final boolean hidesIfCan(Operation operation) {
    if (!isExecuted || !operation.isExecuted())
      throw new OperationException(OperationException.ILLEGAL_STATE);
    if (canHide(operation)) {
      hides(operation);
      return true;
    }
    else
      return false;
  }

  // Replace the operation composed by itself preceded by operation.
  // Pre-condition : isExecuted == true
  // Pre-condition : operation.isExecuted == true
  // Pre-condition : this.canHide(operation)
  // Post-condition : isExecuted == true
  // Post-condition : operation is unchanged
  protected void hides(Operation operation) {
  }
}
