/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.document.util.NodeIntervalUtils;
import org.xmloperator.operation.core.AbstractOperation;
import org.xmloperator.operation.core.OperationNameUtils;

public final class NodeIntervalMoveOperation extends AbstractOperation {
  private final NodeManager nodeManager;
  private final Node firstSourceNode;
  private final Node lastSourceNode;
  private final Node destinationParentNode;
  private final Node destinationNextSibling;
  private Node sourceParentNode = null;
  private Node sourceNextSibling = null;

  public NodeIntervalMoveOperation(
    NodeManager nodeManager, Node firstSourceNode, Node lastSourceNode,
    Node destinationParentNode, Node destinationNextSibling) {
    if (firstSourceNode == null || lastSourceNode == null
      || destinationParentNode == null
      || (destinationNextSibling != null
      && destinationNextSibling.getParentNode() != destinationParentNode))
      throw new IllegalArgumentException();
    this.nodeManager = nodeManager;
    this.firstSourceNode = firstSourceNode;
    this.lastSourceNode = lastSourceNode;
    this.destinationParentNode = destinationParentNode;
    this.destinationNextSibling = destinationNextSibling;
  }

  public Element execute(Document operationDocument) {
    if (this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    this.sourceParentNode = this.firstSourceNode.getParentNode();
    if (this.sourceParentNode == null)
      throw new OperationException(OperationException.NODE_HAS_NO_PARENT);
    if (!NodeIntervalUtils.isNodeInterval(
        this.firstSourceNode, this.lastSourceNode))
      throw new OperationException(OperationException.NOT_A_NODE_INTERVAL);
    {
      boolean isOneNode = this.firstSourceNode == this.lastSourceNode;
      this.operationName = "Move " + (isOneNode ? "" : "within ")
          + OperationNameUtils.getNodeName(
          isOneNode ? this.firstSourceNode : this.destinationParentNode);
    }
    Element operationElement = this.createOperationElement(operationDocument);
    this.sourceNextSibling = this.lastSourceNode.getNextSibling();
    // pour la future cration de l'opration inverse
    {
      Node[] sourceNodes =
          NodeIntervalUtils.nodesFromInterval(firstSourceNode, lastSourceNode);
      this.nodeManager.moveNodes(
          sourceNodes, this.destinationParentNode, this.destinationNextSibling);
    }
    this.isExecuted = true;
    return operationElement;
  }

  private Element createOperationElement(Document operationDocument) {
    if (operationDocument == null)
      return null;
    try {
      Element operationElement =
          operationDocument.createElement("nodeIntervalMoveOperation");
      Element parentElement =
          sourceParentNode.getNodeType() == Node.ELEMENT_NODE ?
          (Element)this.sourceParentNode : null;
      if (parentElement != null)
        operationElement.setAttribute(
            "sourceParentElementPath",
            NodePathBuilder.buildElementPathWithNames(parentElement));
      operationElement.setAttribute(
          "firstSourceNodeIndex",
          String.valueOf(NodePathBuilder.getNodeIndex(this.firstSourceNode)));
      operationElement.setAttribute(
          "lastSourceNodeIndex",
          String.valueOf(NodePathBuilder.getNodeIndex(this.lastSourceNode)));
      if (destinationParentNode.getNodeType() == Node.ELEMENT_NODE)
        operationElement.setAttribute(
            "destinationElementPath",
            NodePathBuilder.buildElementPathWithNames(
            (Element)this.destinationParentNode));
      if (this.destinationNextSibling != null)
        operationElement.setAttribute(
            "destinationNodeIndex",
            String.valueOf(NodePathBuilder.getNodeIndex(
            this.destinationNextSibling)));
      return operationElement;
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return null;
    }
  }

  public Operation createReverseOperation() {
    if (!this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    AbstractOperation reverseOperation = new NodeIntervalMoveOperation(
        this.nodeManager, firstSourceNode, lastSourceNode,
        sourceParentNode, sourceNextSibling);
    reverseOperation.reverseOperationName = this.operationName;
    return reverseOperation;
  }
}
