/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.operation.core.AbstractOperation;
import org.xmloperator.operation.core.OperationNameUtils;
import org.xmloperator.operation.core.NodesInsertOperation; // import rciproque

public final class NodesRemoveOperation extends AbstractOperation {
  private final NodeManager nodeManager;
  private final Node[] nodesToRemove;
  private Node parentNode = null;
  private Node nextSibling = null;

  public NodesRemoveOperation(NodeManager nodeManager, Node nodeToRemove) {
    this(nodeManager, new Node[] {nodeToRemove});
  }

  public NodesRemoveOperation(NodeManager nodeManager, Node[] nodesToRemove) {
    if (nodesToRemove == null || nodesToRemove.length == 0)
      throw new IllegalArgumentException();
    this.nodeManager = nodeManager;
    this.nodesToRemove = nodesToRemove;
  }

  public Element execute(Document operationDocument) {
    if (this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    int nodesCount = this.nodesToRemove.length;
    Node firstNodeToRemove = this.nodesToRemove[0];
    Node lastNodeToRemove = this.nodesToRemove[nodesCount - 1];
    this.parentNode = firstNodeToRemove.getParentNode();
    if (this.parentNode == null)
      throw new OperationException(
        OperationException.NODE_HAS_NO_PARENT);
    this.nextSibling = lastNodeToRemove.getNextSibling();
      // pour la future cration de l'opration inverse
    if (this.nextSibling != null
        && nextSibling.getParentNode() != this.parentNode) {
      ExceptionControler.println("firstNode.parentNode : "
        + OperationNameUtils.getNodeName(this.parentNode));
      ExceptionControler.println("lastNode.parentNode : "
        + OperationNameUtils.getNodeName(lastNodeToRemove.getParentNode()));
      ExceptionControler.println("nextSibling.parentNode : "
        + OperationNameUtils.getNodeName(this.nextSibling.getParentNode()));
      throw new OperationException(
        OperationException.NODES_HAVE_NOT_THE_SAME_PARENT);
    }
    boolean isOneNode = nodesCount == 1;
    this.operationName =
        "Remove " + (isOneNode ?
        OperationNameUtils.getNodeName(firstNodeToRemove) :
        "childs of " + OperationNameUtils.getNodeName(parentNode));
    Element operationElement = this.createOperationElement(operationDocument);
    Node nextSelected = this.nextSibling;
    if (nextSelected == null) {
      nextSelected = firstNodeToRemove.getPreviousSibling();
      if (nextSelected == null)
        nextSelected = this.parentNode;
    }
    this.nodeManager.removeNodes(this.nodesToRemove);
    this.isExecuted = true;
    return operationElement;
  }

  private Element createOperationElement(Document operationDocument) {
    if (operationDocument == null)
      return null;
    try {
      Node firstNodeToRemove = this.nodesToRemove[0];
      Node lastNodeToRemove = this.nodesToRemove[this.nodesToRemove.length - 1];
      Element operationElement =
          operationDocument.createElement("nodeIntervalRemoveOperation");
      if (parentNode.getNodeType() == Node.ELEMENT_NODE)
        operationElement.setAttribute(
            "elementPath",
            NodePathBuilder.buildElementPathWithNames((Element)this.parentNode));
      operationElement.setAttribute(
          "firstNodeIndex",
          String.valueOf(NodePathBuilder.getNodeIndex(firstNodeToRemove)));
      operationElement.setAttribute(
          "lastNodeIndex",
          String.valueOf(NodePathBuilder.getNodeIndex(lastNodeToRemove)));
      return operationElement;
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return null;
    }
  }

  public Operation createReverseOperation() {
    if (!this.isExecuted || this.parentNode == null)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    AbstractOperation reverseOperation =
      new NodesInsertOperation(this.nodeManager,
        this.parentNode, this.nextSibling, this.nodesToRemove);
    reverseOperation.reverseOperationName = this.operationName;
    return reverseOperation;
  }
}
