/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

public abstract class OperationNameUtils {

  // Return a string representation of a node.
  public static final String getNodeName(Node node) {
    if (node != null)
      switch (node.getNodeType()) {
        case Node.DOCUMENT_NODE:
          return "DOCUMENT";
        case Node.DOCUMENT_TYPE_NODE:
          return "DOCTYPE";
        case Node.ELEMENT_NODE:
          return getElementName((Element)node);
        case Node.TEXT_NODE:
          return "TEXT" + getNodeContext(node);
        case Node.CDATA_SECTION_NODE:
          return "CDATA" + getNodeContext(node);
        case Node.COMMENT_NODE:
          return "COMMENT" + getNodeContext(node);
        case Node.ENTITY_REFERENCE_NODE:
          return "ENTITY REFERENCE" + getNodeContext(node);
        case Node.PROCESSING_INSTRUCTION_NODE:
          return "PROCESSING INSTRUCTION" + getNodeContext(node);
        case Node.ATTRIBUTE_NODE:
          return "ATTRIBUTE" + getNodeContext(node);
      }
    return "??";
  }

  private static final String getNodeContext(Node node) {
    Node parentNode = node.getParentNode();
    if (parentNode != null)
      return " of " + getNodeName(parentNode);
    return "";
  }

  // Return a string representation of an element.
  private static final String getElementName(Element element) {
    StringBuffer s = new StringBuffer();
    s.append('<');
    String elementName = element.getNodeName();
    s.append(elementName);
    NamedNodeMap attrs = element.getAttributes();
    int attrCount = (attrs != null) ? attrs.getLength() : 0;
    for (int k = 0; k < attrCount; k++) {
      Node attr = attrs.item(k);
      String attrName = attr.getNodeName();
      s.append(' ');
      s.append(attrName);
      s.append('=');
      s.append('"');
      s.append(attr.getNodeValue());
      s.append('"');
    }
    s.append('>');
    return s.toString();
  }
}
