/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option;

import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.StringTokenizer;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.IntOption;
import org.xmloperator.framework.option.StringOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.io.MIME2Java;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.IntOptionPartType;
import org.xmloperator.option.part.ThreeStateOptionPartType;
import org.xmloperator.option.impl.ThreeStateOptionImpl;
import org.xmloperator.option.impl.IntOptionImpl;
import org.xmloperator.option.impl.StringOptionImpl;
import org.xmloperator.option.impl.ResourceContextImpl;
import org.xmloperator.option.impl.ProxyImpl;
import org.xmloperator.option.impl.ParameterImpl;
import org.xmloperator.option.impl.DocumentTypeInfoImpl;
import org.xmloperator.option.impl.TransformerInfoImpl;
import org.xmloperator.option.impl.OperatorInfoImpl;
import org.xmloperator.option.impl.MenuItemDataImpl;
import org.xmloperator.option.impl.NamespacePrefixPairImpl;

public final class OptionsImpl implements Options {
  private static final String DEFAULT_DATA_PATH = "data/";
  private static final String DEFAULT_STYLE_PATH = "style/";
  private static final String DEFAULT_RELAXNG_PATH = "relaxng/";
  // Option type Ids
  public static final int OPTIONTYPE_THREESTATE = 0;
  public static final int OPTIONTYPE_INTEGER = 1;
  public static final int OPTIONTYPE_STRING = 2;
  public static final int OPTIONTYPE_RESOURCE = 3;
  public static final int OPTIONTYPE_DOCUMENTTYPE = 4;
  public static final int OPTIONTYPE_PROXY = 5;
  public static final int OPTIONTYPE_PARAMETER = 6;
  public static final int OPTIONTYPE_MENUITEM = 7;
  public static final int OPTIONTYPE_TRANSFORMER = 8;
  public static final int OPTIONTYPE_OPERATOR = 9;
  public static final int OPTIONTYPE_NAMESPACE = 10;
  // Option definition structure
  private static final int OPTIONDEF_COUNT = 5;
  private static final int OPTIONDEF_ID = 0;
  private static final int OPTIONDEF_TYPE = 1;
  private static final int OPTIONDEF_TYPEID = 2;
  private static final int OPTIONDEF_PACKAGE = 3;
  private static final int OPTIONDEF_ADVANCED = 4;
  // Option definitions
  private static final Object[][] optionDefinitions = {
    {ADVANCEDUSE,
      ThreeStateOptionImpl.createOptionType("isAdvanced", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "General",
      Boolean.FALSE},
    {XMLPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_DATA_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {XSLPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_STYLE_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {DTDPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_DATA_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {RNGPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_RELAXNG_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {OPTIONSPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_DATA_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {OPERATIONSPATH,
      StringOptionImpl.createOptionType("path", DEFAULT_DATA_PATH),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {FILEEXTENSIONS,
      StringOptionImpl.createOptionType("extensions", "xml html xsl rng svg"),
      new Integer(OPTIONTYPE_STRING),     "Load",
      Boolean.TRUE},
    {PROXY,
      ProxyImpl.optionType,
      new Integer(OPTIONTYPE_PROXY),      "Load",
      Boolean.TRUE},
    {OPENERRORS,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Load",
      Boolean.TRUE},
    {RESOURCE,
      ResourceContextImpl.optionType,
      new Integer(OPTIONTYPE_RESOURCE),   "Load",
      Boolean.TRUE},
    {MAXRESOURCES,
      IntOptionImpl.createOptionType("count", 20),
      new Integer(OPTIONTYPE_INTEGER),    "Load",
      Boolean.TRUE},
    {AUTOMATICLOADING,
      ThreeStateOptionImpl.createOptionType("isAutomatic", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Load",
      Boolean.FALSE},
    {DTDAUTOMATICLOADING,
      ThreeStateOptionImpl.createOptionType("isAutomatic", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Load",
      Boolean.TRUE},
    {RNGAUTOMATICLOADING,
      ThreeStateOptionImpl.createOptionType("isAutomatic", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Load",
      Boolean.TRUE},
    {PARSINGVALIDATION,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {PARSINGVALIDATIONDYNAMIC,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {PARSINGVALIDATIONSCHEMA,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {PARSINGNONVALIDATINGLOADDTDGRAMMAR,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {PARSINGNONVALIDATINGLOADEXTERNALDTD,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {PARSINGDOMINCLUDEIGNORABLEWHITESPACE,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Parse",
      Boolean.TRUE},
    {DEFAULTENCODING,
      StringOptionImpl.createOptionType("name", "UTF-8"),
      new Integer(OPTIONTYPE_STRING),       "Save",
      Boolean.TRUE},
    {FILENAMEAUTOMATICINCREMENTING,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Save",
      Boolean.TRUE},
    {DOCTYPEIFDTD,
      ThreeStateOptionImpl.createOptionType("toSave", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Save",
      Boolean.TRUE},
    {ADDLINEFEEDATSTARTTAGEND,
      ThreeStateOptionImpl.createOptionType("toSave", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Save",
      Boolean.TRUE},
    {XSLTTRACEPATHNAME,
      StringOptionImpl.createOptionType("path", DEFAULT_DATA_PATH + "xslt.log"),
      new Integer(OPTIONTYPE_STRING),     "XSLT",
      Boolean.TRUE},
    {TRACEELEMENTS,
      ThreeStateOptionImpl.createOptionType("areTraced", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "XSLT",
      Boolean.TRUE},
    {TRACEGENERATION,
      ThreeStateOptionImpl.createOptionType("areTraced", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "XSLT",
      Boolean.TRUE},
    {TRACESELECTION,
      ThreeStateOptionImpl.createOptionType("areTraced", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "XSLT",
      Boolean.TRUE},
    {TRACETEMPLATES,
      ThreeStateOptionImpl.createOptionType("areTraced", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "XSLT",
      Boolean.TRUE},
    {QUIETCONFLICTWARNINGS,
      ThreeStateOptionImpl.createOptionType("areQuiet", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "XSLT",
      Boolean.TRUE},
    {SPLIT,
      ThreeStateOptionImpl.createOptionType("isHorizontal", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Display",
      Boolean.FALSE},
    {MAINWIDTH,
      IntOptionImpl.createOptionType("width", 600),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {MAINHEIGHT,
      IntOptionImpl.createOptionType("height", 400),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {DETAILPART,
      IntOptionImpl.createOptionType("perCent", 30),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {AUTOMATICLABELLING,
      ThreeStateOptionImpl.createOptionType("isAutomatic", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Display Tree",
      Boolean.TRUE},
    {STATUSSHOWN,
      ThreeStateOptionImpl.createOptionType("isShown", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Display Status",
      Boolean.TRUE},
    {LAZYSHOWN,
      ThreeStateOptionImpl.createOptionType("isShown", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Display Status",
      Boolean.TRUE},
    {LOOKANDFEELINDEX,
      IntOptionImpl.createOptionType("index", 0),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {LOCALEINDEX,
      IntOptionImpl.createOptionType("index", 0),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {FONT,
      ThreeStateOptionImpl.createOptionType("isLarge", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Display",
      Boolean.FALSE},
    {MONOSPACED,
      ThreeStateOptionImpl.createOptionType("isMonospaced", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Display",
      Boolean.FALSE},
    {MESSAGEPERSISTENCE,
      IntOptionImpl.createOptionType("timeSecond", 2),
      new Integer(OPTIONTYPE_INTEGER), "Display",
      Boolean.FALSE},
    {NODELISTWIDTH,
      IntOptionImpl.createOptionType("width", 400),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {NODELISTHEIGHT,
      IntOptionImpl.createOptionType("height", 150),
      new Integer(OPTIONTYPE_INTEGER),    "Display",
      Boolean.FALSE},
    {TARGET,
      StringOptionImpl.createOptionType("name", ""),
      new Integer(OPTIONTYPE_STRING),     "ProcessingInstructions",
      Boolean.TRUE},
    {DOCUMENTTYPE,
      DocumentTypeInfoImpl.optionType,
      new Integer(OPTIONTYPE_DOCUMENTTYPE), "DocumentTypes",
      Boolean.TRUE},
    {TRANSFORMER,
      TransformerInfoImpl.optionType,
      new Integer(OPTIONTYPE_TRANSFORMER), "Transformers",
      Boolean.TRUE},
    {OPERATOR,
      OperatorInfoImpl.optionType,
      new Integer(OPTIONTYPE_OPERATOR), "Operators",
      Boolean.TRUE},
    {PARAMETER,
      ParameterImpl.optionType,
      new Integer(OPTIONTYPE_PARAMETER),  "Parameters",
      Boolean.TRUE},
    {MENUITEM,
      MenuItemDataImpl.optionType,
      new Integer(OPTIONTYPE_MENUITEM),   "MenuItems",
      Boolean.TRUE},
    {NAMESPACE,
      NamespacePrefixPairImpl.optionType,
      new Integer(OPTIONTYPE_NAMESPACE),  "Namespaces",
      Boolean.TRUE},
    {OPERATIONSSTORED,
      ThreeStateOptionImpl.createOptionType("areStored", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Operation",
      Boolean.TRUE},
    {CLEARUNDOAFTERSAVE,
      ThreeStateOptionImpl.createOptionType("toDo", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Operation",
      Boolean.TRUE},
    {IDLENGTH,
      IntOptionImpl.createOptionType("length", 4),
      new Integer(OPTIONTYPE_INTEGER),    "Identification",
      Boolean.TRUE},
    {IDPREFIX,
      StringOptionImpl.createOptionType("prefix", "x"),
      new Integer(OPTIONTYPE_STRING),    "Identification",
      Boolean.TRUE},
    {ALLOWSCOMMENT,
      ThreeStateOptionImpl.createOptionType("toAllow", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "NodeTypes",
      Boolean.TRUE},
    {ALLOWSPROCESSINGINSTRUCTION,
      ThreeStateOptionImpl.createOptionType("toAllow", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "NodeTypes",
      Boolean.TRUE},
    {ALLOWSCDATASECTION,
      ThreeStateOptionImpl.createOptionType("toAllow", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "NodeTypes",
      Boolean.TRUE},
    {ALLOWSENTITYREFERENCE,
      ThreeStateOptionImpl.createOptionType("toAllow", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "NodeTypes",
      Boolean.TRUE},
    {LASTSEARCH,
      StringOptionImpl.createOptionType("value", ""),
      new Integer(OPTIONTYPE_STRING),     "Search",
      Boolean.FALSE},
    {CASEIGNOREDFORSEARCH,
      ThreeStateOptionImpl.createOptionType("isCaseIgnored", ThreeStateOption.TRUE),
      new Integer(OPTIONTYPE_THREESTATE), "Search",
      Boolean.FALSE},
    {ENTIRECONFORMANCEFORSEARCH,
      ThreeStateOptionImpl.createOptionType("isRequired", ThreeStateOption.FALSE),
      new Integer(OPTIONTYPE_THREESTATE), "Search",
      Boolean.FALSE},
    {LASTXPATHEXPRESSION,
      StringOptionImpl.createOptionType("value", ""),
      new Integer(OPTIONTYPE_STRING),   "Search",
      Boolean.FALSE},
    {HISTORYSIZE,
      IntOptionImpl.createOptionType("count", 20),
      new Integer(OPTIONTYPE_INTEGER),    "Search",
      Boolean.TRUE},
    {LOCATORLINENUMBER,
      IntOptionImpl.createOptionType("lineNumber", 1),
      new Integer(OPTIONTYPE_INTEGER),    "Search",
      Boolean.TRUE},
    {LOCATORCOLUMNNUMBER,
      IntOptionImpl.createOptionType("columnNumber", 1),
      new Integer(OPTIONTYPE_INTEGER),    "Search",
      Boolean.TRUE}};
  private static final Map fromOptionIdToOptionIndex = new HashMap();
  private final ArrayList[] optionLists =
      new ArrayList[optionDefinitions.length];

  static {
    for (int k = 0; k < optionDefinitions.length; k++) {
      Object[] optionDefinition = optionDefinitions[k];
      fromOptionIdToOptionIndex.put(optionDefinition[OPTIONDEF_ID],
        new Integer(k));
      if (optionDefinition.length != OPTIONDEF_COUNT) {
        System.err.println("Bad option definition : " + k);
        System.exit(1);
      }
    }
    for (int k = 0; k < Constants.encodingNames.length; k++) {
      String encoding = MIME2Java.convert(Constants.encodingNames[k]);
      if (encoding == null) {
        System.err.println("Unsupported encoding : "
          + Constants.encodingNames[k]);
        //System.exit(1);
      }
    }
  }

  public static final int getOptionIndex(String optionId) {
    Integer optionIndex = (Integer)fromOptionIdToOptionIndex.get(optionId);
    return optionIndex != null ? optionIndex.intValue() : -1;
  }

  public static final String getOptionId(int optionIndex) {
    return (String)optionDefinitions[optionIndex][OPTIONDEF_ID];
  }

  public static final OptionType getOptionType(int optionIndex) {
    return (OptionType)optionDefinitions[optionIndex][OPTIONDEF_TYPE];
  }

  public static final int getOptionTypeId(int optionIndex) {
    return ((Integer)optionDefinitions[optionIndex][OPTIONDEF_TYPEID]).
      intValue();
  }

  public static final List getOptionPackage(int optionIndex) {
    String packageString =
      (String)optionDefinitions[optionIndex][OPTIONDEF_PACKAGE];
    StringTokenizer tokenizer = new StringTokenizer(packageString);
    List elementList = new ArrayList();
    while (tokenizer.hasMoreTokens())
      elementList.add(tokenizer.nextToken());
    return elementList;
  }

  public static final boolean isAdvancedOption(int optionIndex) {
    return ((Boolean)optionDefinitions[optionIndex][OPTIONDEF_ADVANCED]).
      booleanValue();
  }

  public OptionsImpl() {
    for (int k = 0; k < optionLists.length; k++)
      optionLists[k] = new ArrayList(4);
  }

  public final Option getOption(int optionIndex) {
    List optionList = getOptionList(optionIndex);
    return optionList.size() >= 1 ? (Option)optionList.get(0) : null;
  }

  public final void setOption(int optionIndex, Option option) {
    if (option == null)
      throw new IllegalArgumentException();
    List optionList = getOptionList(optionIndex);
    if (optionList.isEmpty())
      optionList.add(option);
    else
      getOptionList(optionIndex).set(0, option);
  }

  /////////////////////////
  // Options implementation
  /////////////////////////

  public void clear() {
    for (int k = 0; k < optionLists.length; k++)
      optionLists[k].clear();
  }

  public final int getOptionDefCount() {
    return optionDefinitions.length;
  }

  public final List getOptionList(int optionIndex) {
    return optionLists[optionIndex];
  }

  public final List getOptionList(String optionId) {
    int optionIndex = getOptionIndex(optionId);
    return optionIndex >= 0 ? getOptionList(optionIndex) : null;
  }

  public final Option getOption(String optionId) {
    int optionIndex = getOptionIndex(optionId);
    return optionIndex >= 0 ? getOption(optionIndex) : null;
  }

  public final void setOption(String optionId, Option option) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex >= 0)
      setOption(optionIndex, option);
  }

  public String getStringOptionValue(String optionId) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return "";
    StringOption option = (StringOption)getOption(optionIndex);
    if (option != null) {
      String value = option.getStringValue();
      if (value.length() != 0)
        return value;
    }
    return ((StringOptionPartType)getOptionType(optionIndex).
      getOptionPartType(0)).getDefaultValue();
  }

  public final void setStringOptionValue(String optionId, String value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return;
    List optionList = getOptionList(optionIndex);
    if (optionList.isEmpty()) {
      StringOption option =
        (StringOption)getOptionType(optionIndex).createOption();
      option.setStringValue(value);
      optionList.add(option);
    }
    else
      ((StringOption)optionList.get(0)).setStringValue(value);
  }

  public final StringOption createStringOption(String optionId, String value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return null;
    StringOption option =
      (StringOption)getOptionType(optionIndex).createOption();
    option.setStringValue(value);
    return option;
  }

  public final int getIntOptionValue(String optionId) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return -1;
    IntOption option = (IntOption)getOption(optionIndex);
    return option != null ? option.getIntValue() :
      ((IntOptionPartType)getOptionType(optionIndex).
      getOptionPartType(0)).getDefaultValue();
  }

  public final void setIntOptionValue(String optionId, int value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return;
    List optionList = getOptionList(optionIndex);
    if (optionList.isEmpty()) {
      IntOption option = (IntOption)getOptionType(optionIndex).createOption();
      option.setIntValue(value);
      optionList.add(option);
    }
    else
      ((IntOption)optionList.get(0)).setIntValue(value);
  }

  public final IntOption createIntOption(String optionId, int value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return null;
    IntOption option = (IntOption)getOptionType(optionIndex).createOption();
    option.setIntValue(value);
    return option;
  }

  public final int getThreeStateOptionValue(String optionId) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return ThreeStateOption.UNDEFINED;
    ThreeStateOption option = (ThreeStateOption)getOption(optionIndex);
    return option != null ? option.getThreeStateValue() :
      ((ThreeStateOptionPartType)OptionsImpl.getOptionType(optionIndex).
      getOptionPartType(0)).getDefaultValue();
  }

  public final void setThreeStateOptionValue(String optionId, int value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return;
    List optionList = getOptionList(optionIndex);
    if (optionList.isEmpty()) {
      ThreeStateOption option =
        (ThreeStateOption)getOptionType(optionIndex).createOption();
      option.setThreeStateValue(value);
      optionList.add(option);
    }
    else
      ((ThreeStateOption)optionList.get(0)).setThreeStateValue(value);
  }

  public final ThreeStateOption createThreeStateOption(
    String optionId, int value) {
    int optionIndex = getOptionIndex(optionId);
    if (optionIndex < 0)
      return null;
    ThreeStateOption option =
      (ThreeStateOption)getOptionType(optionIndex).createOption();
    option.setThreeStateValue(value);
    return option;
  }
}
