/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.BooleanOptionPart;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.document.doctype.RNGInfo;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.option.part.BooleanOptionPartType;
import org.xmloperator.option.part.BooleanOptionPartImpl;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class DocumentTypeInfoImpl extends OptionImpl
  implements DocumentTypeInfo, DTDInfo, RNGInfo, Comparable {
  private static final String XMLOPERATOR_NAMESPACE_URI =
    "http://www.xmloperator.net/namespace/";
  // Persistent content structure
  private static final int NAME = 0;
  private static final int KEY = 1;
  private static final int DTDPUBLICID = 2;
  private static final int HTTPDTDSYSTEMID = 3;
  private static final int DOCUMENTRELATIVEDTDSYSTEMID = 4;
  private static final int USERRELATIVEDTDSYSTEMID = 5;
  private static final int DTDXMLSYSTEMID = 6;
  private static final int FULLRNGSYSTEMID = 7;
  private static final int SIMPLERNGSYSTEMID = 8;
  private static final int NAMESPACEID = 9;
  private static final int ISEMPTYELEMENTSHORTCUTUSED = 10;
  private static final int TRANSFORMERNAME = 11;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "name",
        "key",
        "dTDPublicId",
        "hTTPDTDSystemId",
        "documentRelativeDTDSystemId",
        "userRelativeDTDSystemId",
        "dTDXMLSystemId",
        "fullRNGSystemId",
        "simpleRNGSystemId",
        "namespaceId",
        "isEmptyElementShortcutUsed",
        "transformerName"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new BooleanOptionPartType(false),
        new StringOptionPartType("")}) {
      public Option createOption() {
        return new DocumentTypeInfoImpl();
      }
    };

  public static final void initializeListIfEmpty(Options options, List list) {
    if (!list.isEmpty())
      return;
    final String rNGPath = options.getStringOptionValue(Options.RNGPATH);
    final String dTDPath = options.getStringOptionValue(Options.DTDPATH);
    list.add(new DocumentTypeInfoImpl(Constants.TOOL, Constants.TOOL,
      rNGPath + Constants.TOOL_FILE + "_options.rng",
      rNGPath + Constants.TOOL_FILE + "_options_s.rng",
      null, true, null));
    list.add(new DocumentTypeInfoImpl("operations", Constants.TOOL,
      rNGPath + Constants.TOOL_FILE + "_operations.rng",
      rNGPath + Constants.TOOL_FILE + "_operations_s.rng",
      null, true, null));
    list.add(new DocumentTypeInfoImpl("batch", Constants.TOOL,
      rNGPath + Constants.TOOL_FILE + "_batch.rng",
      rNGPath + Constants.TOOL_FILE + "_batch_s.rng",
      null, true, null));
    list.add(new DocumentTypeInfoImpl("dtd", Constants.TOOL,
      "-//xmloperator//DTD DTDXML 1.0//EN//XML", null,
      "dtdxml_1_0.dtd", dTDPath + "dtdxml_1_0.dtd",
      dTDPath + "dtdxml_1_0_dtd.xml",
      null, null,
      XMLOPERATOR_NAMESPACE_URI + "dtdxml", true,
      "DtdToHtml"));
    list.add(new DocumentTypeInfoImpl("schema", Constants.TOOL,
      rNGPath + "schema_1_3.rng", rNGPath + "schema_1_3_s.rng",
      XMLOPERATOR_NAMESPACE_URI + "schema", true,
      "SchemaToHtml"));
    /* Deprecated by JavaSkeleton
    list.add(new DocumentTypeInfoImpl("depend", Constants.TOOL,
      rNGPath + "depend_1_3.rng", rNGPath + "depend_1_3_s.rng",
      XMLOPERATOR_NAMESPACE_URI + "depend", true,
      "DependToHtml"));
    */
    list.add(new DocumentTypeInfoImpl("change", Constants.TOOL,
      rNGPath + "change_1_2.rng", rNGPath + "change_1_2_s.rng",
      XMLOPERATOR_NAMESPACE_URI + "change", true,
      null));
    list.add(new DocumentTypeInfoImpl("javaSkeleton", Constants.TOOL,
      rNGPath + "javaSkeleton_1_0.rng", rNGPath + "javaSkeleton_1_0_s.rng",
      XMLOPERATOR_NAMESPACE_URI + "java", true,
      "JavaSkeletonToHtml"));
    list.add(new DocumentTypeInfoImpl("grammar", "relaxng_grammar",
      rNGPath + "relaxng_1_0.rng", rNGPath + "relaxng_1_0_s.rng",
      "http://relaxng.org/ns/structure/1.0", true,
      "RelaxNGToHtml"));
    list.add(new DocumentTypeInfoImpl("element", "relaxng_element",
      rNGPath + "relaxng_1_0.rng", rNGPath + "relaxng_1_0_s.rng",
      "http://relaxng.org/ns/structure/1.0", true,
      "RelaxNGToHtml"));
    list.add(new DocumentTypeInfoImpl("", "relaxng",
      rNGPath + "relaxng_1_0.rng", rNGPath + "relaxng_1_0_s.rng",
      "http://relaxng.org/ns/structure/1.0", true,
      "RelaxNGToHtml"));
    list.add(new DocumentTypeInfoImpl("xsl:stylesheet", "xslt_stylesheet",
      rNGPath + "xslt.rng", rNGPath + "xslt_s.rng",
      "http://www.w3.org/1999/XSL/Transform", true, null));
    list.add(new DocumentTypeInfoImpl("xsl:transform", "xslt_transform",
      rNGPath + "xslt.rng", rNGPath + "xslt_s.rng",
      "http://www.w3.org/1999/XSL/Transform", true, null));
    list.add(new DocumentTypeInfoImpl("", "xslt",
      rNGPath + "xslt.rng", rNGPath + "xslt_s.rng",
      "http://www.w3.org/1999/XSL/Transform", true, null));
    list.add(new DocumentTypeInfoImpl("", "any",
      rNGPath + "any.rng", rNGPath + "any_s.rng",
      null, true, null));
    list.add(new DocumentTypeInfoImpl("", "internal_dtd",
      null, null, null, null, null, null, null, null, true, null));
    list.add(new DocumentTypeInfoImpl("html", "xhtml-strict",
      rNGPath + "xhtml/xhtml-strict.rng", rNGPath + "xhtml/xhtml-strict_s.rng",
      "http://www.w3.org/1999/xhtml", false,
      null));
    list.add(new DocumentTypeInfoImpl("html", "xhtml-strict-charEntities",
      null, null,
      "xhtml-charEntities.dtd", dTDPath + "data/xhtml-charEntities.dtd", null,
      rNGPath + "xhtml/xhtml-strict.rng", rNGPath + "xhtml/xhtml-strict_s.rng",
      "http://www.w3.org/1999/xhtml", false,
      null));
    list.add(new DocumentTypeInfoImpl("html", "xhtml-basic",
      rNGPath + "xhtml/xhtml-basic.rng", rNGPath + "xhtml/xhtml-basic_s.rng",
      "http://www.w3.org/1999/xhtml", false,
      null));
    list.add(new DocumentTypeInfoImpl("html", "xhtml-rddl",
      rNGPath + "rddl/xhtml-rddl.rng", rNGPath + "rddl/xhtml-rddl_s.rng",
      "http://www.w3.org/1999/xhtml", false,
      null));
    list.add(new DocumentTypeInfoImpl("svg", "svg",
      rNGPath + "svg/svg.rng", rNGPath + "svg/svg_s.rng",
      "http://www.w3.org/2000/svg", false,
      null));
    list.add(new DocumentTypeInfoImpl("", "svg",
      rNGPath + "svg/svg.rng", rNGPath + "svg/svg_s.rng",
      "http://www.w3.org/2000/svg", false,
      null));
    list.add(new DocumentTypeInfoImpl("math", "mathml2",
      rNGPath + "mathml2/mathml2.rng", rNGPath + "mathml2/mathml2_s.rng",
      "http://www.w3.org/1998/Math/MathML", false,
      null));
    list.add(new DocumentTypeInfoImpl("schema", "w3c_xml_schema",
      rNGPath + "xmlshema.rng", rNGPath + "xmlschema_s.rng",
      "http://www.w3.org/2001/XMLSchema", false, null));
    list.add(new DocumentTypeInfoImpl("project", "ant",
      rNGPath + "ant_project_1_5_3.rng", rNGPath + "ant_project_1_5_3_s.rng",
      null, false, null));
  }

  public DocumentTypeInfoImpl() {
    super(optionType);
  }

  public DocumentTypeInfoImpl(
    String documentTypeName, String documentTypeAdditionalKey) {
    this();
    if (documentTypeName == null || documentTypeAdditionalKey == null)
      throw new IllegalArgumentException();
    setDocumentTypeName(documentTypeName);
    setDocumentTypeAdditionalKey(documentTypeAdditionalKey);
  }

  public DocumentTypeInfoImpl(
    String documentTypeName,
    String documentTypeAdditionalKey,
    String rNGFullSyntaxSystemId, String rNGSimpleSyntaxSystemId,
    String namespaceId, boolean isShortcutUsedForEmptyElement,
    String defaultTransformerName) {
    this(documentTypeName, documentTypeAdditionalKey);
    setRNGFullSyntaxSystemId(rNGFullSyntaxSystemId);
    setRNGSimpleSyntaxSystemId(rNGSimpleSyntaxSystemId);
    setNamespaceId(namespaceId);
    setShortcutUsedForEmptyElement(isShortcutUsedForEmptyElement);
    setDefaultTransformerName(defaultTransformerName);
  }

  public DocumentTypeInfoImpl(
    String documentTypeName,
    String documentTypeAdditionalKey,
    String dTDPublicId, String hTTPDTDSystemId,
    String documentRelativeDTDSystemId, String userRelativeDTDSystemId,
    String dTDXMLSystemId,
    String rNGFullSyntaxSystemId, String rNGSimpleSyntaxSystemId,
    String namespaceId, boolean isShortcutUsedForEmptyElement,
    String defaultTransformerName) {
  this(documentTypeName, documentTypeAdditionalKey,
       rNGFullSyntaxSystemId, rNGSimpleSyntaxSystemId,
       namespaceId, isShortcutUsedForEmptyElement, defaultTransformerName);
    setDTDPublicId(dTDPublicId);
    setHTTPDTDSystemId(hTTPDTDSystemId);
    setDocumentRelativeDTDSystemId(documentRelativeDTDSystemId);
    setUserRelativeDTDSystemId(userRelativeDTDSystemId);
    setDTDXMLSystemId(dTDXMLSystemId);
  }

  public DocumentTypeInfoImpl(DocumentTypeInfo documentTypeInfo) {
    this(
      documentTypeInfo.getDocumentTypeName(),
      documentTypeInfo.getDocumentTypeAdditionalKey(),
      documentTypeInfo.getDTDInfo().getDTDPublicId(),
      documentTypeInfo.getDTDInfo().getHTTPDTDSystemId(),
      documentTypeInfo.getDTDInfo().getDocumentRelativeDTDSystemId(),
      documentTypeInfo.getDTDInfo().getUserRelativeDTDSystemId(),
      documentTypeInfo.getDTDInfo().getDTDXMLSystemId(),
      documentTypeInfo.getRNGInfo().getRNGFullSyntaxSystemId(),
      documentTypeInfo.getRNGInfo().getRNGSimpleSyntaxSystemId(),
      documentTypeInfo.getNamespaceId(),
      documentTypeInfo.isShortcutUsedForEmptyElement(),
      documentTypeInfo.getDefaultTransformerName());
  }

  /////////////////////
  // Object overloading
  /////////////////////

  public boolean equals(Object object) {
    return (object != null)
      && (object instanceof DocumentTypeInfo)
      && getDocumentTypeName().equals(
      ((DocumentTypeInfo)object).getDocumentTypeName())
      && getDocumentTypeAdditionalKey().equals(
      ((DocumentTypeInfo)object).getDocumentTypeAdditionalKey());
  }

  public String toString() {
    return getDocumentTypeName() + '(' + getDocumentTypeAdditionalKey() + ')';
  }

  ////////////////////////////
  // Comparable implementation
  ////////////////////////////

  public int compareTo(Object other) {
    if (other == null || !(other instanceof DocumentTypeInfo))
      throw new IllegalArgumentException();
    String selfDocumentTypeName = this.getDocumentTypeName();
    String otherDocumentTypeName =
        ((DocumentTypeInfo)other).getDocumentTypeName();
    if (selfDocumentTypeName.equals(otherDocumentTypeName))
      return getDocumentTypeAdditionalKey().compareTo(
        ((DocumentTypeInfo)other).getDocumentTypeAdditionalKey());
    if (selfDocumentTypeName.length() == 0)
      return +1;
    if (otherDocumentTypeName.length() == 0)
      return -1;
    return selfDocumentTypeName.compareTo(otherDocumentTypeName);
  }

  /////////////////////////
  // DTDInfo implementation
  /////////////////////////

  public String getDTDPublicId() {
    return ((StringOptionPartImpl)getOptionPart(DTDPUBLICID)).getStringValue();
  }

  public void setDTDPublicId(String dTDPublicId) {
    ((StringOptionPartImpl)getOptionPart(DTDPUBLICID)).setStringValue(
      dTDPublicId != null ? dTDPublicId : "");
  }

  public String getHTTPDTDSystemId() {
    return ((StringOptionPartImpl)getOptionPart(HTTPDTDSYSTEMID)).
      getStringValue();
  }

  public void setHTTPDTDSystemId(String hTTPDTDSystemId) {
    ((StringOptionPartImpl)getOptionPart(HTTPDTDSYSTEMID)).setStringValue(
      hTTPDTDSystemId != null ? hTTPDTDSystemId : "");
  }

  public String getDocumentRelativeDTDSystemId() {
    return ((StringOptionPartImpl)getOptionPart(DOCUMENTRELATIVEDTDSYSTEMID)).
      getStringValue();
  }

  public void setDocumentRelativeDTDSystemId(
    String documentRelativeDTDSystemId) {
    ((StringOptionPartImpl)getOptionPart(DOCUMENTRELATIVEDTDSYSTEMID)).
      setStringValue(documentRelativeDTDSystemId != null ?
      documentRelativeDTDSystemId : "");
    if (documentRelativeDTDSystemId != null
      && documentRelativeDTDSystemId.startsWith("http://"))
      setHTTPDTDSystemId(documentRelativeDTDSystemId);
  }

  public String getUserRelativeDTDSystemId() {
    return ((StringOptionPartImpl)getOptionPart(USERRELATIVEDTDSYSTEMID)).
      getStringValue();
  }

  public void setUserRelativeDTDSystemId(String userRelativeDTDSystemId) {
    ((StringOptionPartImpl)getOptionPart(USERRELATIVEDTDSYSTEMID)).
      setStringValue(userRelativeDTDSystemId != null ?
      userRelativeDTDSystemId : "");
  }

  public String getDTDXMLSystemId() {
    return ((StringOptionPartImpl)getOptionPart(DTDXMLSYSTEMID)).
      getStringValue();
  }

  public void setDTDXMLSystemId(String dTDXMLSystemId) {
    ((StringOptionPartImpl)getOptionPart(DTDXMLSYSTEMID)).
      setStringValue(dTDXMLSystemId != null ? dTDXMLSystemId : "");
  }

  /////////////////////////
  // RNGInfo implementation
  /////////////////////////

  public String getRNGFullSyntaxSystemId() {
    return ((StringOptionPartImpl)getOptionPart(FULLRNGSYSTEMID)).
      getStringValue();
  }

  public void setRNGFullSyntaxSystemId(String systemId) {
    ((StringOptionPartImpl)getOptionPart(FULLRNGSYSTEMID)).
      setStringValue(systemId != null ? systemId : "");
  }

  public String getRNGSimpleSyntaxSystemId() {
    return ((StringOptionPartImpl)getOptionPart(SIMPLERNGSYSTEMID)).
      getStringValue();
  }

  public void setRNGSimpleSyntaxSystemId(String systemId) {
    ((StringOptionPartImpl)getOptionPart(SIMPLERNGSYSTEMID)).
      setStringValue(systemId != null ? systemId : "");
  }

  //////////////////////////////////
  // DocumentTypeInfo implementation
  //////////////////////////////////

  public final boolean isValid() {
    return getDocumentTypeName().length() != 0;
  }

  public final String getDocumentTypeName() {
    return ((StringOptionPartImpl)getOptionPart(NAME)).getStringValue();
  }

  public final void setDocumentTypeName(String documentTypeName) {
    if (documentTypeName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(NAME)).
      setStringValue(documentTypeName);
  }

  public final String getDocumentTypeAdditionalKey() {
    return ((StringOptionPartImpl)getOptionPart(KEY)).getStringValue();
  }

  public final void setDocumentTypeAdditionalKey(
    String documentTypeAdditionalKey) {
    if (documentTypeAdditionalKey == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(KEY)).
      setStringValue(documentTypeAdditionalKey);
  }

  public final DTDInfo getDTDInfo() {
    return (DTDInfo)this;
  }

  public final RNGInfo getRNGInfo() {
    return (RNGInfo)this;
  }

  public final String getNamespaceId() {
    return ((StringOptionPartImpl)getOptionPart(NAMESPACEID)).getStringValue();
  }

  public final void setNamespaceId(String namespaceId) {
    ((StringOptionPartImpl)getOptionPart(NAMESPACEID)).
      setStringValue(namespaceId != null ? namespaceId : "");
  }

  public final boolean isShortcutUsedForEmptyElement() {
    return ((BooleanOptionPartImpl)getOptionPart(ISEMPTYELEMENTSHORTCUTUSED)).
      getBooleanValue();
  }

  public final void setShortcutUsedForEmptyElement(
    boolean isShortcutUsedForEmptyElement) {
    ((BooleanOptionPartImpl)getOptionPart(ISEMPTYELEMENTSHORTCUTUSED)).
      setBooleanValue(isShortcutUsedForEmptyElement);
  }

  public final String getDefaultTransformerName() {
    return ((StringOptionPartImpl)getOptionPart(TRANSFORMERNAME)).
      getStringValue();
  }

  public final void setDefaultTransformerName(String transformerName) {
    ((StringOptionPartImpl)getOptionPart(TRANSFORMERNAME)).
      setStringValue(transformerName != null ? transformerName : "");
  }
}
