/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.IntOptionPart;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.util.ResourceContext;
import org.xmloperator.option.part.IntOptionPartType;
import org.xmloperator.option.part.IntOptionPartImpl;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class ResourceContextImpl extends OptionImpl
  implements ResourceContext {
  private static final int SYSTEMID = 0;
  private static final int KEY = 1;
  private static final int ELEMENTPATH = 2;
  private static final int NODEINDEX = 3;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "systemId",
        "key",
        "elementPath",
        "nodeIndex"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new IntOptionPartType(0)}) {
      public Option createOption() {
        return new ResourceContextImpl();
      }
    };

  public static final void initializeListIfEmpty(Options options, List list) {
    if (!list.isEmpty())
      return;
    final String xMLPath = options.getStringOptionValue(Options.XMLPATH);
    //list.add(new ResourceContextImpl(xMLPath + "demo.xml", Constants.TOOL));
  }

  private ResourceContextImpl() {
    super(optionType);
  }

  public ResourceContextImpl(String documentSystemId) {
    this(documentSystemId, null, "", -1);
  }

  public ResourceContextImpl(String documentSystemId,
    String documentTypeAdditionalKey) {
    this(documentSystemId, documentTypeAdditionalKey, "", -1);
  }

  public ResourceContextImpl(String documentSystemId,
    String documentTypeAdditionalKey,
    String selectedElementPath, int selectedNodeIndex) {
    this();
    setDocumentSystemId(documentSystemId);
    setDocumentTypeAdditionalKey(documentTypeAdditionalKey);
    setSelectedElementPath(selectedElementPath);
    setSelectedNodeIndex(selectedNodeIndex);
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return (object != null) && object instanceof ResourceContextImpl
      && getDocumentSystemId().equals(
      ((ResourceContextImpl)object).getDocumentSystemId());
  }

  public String toString() {
    return getDocumentSystemId();
  }

  /////////////////////////////////
  // ResourceContext implementation
  /////////////////////////////////

  public boolean isValid() {
    return getDocumentSystemId().length() != 0;
  }

  public String getDocumentSystemId() {
    return ((StringOptionPartImpl)getOptionPart(SYSTEMID)).getStringValue();
}

  public void setDocumentSystemId(String documentSystemId) {
    ((StringOptionPartImpl)getOptionPart(SYSTEMID)).
      setStringValue(documentSystemId != null ? documentSystemId : "");
  }

  public String getDocumentTypeAdditionalKey() {
    return ((StringOptionPartImpl)getOptionPart(KEY)).getStringValue();
  }

  public void setDocumentTypeAdditionalKey(String documentTypeAdditionalKey) {
    ((StringOptionPartImpl)getOptionPart(KEY)).setStringValue(
      documentTypeAdditionalKey != null ? documentTypeAdditionalKey : "");
  }

  public String getSelectedElementPath() {
    return ((StringOptionPartImpl)getOptionPart(ELEMENTPATH)).getStringValue();
  }

  public void setSelectedElementPath(String selectedElementPath) {
    ((StringOptionPartImpl)getOptionPart(ELEMENTPATH)).setStringValue(
      selectedElementPath != null ? selectedElementPath : "");
  }

  public int getSelectedNodeIndex() {
    return ((IntOptionPartImpl)getOptionPart(NODEINDEX)).getIntValue();
  }

  public void setSelectedNodeIndex(int selectedNodeIndex) {
    ((IntOptionPartImpl)getOptionPart(NODEINDEX)).
      setIntValue(selectedNodeIndex);
  }
}
