/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import java.util.Arrays;
import java.util.StringTokenizer;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

public final class ComparableNode implements Comparable {

  public static final boolean areNodesElementsWithTheSameName(Node[] nodes) {
    String elementName = nodes[0].getNodeName();
    for (int k = 0; k < nodes.length; k++) {
      Node node = nodes[k];
      if (node.getNodeType() != Node.ELEMENT_NODE
        || !node.getNodeName().equals(elementName))
        return false;
    }
    return true;
  }

  public static final void reorderElements(Node[] elements,
    String attributeNames, boolean isReverse) {
    ComparableNode[] comparableNodes = new ComparableNode[elements.length];
    for (int k = 0; k < elements.length; k++) {
      Element element = (Element)elements[k];
      StringBuffer sb = new StringBuffer();
      StringTokenizer tokenizer = new StringTokenizer(attributeNames, " ,/");
      while (tokenizer.hasMoreTokens()) {
        sb.append(element.getAttribute(tokenizer.nextToken()));
        sb.append('~');
      }
      comparableNodes[k] =
        new ComparableNode(elements[k], sb.toString(), isReverse);
    }
    Arrays.sort(comparableNodes);
    for (int k = 0; k < elements.length; k++)
      elements[k] = comparableNodes[k].node;
  }

  private Node node;
  private String value;
  private boolean isReverse;

  private ComparableNode(Node node, String value, boolean isReverse) {
    this.node = node;
    this.value = value;
    this.isReverse = isReverse;
  }

  public boolean equals(Object o) {
    return o instanceof ComparableNode ?
      value.equals(((ComparableNode)o).value) : false;
  }

  public int compareTo(Object o) {
    return isReverse ?
      ((ComparableNode)o).value.compareTo(value) :
      value.compareTo(((ComparableNode)o).value);
  }
}

