/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import java.io.PrintWriter;

import org.xmloperator.framework.document.util.DomWriterReport;

public abstract class DomWriter implements DomWriterReport {

  /**
   * Translate a serialisation report from int to string.
   */
  public static final String fromReportCodeToReplacedStringList(int reportCode){
    StringBuffer sb = new StringBuffer();
    if ((reportCode & REPLACED_AMPERSAND) != 0)
      sb.append("\"&\" ");
    if ((reportCode & REPLACED_LESSTHAN) != 0)
      sb.append("\"<\" ");
    if ((reportCode & REPLACED_GREATERTHAN) != 0)
      sb.append("\">\" ");
    if ((reportCode & REPLACED_DOUBLEQUOTE) != 0)
      sb.append("'\"' ");
    if ((reportCode & REPLACED_FORBIDENCOMMENT) != 0)
      sb.append("\"--\" ");
    return sb.toString();
  }

  /**
   * Print a text node given its value.
   * Return a serialisation report as sum of REPLACED_*
   */
  public static final int printText(PrintWriter out, String value) {
    int returnCode = 0;
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      switch (ch) {
        case '&':
          out.print("&amp;");
          returnCode |= REPLACED_AMPERSAND;
          break;
        case '<':
          out.print("&lt;");
          returnCode |= REPLACED_LESSTHAN;
          break;
        case '>':
          out.print("&gt;");
          returnCode |= REPLACED_GREATERTHAN;
          break;
        case '\r':
          break;
        case '\n':
          out.println();
          break;
        default: {
          out.print(ch);
        }
      }
    }
    return returnCode;
  }

  /**
   * Print an attribute, name and value.
   * Return a serialisation report as sum of REPLACED_*
   */
  public static final int printAttribute(
      PrintWriter out, String name, String value) {
    out.print(' ');
    out.print(name);
    out.print('=');
    return printAttributeValue(out, value);
  }

  /**
   * Print an attribute value.
   * Return a serialisation report as sum of REPLACED_*
   */
  private static final int printAttributeValue(PrintWriter out, String value) {
    out.print('"');
    int returnCode = 0;
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      switch (ch) {
        case '&':
          out.print("&amp;");
          returnCode |= REPLACED_AMPERSAND;
          break;
        case '<':
          out.print("&lt;");
          returnCode |= REPLACED_LESSTHAN;
          break;
        case '>':
          out.print("&gt;");
          returnCode |= REPLACED_GREATERTHAN;
          break;
        case '"':
          out.print("&quot;");
          returnCode |= REPLACED_DOUBLEQUOTE;
          break;
        case '\r':
          break;
        case '\n':
          out.println();
          break;
        default: {
          out.print(ch);
        }
      }
    }
    out.print('"');
    return returnCode;
  }

  /**
   * Print a comment node given its value.
   * Return a serialisation report as sum of REPLACED_*
   */
  public static final int printComment(PrintWriter out, String value) {
    int returnCode = 0;
    out.print("<!--");
    while(true) {
      int forbidenContentIndex = value.indexOf("--");
      if (forbidenContentIndex < 0)
        break;
      // Replace "--" by "- -"
      forbidenContentIndex++;
      value = value.substring(0, forbidenContentIndex) + " "
        + value.substring(forbidenContentIndex);
      returnCode |= REPLACED_FORBIDENCOMMENT;
    }
    printValue(out, value);
    out.print("-->");
    return returnCode;
  }

  /**
   * Print a processing instruction, given its name and value.
   * Return a serialisation report as sum of REPLACED_*
   */
  public static final int printProcessingInstruction(
      PrintWriter out, String name, String value) {
    out.print("<?");
    out.print(name);
    out.print(' ');
    int returnCode = printProcessingInstructionValue(out, value);
    out.print("?>");
    return returnCode;
  }

  /**
   * Print a processing instruction value.
   * Return a serialisation report as sum of REPLACED_*
   */
  private static final int printProcessingInstructionValue(
    PrintWriter out, String value) {
    int returnCode = 0;
    while(true) {
      int forbidenContentIndex = value.indexOf("?>");
      if (forbidenContentIndex < 0)
        break;
      // Replace "?>" by "?&gt;"
      forbidenContentIndex++;
      value = value.substring(0, forbidenContentIndex) + "&gt;"
        + value.substring(forbidenContentIndex + 1);
      returnCode |= REPLACED_GREATERTHAN;
    }
    printValue(out, value);
    return returnCode;
  }

  /**
   * Print a cdata section node given its value.
   * Return a serialisation report as sum of REPLACED_*
   */
  public static final int printCdataSection(
    PrintWriter out, String value) {
    int returnCode = 0;
    out.print("<![CDATA[");
    while(true) {
      int forbidenContentIndex = value.indexOf("]]>");
      if (forbidenContentIndex < 0)
        break;
      // Replace "]]>" by "]]&gt;"
      forbidenContentIndex+= 2;
      value = value.substring(0, forbidenContentIndex) + "&gt;"
        + value.substring(forbidenContentIndex + 1);
      returnCode |= REPLACED_GREATERTHAN;
    }
    printValue(out, value);
    out.print("]]>");
    return returnCode;
  }

  private static final void printValue(PrintWriter out, String value) {
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      switch (ch) {
        case '\r':
          break;
        case '\n':
          out.println();
          break;
        default: {
          out.print(ch);
        }
      }
    }
  }
}
