/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

public abstract class ElementBrowser {

  ///////////////////
  // Element existing
  ///////////////////

  public static final boolean hasChildElements(final Element parent) {
    return getFirstChildElement(parent) != null;
  }

  public static final boolean hasChildElements(final Element parent,
      final String childName) {
    return getFirstChildElement(parent, childName) != null;
  }

  ////////////////
  // Element count
  ////////////////

  public static final int getChildElementCount(final Element parent) {
    int count = 0;
    Element childElement = getFirstChildElement(parent);
    while (childElement != null) {
      count++;
      childElement = getNextSiblingElement(childElement);
    }
    return count;
  }

  public static final int getChildElementCount(final Element parent,
    String childName) {
    int count = 0;
    Element childElement = getFirstChildElement(parent);
    while (childElement != null) {
      if (childName == null || childElement.getNodeName().equals(childName))
        count++;
      childElement = getNextSiblingElement(childElement);
    }
    return count;
  }

  ///////////////////
  // Element browsing
  ///////////////////

  public static final Element getFirstChildElement(final Element parent) {
    return getNextElement(parent.getFirstChild());
  }

  public static final Element getFirstChildElement(final Element parent,
    final String elementName) {
    return getNextElement(parent.getFirstChild(), elementName);
  }

  public static final Element getFirstChildElement(final Element parent,
    final String[] elementNames) {
    return getNextElement(parent.getFirstChild(), elementNames);
  }

  public static final Element getLastChildElement(final Element parent) {
    return getPreviousElement(parent.getLastChild());
  }

  public static final Element getLastChildElement(final Element parent,
    String elementName) {
    return getPreviousElement(parent.getLastChild(), elementName);
  }

  public static final Element getLastChildElement(final Element parent,
    final String[] elementNames) {
    return getPreviousElement(parent.getLastChild(), elementNames);
  }

  public static final Element getNextSiblingElement(final Element element) {
    return getNextElement(element.getNextSibling());
  }

  public static final Element getNextSiblingElement(final Element element,
    final String elementName) {
    return getNextElement(element.getNextSibling(), elementName);
  }

  public static final Element getNextSiblingElement(final Element element,
    final String[] elementNames) {
    return getNextElement(element.getNextSibling(), elementNames);
  }

  public static final Element getPreviousSiblingElement(final Element element) {
    return getPreviousElement(element.getPreviousSibling());
  }

  public static final Element getPreviousSiblingElement(final Element element,
    final String elementName) {
    return getPreviousElement(element.getPreviousSibling(), elementName);
  }

  public static final Element getPreviousSiblingElement(final Element element,
    String[] elementNames) {
    return getPreviousElement(element.getPreviousSibling(), elementNames);
  }

  public static final Element getNextElement(Node node) {
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE)
        return (Element)node;
      node = node.getNextSibling();
    }
    return null;
  }

  public static final Element getNextElement(Node node,
      final String elementName) {
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE
        && (elementName == null || node.getNodeName().equals(elementName)))
        return (Element)node;
      node = node.getNextSibling();
    }
    return null;
  }

  public static final Element getNextElement(Node node,
      final String[] elementNames) {
    noNull(elementNames);
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE)
        for (int k = 0; k < elementNames.length; k++)
          if (node.getNodeName().equals(elementNames[k]))
            return (Element)node;
      node = node.getNextSibling();
    }
    return null;
  }

  public static final Element getPreviousElement(Node node) {
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE)
        return (Element)node;
      node = node.getPreviousSibling();
    }
    return null;
  }

  public static final Element getPreviousElement(Node node, final String elementName)
  {
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE
        && (elementName == null || node.getNodeName().equals(elementName)))
        return (Element)node;
      node = node.getPreviousSibling();
    }
    return null;
  }

  public static final Element getPreviousElement(Node node,
    final String[] elementNames) {
    noNull(elementNames);
    while (node != null) {
      if (node.getNodeType() == Node.ELEMENT_NODE)
        for (int k = 0; k < elementNames.length; k++)
          if (node.getNodeName().equals(elementNames[k]))
            return (Element)node;
      node = node.getPreviousSibling();
    }
    return null;
  }

  private static final void noNull(final String[] elementNames) {
    for (int k = 0; k < elementNames.length; k++)
      if (elementNames[k] == null)
        throw new IllegalArgumentException();
  }

  /////////////////////////////
  // Elements within node array
  /////////////////////////////

  public static final boolean hasElements(final Node[] nodes) {
    for (int k = 0; k < nodes.length; k++)
      if (nodes[k].getNodeType() == Node.ELEMENT_NODE)
        return true;
    return false;
  }

  public static final Element getFirstElement(final Node[] nodes) {
    for (int k = 0; k < nodes.length; k++) {
      Node node = nodes[k];
      if (node.getNodeType() == Node.ELEMENT_NODE)
        return (Element)node;
    }
    return null;
  }

  public static final Element getLastElement(final Node[] nodes) {
    for (int k = nodes.length - 1; k >= 0; k++) {
      Node node = nodes[k];
      if (node.getNodeType() == Node.ELEMENT_NODE)
        return (Element)node;
    }
    return null;
  }
}
