/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.exception;

import java.util.Enumeration;
import java.util.Date;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Reader;
import java.io.StringReader;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.xmloperator.framework.control.task.Action;

public abstract class ExceptionControler {
  public static String editor_email = null;
  public static String software = null;
  private static boolean isTraceStarted = false;
  private static boolean hasExceptionOccured = false;
  private static StringWriter stringWriter = new StringWriter();

  public static boolean hasExceptionOccured() {
    return hasExceptionOccured;
  }

  public static void acceptException() {
    hasExceptionOccured = false;
  }

  public static Reader getTraceReader() {
    String trace = stringWriter.toString();
    return new StringReader(trace);
  }

  public static void println(String message) {
    PrintWriter out = out();
    out.println(message);
    out.close();
  }

  public static void println(Enumeration messages) {
    PrintWriter out = out();
    while (messages.hasMoreElements())
      out.println(messages.nextElement());
    out.close();
  }

  public static void printStackTrace(Exception exception) {
    PrintWriter out = out();
    exception.printStackTrace(out);
    out.close();
    hasExceptionOccured = true;
  }

  public static void printStackTrace(Exception exception, String message) {
    PrintWriter out = out();
    out.println(message);
    exception.printStackTrace(out);
    out.close();
    hasExceptionOccured = true;
  }

  private static PrintWriter out() {
    PrintWriter out = new PrintWriter(stringWriter);
    if (!isTraceStarted) {
      if (editor_email != null)
        out.println("PLEASE SEND A COPY OF THIS TRACE TO " + editor_email);
      if (software != null)
        out.println(software);
      out.println(new Date().toString());
      isTraceStarted = true;
    }
    return out;
  }

  /**
   * Translates an action to an actionListener by catching any exception.
   */
  public static final ActionListener control(final Action action) {
    return new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        try {
          action.execute(event);
        }
        catch (Exception exception) {
          ExceptionControler.printStackTrace(exception);
        }
      }
    };
  }
}
