/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.io;

import java.io.File;

import org.xmloperator.util.io.URI;

public abstract class FileUtils {

  public static final String normalisePathName(String pathName) {
    pathName = pathName.replace(java.io.File.separatorChar, '/');
    // Windows fix
    if (pathName.length() >= 2 && pathName.charAt(1) == ':') {
      char ch0 = Character.toUpperCase(pathName.charAt(0));
      if (ch0 >= 'A' && ch0 <= 'Z')
        pathName = "/" + pathName;
    }
    return pathName;
  }

  public static final String unnormalisePathName(String pathName) {
    // Windows fix
    if (pathName.length() >= 3
      && pathName.charAt(0) == '/' && pathName.charAt(2) == ':') {
      char ch0 = Character.toUpperCase(pathName.charAt(1));
      if (ch0 >= 'A' && ch0 <= 'Z')
        pathName = pathName.substring(1);
    }
    pathName = pathName.replace('/', java.io.File.separatorChar);
    return pathName;
  }

  public static final String userDir() {
    String dir;
    try {
      dir = normalisePathName(System.getProperty("user.dir"));
    }
    catch (SecurityException exception) {
      dir = "";
    }
    if (!dir.endsWith("/"))
      dir = dir + "/";
    return dir;
  }

  public static final String fromFilePathNameToSystemId(String pathName) {
    return "file://" + normalisePathName(pathName);
  }

  public static final String fromSystemIdToFilePathName(String systemId) {
    if (systemId == null)
      return null;
    try {
      URI uRI = new URI(systemId);
      if (uRI.getScheme().equals("file"))
        return uRI.getPath();
    }
    catch (URI.MalformedURIException exception) {
    }
    return systemId;
  }

  public static final String expandedSystemId(String systemIdOrPathName) {
    if (systemIdOrPathName == null || systemIdOrPathName.length() == 0)
      return null;
    // systemIdOrPathName is supposed to be an URI.
    try {
      return new URI(systemIdOrPathName).toString();
    }
    catch (Exception exception) {
    }
    // systemIdOrPathName is supposed to be a file path name.
    String pathName = normalisePathName(systemIdOrPathName);
    // SystemId is supposed to be a relative file path name.
    try {
      URI base = new URI("file", "", userDir(), null, null);
      return new URI(base, pathName).toString();
    }
    catch (Exception exception) {
    }
    // systemIdOrPathName is supposed to be an absolute file path name.
    return "file://" + pathName;
  }

  public static final String reducedSystemId(String systemIdOrPathName) {
    String systemId = expandedSystemId(systemIdOrPathName);
    if (systemId == null)
      return null;
    try {
      String userDirSystemId = "file://" + userDir();
      if (systemId.indexOf(userDirSystemId) == 0)
        systemId = systemId.substring(userDirSystemId.length());
    }
    catch (Exception exception) {
    }
    return systemId;
  }
}
