/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Container;
import java.awt.Frame;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Event;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.Dialog;

public abstract class DialogModal extends JDialog implements Dialog {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected final JButton buttonOK = new JButton();
  protected final JButton buttonCancel = new JButton();
  protected final JPanel contentPanel = new JPanel();
  protected final ActionListener oKActionListener = new ActionListener() {
    public void actionPerformed(ActionEvent event) {
      buttonOK_actionPerformed(event);
    }
  };
  protected final KeyListener okOnEnterKeyListener = new KeyAdapter() {
    public void keyPressed(KeyEvent event) {
      if (event.getKeyCode() == KeyEvent.VK_ENTER)
        ok();
    }
  };
  protected boolean isOK = false;

  public DialogModal(Frame parent) {
    super(parent, true);
    this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    Container voletDialogue = getContentPane();
    contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
    contentPanel.setBorder(new EmptyBorder(4, 4, 4, 4));
    final ActionListener disposeActionListener =
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          dispose();
        }
      };
    contentPanel.registerKeyboardAction(disposeActionListener, "ESCAPE",
      KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    voletDialogue.add(contentPanel);
    // Volet des boutons Ajouter et Annuler
    JPanel voletBoutons = new JPanel(new FlowLayout());
    buttonOK.setText(viewResource.getString(Label.VALIDER));
    buttonOK.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.VALIDER).charAt(0));
    buttonOK.addActionListener(oKActionListener);
    voletBoutons.add(buttonOK);
    buttonCancel.setText(viewResource.getString(Label.ANNULER));
    buttonCancel.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.ANNULER).charAt(0));
    buttonCancel.addActionListener(disposeActionListener);
    voletBoutons.add(buttonCancel);
    voletDialogue.add(voletBoutons, BorderLayout.SOUTH);
    // <code>pack()</code> to do.
  }

  protected void buttonOK_actionPerformed(ActionEvent event) {
    if (this.buttonOK.isEnabled())
      this.ok();
  }

  protected final void ok() {
    this.isOK = true;
    this.dispose();
  }

  protected final void addSeparator() {
    JPanel panelSeparator = new JPanel();
    panelSeparator.setPreferredSize(new Dimension(0, 4));
    panelSeparator.setBorder(new BevelBorder(BevelBorder.LOWERED));
    this.contentPanel.add(panelSeparator);
  }

  public final void setContentPreferredSize(Dimension preferredSize) {
    this.contentPanel.setPreferredSize(preferredSize);
    this.pack();
  }

  public final void setCancelEnabled(boolean isCancelEnabled) {
    this.buttonCancel.setEnabled(isCancelEnabled);
  }

  ////////////////////////
  // Dialog implementation
  ////////////////////////

  public void setTitle(String title) {
    try {
      title = viewResource.getString(title);
    }
    catch (Exception exception) {
    }
    super.setTitle(title);
  }

  public void show() {
    this.isOK = false;
    super.show();
  }

  public final boolean isOK() {
    return this.isOK;
  }
}
