/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JMenuBar;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Position;
import javax.swing.text.Document;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.util.view.ViewUtils;

public class TextFileEditorImpl extends JFrame implements TextFileEditor {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected ViewObserver viewObserver = null;
  protected String pathname = null;
  protected JMenuItem menuSave;
  protected JMenuItem menuSaveAs;
  protected JTextArea textField;
  protected JScrollPane scrollPanel;
  protected boolean isReadOnly = false;
  protected boolean hasBeenModifiedAfterLastLoad = false;
  protected boolean hasBeenModifiedAfterLastSave = false;
  protected Vector lineStartPositionList = null;

  public TextFileEditorImpl() {
    this(null);
  }

  public TextFileEditorImpl(Dimension contentDimension) {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Content panel
    JPanel contentPanel = new JPanel(new BorderLayout());
    this.getContentPane().add(contentPanel);
    contentPanel.setPreferredSize(
      contentDimension != null ? contentDimension : new Dimension(500, 300));
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    menuSave =
      new JMenuItem(viewResource.getString(Label.SAVE));
    menuSave.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SAVE).charAt(0));
    menuSave.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          save_actionPerformed();
        }
      });
    menuBar.add(menuSave);
    menuSaveAs =
      new JMenuItem(viewResource.getString(Label.SAVE_AS) + "...");
    menuSaveAs.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SAVE_AS).charAt(0));
    menuSaveAs.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          saveAs_actionPerformed();
        }
      });
    menuBar.add(menuSaveAs);
    // Text field
    textField = new JTextArea();
    textField.setLineWrap(true);
    textField.setWrapStyleWord(true);
    textField.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          textField_update();
        };
        public void removeUpdate(DocumentEvent event) {
          textField_update();
        };
        public void changedUpdate(DocumentEvent event) {
          textField_update();
        };
      });
    scrollPanel = new JScrollPane(textField);
    contentPanel.add(scrollPanel);
    this.pack();
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  protected void textField_update() {
    hasBeenModifiedAfterLastLoad = true;
    hasBeenModifiedAfterLastSave = true;
    menuSave.setEnabled(true);
  }

  protected void save_actionPerformed() {
    try {
      save();
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
  }

  protected void saveAs_actionPerformed() {
    try {
      save(null);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
  }

  private void clear() {
    textField.setText("");
    lineStartPositionList = null;
    menuSave.setEnabled(false);
    hasBeenModifiedAfterLastLoad = false;
    hasBeenModifiedAfterLastSave = false;
  }

  protected void save(String path) throws IOException {
    if (path == null) {
      path = choosePathName();
      if (path == null)
        return;
      if (new File(path).exists()
        && JOptionPane.showConfirmDialog((Frame)this,
        viewResource.getString(Label.DO_YOU_CONFIRM) + " "
        + viewResource.getString(Label.L_ECRASEMENT_DE) + "\n" + path + " ?",
        viewResource.getString(Label.LE_FICHIER_EXISTE_DEJA),
        JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
        return;
    }
    OutputStream outputStream = new FileOutputStream(path);
    PrintWriter out = new PrintWriter(new OutputStreamWriter(outputStream));
    String texte = textField.getText();
    int pos = 0;
    while (true) {
      int posNewLine = texte.indexOf('\n', pos);
      if (posNewLine < 0) {
        out.println(texte.substring(pos));
        break;
      }
      else {
        out.println(texte.substring(pos, posNewLine));
        pos = posNewLine + 1;
      }
    }
    out.close();
    hasBeenModifiedAfterLastSave = false;
    menuSave.setEnabled(false);
  }

  protected String choosePathName() {
    JFileChooser fileChooser = new JFileChooser(".");
    fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
    fileChooser.setDialogTitle(viewResource.getString(Label.SAVE_AS));
    fileChooser.setFileFilter(fileChooser.getAcceptAllFileFilter());
    return fileChooser.showDialog((Frame)this, null)
      == JFileChooser.APPROVE_OPTION ?
      fileChooser.getSelectedFile().getPath() : null;
  }

  ////////////////////////////////
  // TextFileEditor implementation
  ////////////////////////////////

  public void dispose() {
    super.dispose();
    if (viewObserver != null)
      viewObserver.notifyDispose(this);
  }

  public void setViewObserver(ViewObserver viewObserver) {
    this.viewObserver = viewObserver;
  }

  public final boolean isReadOnly() {
    return isReadOnly;
  }

  public final void setReadOnly(boolean isReadOnly) {
    if (hasBeenModifiedAfterLastSave)
      return;
    this.isReadOnly = isReadOnly;
    textField.setEnabled(!isReadOnly);
  }

  public final void load(String pathname) throws IOException {
    if (pathname == null || pathname.length() == 0) {
      this.pathname = null;
      clear();
    }
    else {
      this.pathname = pathname;
      load(new FileInputStream(pathname));
    }
  }

  public final void load(InputStream inputStream) throws IOException {
    if (inputStream != null)
      load(new InputStreamReader(inputStream));
    else
      clear();
  }

  public final void load(Reader reader) throws IOException {
    clear();
    if (reader == null)
      return;
    lineStartPositionList = new Vector(64);
    BufferedReader in = new BufferedReader(reader);
    StringBuffer s = new StringBuffer();
    int offset = 0;
    while (true) {
      String line = in.readLine();
      if (line == null)
        break;
      lineStartPositionList.addElement(new Integer(offset));
      s.append(line);
      s.append('\n');
      offset += line.length() + 1;
    }
    lineStartPositionList.addElement(new Integer(offset));
    in.close();
    textField.setText(s.toString());
    Document document = textField.getDocument();
    for (int k = 0; k < lineStartPositionList.size(); k++)
      try {
        lineStartPositionList.setElementAt(document.createPosition(
          ((Integer)lineStartPositionList.elementAt(k)).intValue()), k);
      }
      catch (BadLocationException exception) {
      }
    menuSave.setEnabled(false);
    hasBeenModifiedAfterLastLoad = false;
    hasBeenModifiedAfterLastSave = false;
  }

  public final void selectLine(int lineNumber) {
    if (lineNumber < 1 || lineNumber >= lineStartPositionList.size())
      return;
    int startOffset =
      ((Position)lineStartPositionList.elementAt(lineNumber - 1)).getOffset();
    int endOffset =
      ((Position)lineStartPositionList.elementAt(lineNumber)).getOffset() - 1;
    textField.setCaretPosition(startOffset);
    textField.moveCaretPosition(endOffset);
    JViewport viewport = scrollPanel.getViewport();
    try {
      Rectangle rectangle = textField.modelToView(startOffset);
      if (rectangle == null)
        return;
      int y = rectangle.y - viewport.getViewRect().height / 2;
      viewport.setViewPosition(new Point(0, y >= 0 ? y : 0));
    }
    catch (BadLocationException exception) {
    }
  }

  public final void save() throws IOException {
    save(!isReadOnly ? pathname : null);
  }

  public final boolean hasBeenModifiedAfterLastLoad() {
    return hasBeenModifiedAfterLastLoad;
  }

  public final boolean hasBeenModifiedAfterLastSave() {
    return hasBeenModifiedAfterLastSave;
  }
}
