/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogLocator;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.DialogModal;

public final class DialogLocatorImpl extends DialogModal
    implements DialogLocator{
  private static final Border borderLigne = new EmptyBorder(4, 4, 4, 4);
  private final JTextField fieldLineNumber = new JTextField(8);
  private final JTextField fieldColumnNumber = new JTextField(8);

  public DialogLocatorImpl(Frame parent) {
    super(parent);
    this.setResizable(false);
    final DocumentListener fieldUpdateListener =
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          field_update();
        };
        public void removeUpdate(DocumentEvent event) {
          field_update();
        };
        public void changedUpdate(DocumentEvent event) {
          field_update();
        };
      };
    JPanel panelLocation = new JPanel(new BorderLayout());
    JPanel panelLineNumber = new JPanel(new BorderLayout());
    panelLineNumber.setBorder(borderLigne);
    panelLineNumber.add(
        new JLabel(viewResource.getString(Label.LINE_NUMBER) + " : "),
        BorderLayout.WEST);
    fieldLineNumber.getDocument().addDocumentListener(fieldUpdateListener);
    fieldLineNumber.addActionListener(oKActionListener);
    panelLineNumber.add(fieldLineNumber);
    panelLocation.add(panelLineNumber, BorderLayout.WEST);
    JPanel panelColumnNumber = new JPanel(new BorderLayout());
    panelColumnNumber.setBorder(borderLigne);
    panelColumnNumber.add(
        new JLabel(viewResource.getString(Label.COLUMN_NUMBER) + " : "),
        BorderLayout.WEST);
    fieldColumnNumber.getDocument().addDocumentListener(fieldUpdateListener);
    fieldColumnNumber.addActionListener(oKActionListener);
    panelColumnNumber.add(fieldColumnNumber);
    panelLocation.add(panelColumnNumber, BorderLayout.EAST);
    contentPanel.add(panelLocation);
    // Mise aux dimensions
    pack();
  }

  private void field_update() {
    buttonOK.setEnabled(getLineNumber() > 0 && getColumnNumber() > 0);
  }

  public void setContentFont(Font font) {
    this.fieldLineNumber.setFont(font);
    this.fieldColumnNumber.setFont(font);
  }

  ///////////////////////////////
  // DialogLocator implementation
  ///////////////////////////////

  public int getLineNumber() {
    try {
      return Integer.parseInt(this.fieldLineNumber.getText());
    }
    catch (NumberFormatException exception) {
      return 0;
    }
  }

  public int getColumnNumber() {
    try {
      return Integer.parseInt(this.fieldColumnNumber.getText());
    }
    catch (NumberFormatException exception) {
      return 0;
    }
  }

  public void setLineNumber(int lineNumber) {
    this.fieldLineNumber.setText(Integer.toString(lineNumber));
  }

  public void setColumnNumber(int columnNumber) {
    this.fieldColumnNumber.setText(Integer.toString(columnNumber));
  }
}
