/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogNewDocument;
import org.xmloperator.framework.view.global.FileChooser;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.DialogModal;

public class DialogNewDocumentImpl extends DialogModal
    implements DialogNewDocument {
  protected static final Border borderLigne = new EmptyBorder(4, 4, 4, 4);
  protected GlobalOptionsControl globalOptionsControl;
  protected Options options;
  protected FileChooser fileChooser;
  protected final JComboBox comboBoxDocumentType = new JComboBox();
  protected final JTextField fieldDocumentPathName = new JTextField(30);

  public DialogNewDocumentImpl(Frame parent,
    GlobalOptionsControl globalOptionsControl, Options options,
    FileChooser fileChooser) {
    super(parent);
    this.globalOptionsControl = globalOptionsControl;
    this.options = options;
    this.fileChooser = fileChooser;
    // Nom de type de document
    JPanel panelDocumentTypeName = new JPanel(new BorderLayout());
    panelDocumentTypeName.setBorder(borderLigne);
    panelDocumentTypeName.add(new JLabel(
      viewResource.getString(Label.DOCUMENT_TYPE) + " : "),
      BorderLayout.WEST);
    updateDocumentTypeList();
    panelDocumentTypeName.add(comboBoxDocumentType);
    JButton newDocumentTypeButton =
      new JButton(viewResource.getString(Label.NEW));
    newDocumentTypeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          newDocumentType_actionPerformed();
        }
      });
    panelDocumentTypeName.add(newDocumentTypeButton, BorderLayout.EAST);
    contentPanel.add(panelDocumentTypeName);
    // path name
    JPanel panelPathName = new JPanel(new BorderLayout());
    panelPathName.setBorder(borderLigne);
    panelPathName.add(new JLabel(
      viewResource.getString(Label.NOM_DE_FICHIER) + " : "),
      BorderLayout.WEST);
    fieldDocumentPathName.addActionListener(oKActionListener);
    panelPathName.add(fieldDocumentPathName);
    JButton browseButton = new JButton(viewResource.getString(Label.BROWSE));
    browseButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          browse_actionPerformed();
        }
      });
    panelPathName.add(browseButton, BorderLayout.EAST);
    contentPanel.add(panelPathName);
    // Mise aux dimensions
    pack();
  }

  protected void updateDocumentTypeList() {
    if (comboBoxDocumentType.getItemCount() != 0)
      comboBoxDocumentType.removeAllItems();
    List documentTypeInfoList = options.getOptionList(Options.DOCUMENTTYPE);
    for (int k = 0; k < documentTypeInfoList.size(); k++) {
      DocumentTypeInfo documentTypeInfo =
        (DocumentTypeInfo)documentTypeInfoList.get(k);
      comboBoxDocumentType.addItem(documentTypeInfo);
    }
    comboBoxDocumentType.setSelectedIndex(0);
  }

  protected void newDocumentType_actionPerformed() {
    try {
      globalOptionsControl.newDocumentType();
      updateDocumentTypeList();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void browse_actionPerformed() {
    try {
      String pathname =
        this.fileChooser.chooseDocumentPathName(true, this.getTitle());
      if (pathname != null)
        setDocumentPathName(pathname);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setContentFont(Font font) {
    this.comboBoxDocumentType.setFont(font);
    this.fieldDocumentPathName.setFont(font);
  }

  ///////////////////////////////////
  // DialogNewDocument implementation
  ///////////////////////////////////

  public DocumentTypeInfo getDocumentTypeInfo() {
    return (DocumentTypeInfo)comboBoxDocumentType.getSelectedItem();
  }

  public String getDocumentPathName() {
    return fieldDocumentPathName.getText();
  }

  public void setDocumentPathName(String documentPathName) {
    fieldDocumentPathName.setText(documentPathName);
  }
}
