/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.ResourceBundle;
import java.util.List;
import java.util.Collections;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.document.doctype.RNGInfo;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.global.DocumentTypeInfoView;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalFileControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.view.ExceptionDialogs;

public final class DocumentTypeInfoFrame extends JFrame
  implements DocumentTypeInfoView {
  private static final int COLUMN_NAME = 0;
  private static final int COLUMN_ADDITIONALKEY = 1;
  private static final int COLUMN_SCHEMATYPE = 2;
  private static final String[] enteteColonne = {
    Label.DOCUMENT_TYPE,
    Label.ADDITIONAL_KEY,
    Label.SCHEMA_TYPE};
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private GlobalControl globalControl;
  private GlobalFileControl globalFileControl;
  private GlobalOptionsControl globalOptionsControl;
  private List documentTypeInfoList;
  private TableModel tableModel;
  private JTable table;
  private JMenuItem menuModify;
  private JMenuItem menuDuplicate;
  private JMenuItem menuDelete;
  private JMenu menuBuildDTDFromDTDXML;
  private JMenu menuEditDTD;
  private JMenu menuEditRNG;

  public DocumentTypeInfoFrame(List documentTypeInfoList,
    GlobalControl globalControl, GlobalFileControl globalFileControl,
    GlobalOptionsControl globalOptionsControl) {
    setTitle(Constants.TOOL + " - "
      + viewResource.getString(Label.DOCUMENT_TYPES));
    this.documentTypeInfoList = documentTypeInfoList;
    this.globalControl = globalControl;
    this.globalFileControl = globalFileControl;
    this.globalOptionsControl = globalOptionsControl;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    Collections.sort(documentTypeInfoList);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(480, 200));
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    // Menu "New type"
    JMenuItem menuNewDocumentType =
      new JMenuItem(viewResource.getString(Label.NEW));
    menuNewDocumentType.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.NEW).charAt(0));
    menuNewDocumentType.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          newDocumentType_actionPerformed();
        }
      });
    menuBar.add(menuNewDocumentType);
    // Menu Modify
    menuModify = new JMenuItem(viewResource.getString(Label.MODIFY));
    menuModify.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.MODIFY).charAt(0));
    menuModify.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          modify_actionPerformed();
        }
      });
    menuBar.add(menuModify);
    // Menu Duplicate
    menuDuplicate =
      new JMenuItem(viewResource.getString(Label.DUPLICATE));
    menuDuplicate.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.DUPLICATE).charAt(0));
    menuDuplicate.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          duplicate_actionPerformed();
        }
      });
    menuBar.add(menuDuplicate);
    // Menu delete
    menuDelete =
      new JMenuItem(viewResource.getString(Label.SUPPRIMER));
    menuDelete.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SUPPRIMER).charAt(0));
    menuDelete.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          delete_actionPerformed();
        }
      });
    menuBar.add(menuDelete);
    // Menu Build DTD from DTDXML
    menuBuildDTDFromDTDXML = new JMenu("DTDXML -> DTD");
    menuBuildDTDFromDTDXML.setMnemonic('D');
    JMenuItem menuBuildDTDWithParameterEntities =
      new JMenuItem(viewResource.getString(Label.WITH_PARAMETER_ENTITIES));
    menuBuildDTDWithParameterEntities.setMnemonic(
      viewResource.getString(Label.MNEMO + Label.WITH_PARAMETER_ENTITIES).
      charAt(0));
    menuBuildDTDWithParameterEntities.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          buildDTDFromDTDXML_actionPerformed(true);
        }
      });
    menuBuildDTDFromDTDXML.add(menuBuildDTDWithParameterEntities);
    JMenuItem menuBuildDTDWithoutParameterEntities =
      new JMenuItem(viewResource.getString(Label.WITHOUT_PARAMETER_ENTITIES));
    menuBuildDTDWithoutParameterEntities.setMnemonic(
      viewResource.getString(Label.MNEMO + Label.WITHOUT_PARAMETER_ENTITIES).
      charAt(0));
    menuBuildDTDWithoutParameterEntities.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          buildDTDFromDTDXML_actionPerformed(false);
        }
      });
    menuBuildDTDFromDTDXML.add(menuBuildDTDWithoutParameterEntities);
    menuBar.add(menuBuildDTDFromDTDXML);
    // Menu Edit DTD
    menuEditDTD = new JMenu(viewResource.getString(Label.EDIT_DTD));
    menuEditDTD.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.EDIT_DTD).charAt(0));
    JMenuItem menuEditDTDasDTD =
      new JMenuItem(viewResource.getString(Label.AS_DTD));
    menuEditDTDasDTD.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.AS_DTD).charAt(0));
    menuEditDTDasDTD.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          editDTD_actionPerformed();
        }
      });
    menuEditDTD.add(menuEditDTDasDTD);
    JMenuItem menuEditDTDasXML =
      new JMenuItem(viewResource.getString(Label.AS_DTDXML));
    menuEditDTDasXML.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.AS_DTDXML).charAt(0));
    menuEditDTDasXML.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          editXMLDTD_actionPerformed();
        }
      });
    menuEditDTD.add(menuEditDTDasXML);
    menuBar.add(menuEditDTD);
    // Menu Edit RNG
    menuEditRNG = new JMenu(viewResource.getString(Label.EDIT_RNG));
    menuEditRNG.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.EDIT_RNG).charAt(0));
    JMenuItem menuEditRNGFull =
      new JMenuItem(viewResource.getString(Label.FULL_SYNTAX));
    menuEditRNGFull.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.FULL_SYNTAX).charAt(0));
    menuEditRNGFull.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          editRNG_actionPerformed(false);
        }
      });
    menuEditRNG.add(menuEditRNGFull);
    JMenuItem menuEditRNGSimple =
      new JMenuItem(viewResource.getString(Label.SIMPLE_SYNTAX));
    menuEditRNGSimple.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SIMPLE_SYNTAX).charAt(0));
    menuEditRNGSimple.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          editRNG_actionPerformed(true);
        }
      });
    menuEditRNG.add(menuEditRNGSimple);
    menuBar.add(menuEditRNG);
    // Table
    tableModel = new AbstractTableModel() {
      public String getColumnName(int col) {
        return table_getColumnName(col);
      }
      public Class getColumnClass(int col) {
        return String.class;
      }
      public int getRowCount() {
        return table_getRowCount();
      }
      public int getColumnCount() {
        return enteteColonne.length;
      }
      public Object getValueAt(int row, int col) {
        return table_getValueAt(row, col);
      }
    };
    table = new JTable(tableModel);
    table.setShowVerticalLines(true);
    table.setRowSelectionAllowed(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getSelectionModel().addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          setItemMenuEnabled(true);
        }
      });
    table.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            modify_actionPerformed();
        }
      });
    JTableHeader tableHeader = table.getTableHeader();
    tableHeader.setResizingAllowed(true);
    tableHeader.setReorderingAllowed(false);
    JScrollPane scrollPane = new JScrollPane(table);
    voletContenu.add(scrollPane);
    // Mise aux dimensions
    this.pack();
    // Initialisation menu
    setItemMenuEnabled(false);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private String table_getColumnName(int col) {
    if (col < 0 || col >= enteteColonne.length)
      return "";
    String columnName = enteteColonne[col];
    try {
      return viewResource.getString(columnName);
    }
    catch (Exception exception) {
      return columnName;
    }
  }

  private int table_getRowCount() {
    return documentTypeInfoList.size();
  }

  private Object table_getValueAt(int row, int column) {
    try {
      if (row < 0 || row >= documentTypeInfoList.size())
        return "";
      DocumentTypeInfo documentTypeInfo =
        (DocumentTypeInfo)documentTypeInfoList.get(row);
      switch (column) {
        case COLUMN_NAME:
          return documentTypeInfo.getDocumentTypeName();
        case COLUMN_ADDITIONALKEY:
          return documentTypeInfo.getDocumentTypeAdditionalKey();
        case COLUMN_SCHEMATYPE:
          return schemaType(documentTypeInfo);
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    return "";
  }

  private static final String schemaType(DocumentTypeInfo documentTypeInfo) {
    RNGInfo rNGInfo = documentTypeInfo.getRNGInfo();
    String rNGFullSyntaxSystemId = rNGInfo.getRNGFullSyntaxSystemId();
    String rNGSimpleSyntaxSystemId = rNGInfo.getRNGSimpleSyntaxSystemId();
    boolean isRNG = (rNGFullSyntaxSystemId != null
      && rNGFullSyntaxSystemId.length() != 0)
      || (rNGSimpleSyntaxSystemId != null
      && rNGSimpleSyntaxSystemId.length() != 0);
    DTDInfo dTDInfo = documentTypeInfo.getDTDInfo();
    String dTDDocumentSystemId = dTDInfo.getDocumentRelativeDTDSystemId();
    String dTDUserSystemId = dTDInfo.getUserRelativeDTDSystemId();
    boolean isDTD =
      (dTDDocumentSystemId != null && dTDDocumentSystemId.length() != 0)
      || (dTDUserSystemId != null && dTDUserSystemId.length() != 0);
    if (isRNG & isDTD)
      return "RNG + DTD";
    else if (isRNG)
      return "RNG";
    else if (isDTD)
      return "DTD";
    else
      return "";
  }

  private void newDocumentType_actionPerformed() {
    try {
      globalOptionsControl.newDocumentType();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void modify_actionPerformed() {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo != null)
        globalOptionsControl.editDocumentTypeInfo(
          documentTypeInfo.getDocumentTypeName(),
          documentTypeInfo.getDocumentTypeAdditionalKey());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void duplicate_actionPerformed() {
    try {
      DocumentTypeInfo documentTypeInfoSource = getSelectedDocumentTypeInfo();
      if (documentTypeInfoSource != null)
        globalOptionsControl.newDocumentType(documentTypeInfoSource);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void delete_actionPerformed() {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo != null) {
        if (JOptionPane.showConfirmDialog(
          getFrame(),
          viewResource.getString(Label.DO_YOU_CONFIRM) + " ?",
          viewResource.getString(Label.REMOVING_A_DOCUMENT_TYPE),
          JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
          return;
        globalOptionsControl.removeDocumentType(
          documentTypeInfo.getDocumentTypeName(),
          documentTypeInfo.getDocumentTypeAdditionalKey());
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void buildDTDFromDTDXML_actionPerformed(
    boolean arePatternsTranslatedToParameterEntityDeclarations) {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo != null) {
        DTDInfo dTDInfo = documentTypeInfo.getDTDInfo();
        globalOptionsControl.buildDTDFromDTDXML(
          dTDInfo.getDTDXMLSystemId(),
          dTDInfo.getUserRelativeDTDSystemId(),
          arePatternsTranslatedToParameterEntityDeclarations);
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void editDTD_actionPerformed() {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo == null)
        return;
      String userRelativeDTDSystemId =
        documentTypeInfo.getDTDInfo().getUserRelativeDTDSystemId();
      if (userRelativeDTDSystemId == null
        || userRelativeDTDSystemId.length() == 0)
        return;
      globalFileControl.editDTD(userRelativeDTDSystemId, false);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void editXMLDTD_actionPerformed() {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo == null)
        return;
      String dTDXMLSystemId = documentTypeInfo.getDTDInfo().getDTDXMLSystemId();
      if (dTDXMLSystemId == null || dTDXMLSystemId.length() == 0)
        return;
      globalControl.createInstance(true, dTDXMLSystemId, null);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void editRNG_actionPerformed(boolean isSimpleSyntax) {
    try {
      DocumentTypeInfo documentTypeInfo = getSelectedDocumentTypeInfo();
      if (documentTypeInfo == null)
        return;
      RNGInfo rNGInfo = documentTypeInfo.getRNGInfo();
      String rNGSystemId = isSimpleSyntax ?
        rNGInfo.getRNGSimpleSyntaxSystemId() :
        rNGInfo.getRNGFullSyntaxSystemId();
      if (rNGSystemId == null || rNGSystemId.length() == 0)
        return;
      globalControl.createInstance(true, rNGSystemId, null);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private DocumentTypeInfo getSelectedDocumentTypeInfo() {
    int index = table.getSelectedRow();
    return index >= 0 && index < documentTypeInfoList.size() ?
      (DocumentTypeInfo)documentTypeInfoList.get(index) : null;
  }

  private void setItemMenuEnabled(boolean isEnabled) {
    DocumentTypeInfo documentTypeInfo =
      isEnabled ? getSelectedDocumentTypeInfo() : null;
    isEnabled = documentTypeInfo != null;
    boolean isUserRelativeDTDSystemIdDefined = isEnabled;
    boolean isDTDXMLSystemIdDefined = isEnabled;
    boolean isRNGSystemIdDefined = isEnabled;
    if (documentTypeInfo != null) {
      DTDInfo dTDInfo = documentTypeInfo.getDTDInfo();
      String userRelativeDTDSystemId = dTDInfo.getUserRelativeDTDSystemId();
      String dTDXMLSystemId = dTDInfo.getDTDXMLSystemId();
      String rNGFullSyntaxSystemId =
          documentTypeInfo.getRNGInfo().getRNGFullSyntaxSystemId();
      if (userRelativeDTDSystemId == null
          || userRelativeDTDSystemId.length() == 0)
        isUserRelativeDTDSystemIdDefined = false;
      if (dTDXMLSystemId == null || dTDXMLSystemId.length() == 0)
        isDTDXMLSystemIdDefined = false;
      if (rNGFullSyntaxSystemId == null || rNGFullSyntaxSystemId.length() == 0)
        isRNGSystemIdDefined = false;
    }
    menuModify.setEnabled(isEnabled);
    menuDuplicate.setEnabled(isEnabled);
    menuDelete.setEnabled(isEnabled);
    menuBuildDTDFromDTDXML.setEnabled(
      isUserRelativeDTDSystemIdDefined && isDTDXMLSystemIdDefined);
    menuEditDTD.setEnabled(isUserRelativeDTDSystemIdDefined);
    menuEditRNG.setEnabled(isRNGSystemIdDefined);
  }

  //////////////////////////////////////
  // DocumentTypeInfoView implementation
  //////////////////////////////////////

  public void update() {
    table.getSelectionModel().clearSelection();
    table.tableChanged(new TableModelEvent(tableModel));
    setItemMenuEnabled(false);
  }

  public Frame getFrame() {
    return (Frame)this;
  }

  public void select(
    String documentTypeName, String documentTypeAdditionalKey) {
    for (int k = 0; k < documentTypeInfoList.size(); k++ ) {
      DocumentTypeInfo documentTypeInfo =
        (DocumentTypeInfo)documentTypeInfoList.get(k);
      if (documentTypeName.equals(documentTypeInfo.getDocumentTypeName())
        && documentTypeAdditionalKey.equals(
        documentTypeInfo.getDocumentTypeAdditionalKey())) {
        table.setRowSelectionInterval(k, k);
        break;
      }
    }
  }
}
