/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.AbstractListModel;
import javax.swing.ListSelectionModel;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.view.model.DomTreeNode;

public class NodeListFrame extends JFrame implements ObserverView {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected ViewObserver viewObserver = null;
  protected FullyFledgedDocument fullyFledgedDocument;
  protected Options options;
  protected ViewControl viewControl;
  protected NodeListModel listModel;
  protected JPanel voletContenu;
  protected JList listPanel;
  protected JScrollPane scrollPanel;

  public NodeListFrame(
      FullyFledgedDocument fullyFledgedDocument, int nodeListType,
      Options options, ViewControl viewControl, Shortcuts shortcuts) {
    if (fullyFledgedDocument == null)
      throw new IllegalArgumentException();
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.viewControl = viewControl;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Volet contenu
    voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(
      new Dimension(options.getIntOptionValue(Options.NODELISTWIDTH),
        options.getIntOptionValue(Options.NODELISTHEIGHT)));
    shortcuts.registerKeyboardAction(voletContenu);
    // Volet liste
    listModel = new NodeListModel(fullyFledgedDocument, nodeListType);
    listPanel = new JList(listModel);
    listPanel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    listPanel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          list_selectionChanged();
        }
      });
    listPanel.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            list_selectionChanged();
        }
      });
    JScrollPane scrollPanel = new JScrollPane(listPanel);
    voletContenu.add(scrollPanel);
    // Mise aux dimensions
    this.pack();
    // Initialisation
    update(null, null);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  protected void list_selectionChanged() {
    try {
      int index = listPanel.getSelectedIndex();
      if (index < 0)
        return;
      if (viewControl == null)
        return;
      ToolView toolView = viewControl.getToolView();
      if (toolView == null)
        return;
      Node node = listModel.getNodeAt(index);
      toolView.setSelectedNode(node, true);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setViewObserver(ViewObserver viewObserver) {
    this.viewObserver = viewObserver;
  }

  protected static class NodeListModel extends AbstractListModel {
    protected FullyFledgedDocument fullyFledgedDocument;
    protected int nodeListType;
    protected int lastSize = 0;

    public NodeListModel(
        FullyFledgedDocument fullyFledgedDocument, int nodeListType) {
      this.fullyFledgedDocument = fullyFledgedDocument;
      this.nodeListType = nodeListType;
    }

    public int getSize() {
      return this.fullyFledgedDocument.getNodeList(this.nodeListType).
          getLength();
    }

    public Object getElementAt(int index) {
      return DomTreeNode.domTreeNodeFromNode(
          this.getNodeAt(index),
          this.fullyFledgedDocument.getNodeLinker(),
          this.fullyFledgedDocument.getElementKeyProvider());
    }

    public Node getNodeAt(int index) {
      return this.fullyFledgedDocument.getNodeList(this.nodeListType).
          item(index);
    }

    public void processUpdate() {
      int size = getSize();
      if (lastSize != 0)
        fireIntervalRemoved(this, 0, lastSize - 1);
      if (size != 0)
        fireIntervalAdded(this, 0, size - 1);
      lastSize = size;
    }
  }

  //////////////////////////
  // Observer implementation
  //////////////////////////

  public void update(Object observable, Object argument) {
    listModel.processUpdate();
  }

  //////////////////////////////
  // ObserverView implementation
  //////////////////////////////

  public void dispose() {
    super.dispose();
    if (viewObserver != null)
      viewObserver.notifyDispose(this);
  }
}
