/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.List;
import java.util.Hashtable;
import java.util.ResourceBundle;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JViewport;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeModel;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.ComparableNode;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.tree.DomTree;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.view.model.DomTreeNode;
import org.xmloperator.view.model.DomTreeModel;
import org.xmloperator.view.main.menu.MenuEdit;
import org.xmloperator.view.main.menu.MenuMove;
import org.xmloperator.view.main.menu.MenuFragment;
import org.xmloperator.view.main.menu.MenuCompare;
import org.xmloperator.view.main.ToolTreeCellRenderer;

public class PanelDocumentTree extends JTree implements DocumentTreeView {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected DomTreeModel domTreeModel;
  protected JViewport viewport = null;
  protected MenuEdit menuEdit;
  protected MenuMove menuMove;
  protected MenuFragment menuFragment;
  protected MenuCompare menuCompare;
  protected DocumentViewControl documentViewControl;
  protected FullyFledgedDocument fullyFledgedDocument;
  protected DocumentAssociationAccess documentAssociationAccess;
  protected Options options;
  protected Hashtable fromArbreDocumentLocalMenuToReferencedElement;
  protected boolean isSelectionChangeInhibited = false;

  public PanelDocumentTree(DomTreeModel domTreeModel,
    MenuEdit menuEdit, MenuMove menuMove, MenuFragment menuFragment,
    MenuCompare menuCompare, DocumentViewControl documentViewControl,
    FullyFledgedDocument fullyFledgedDocument,
    DocumentAssociationAccess documentAssociationAccess, Options options) {
    super((TreeModel)domTreeModel);
    this.domTreeModel = domTreeModel;
    this.menuEdit = menuEdit;
    this.menuMove = menuMove;
    this.menuFragment = menuFragment;
    this.menuCompare = menuCompare;
    this.documentViewControl = documentViewControl;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.documentAssociationAccess = documentAssociationAccess;
    this.options = options;
    this.setRootVisible(false);
    boolean isLargeFont =
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE;
    this.setRowHeight(isLargeFont ? 18 : 16);
    this.setFont(isLargeFont ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
    this.setCellRenderer(new ToolTreeCellRenderer(
      documentAssociationAccess, fullyFledgedDocument, options));
    this.addTreeSelectionListener(
      new TreeSelectionListener() {
        public void valueChanged(TreeSelectionEvent event) {
          arbreDocument_selectionChanged();
        }
      });
    this.addMouseListener(new ArbreDocumentMouseAdapter());
  }

  protected class ArbreDocumentMouseAdapter extends MouseAdapter {

    public ArbreDocumentMouseAdapter() {
    }

    public void mouseClicked(MouseEvent event) {
      if (event.getClickCount() == 2)
        arbreDocument_activation();
    }

    public void mousePressed(MouseEvent event) {
      if ((event.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
        arbreDocument_localMenu(event.getX(), event.getY());
    }
  }

  protected void arbreDocument_selectionChanged() {
    try {
      if (isSelectionChangeInhibited)
        return;
      fromArbreDocumentLocalMenuToReferencedElement = null;
      documentViewControl.displaySelectedNode();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void arbreDocument_activation() {
    try {
      documentViewControl.activateSelectedNode();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void arbreDocument_localMenu(int x, int y) {
    try {
      TreePath treePath = getPathForLocation(x, y);
      if (!isPathSelected(treePath))
        setSelectionPath(treePath);
      Node[] selectedNodes = getSelectedNodes();
      if (selectedNodes == null)
        return;
      boolean isOneNodeSelected = selectedNodes.length == 1;
      Node selectedNode = null;
      int nodeType = -1;
      if (isOneNodeSelected) {
        selectedNode = selectedNodes[0];
        nodeType = selectedNode.getNodeType();
        if (nodeType == Node.DOCUMENT_NODE)
          return;
      }
      Node parentNode = selectedNodes[0].getParentNode();
      boolean isDocumentChilds =
          parentNode != null && parentNode.getNodeType() == Node.DOCUMENT_NODE;
      boolean containsDocumentElement = false;
      if (isDocumentChilds)
        for (int k = 0; k < selectedNodes.length; k++) {
          int childNodeType = selectedNodes[k].getNodeType();
          if (childNodeType == Node.DOCUMENT_TYPE_NODE)
            return;
          if (childNodeType == Node.ELEMENT_NODE)
            containsDocumentElement = true;
        }
      NodeInterval nodeIntervalForMove =
          fullyFledgedDocument.getNodeIntervalForMove();
      boolean isMoveInProgress =
          nodeIntervalForMove != null && nodeIntervalForMove.isDefined();
      JPopupMenu menu = new JPopupMenu();
      if (!isMoveInProgress) {
        if (isOneNodeSelected && nodeType == Node.ELEMENT_NODE) {
          DocumentAssociation documentAssociation =
              documentAssociationAccess.getDocumentAssociation();
          if (documentAssociation != null
              && documentAssociation.getElementAssociation(
              (Element)selectedNode) != null) {
            menu.add(menuCompare.createMenuSeeAssociate());
            menu.addSeparator();
          }
        }
        menu.add(menuEdit.createMenuNewPrevious());
        menu.add(menuEdit.createMenuNewNext());
        if (nodeType == Node.ELEMENT_NODE)
          menu.add(menuEdit.createMenuNewChild());
        if (!containsDocumentElement) {
          if (isOneNodeSelected)
            menu.add(menuEdit.createMenuReplaceElement());
          menu.add(menuEdit.createMenuRemoveNodes());
          menu.addSeparator();
          menu.add(menuFragment.createMenuCutNodes());
          menu.add(menuFragment.createMenuCopyNodes());
        }
        if (fullyFledgedDocument.getPresseFragment().hasNodes()) {
          menu.add(menuFragment.createMenuPasteBefore());
          menu.add(menuFragment.createMenuPasteAfter());
          menu.add(menuFragment.createMenuPasteChild());
        }
      }
      if (!containsDocumentElement) {
        menu.addSeparator();
        menu.add(menuMove.createMenuFairePrecedent());
        menu.add(menuMove.createMenuFaireSuivant());
        menu.add(menuMove.createMenuFaireDescendant());
        if (!isDocumentChilds && !isMoveInProgress)
          menu.add(menuMove.createMenuFaireParent());
        menu.add(menuMove.createMenuToBeMoved());
        if (!isMoveInProgress) {
          menu.addSeparator();
          if (!isDocumentChilds)
            menu.add(menuEdit.createMenuPack());
          if (nodeType == Node.ELEMENT_NODE && selectedNode.hasChildNodes())
            menu.add(menuEdit.createMenuUnpack());
        }
      }
      if (nodeType == Node.ELEMENT_NODE) {
        String[] references =
            fullyFledgedDocument.getDocumentUtil().referencesFromElement(
            (Element)selectedNode);
        if (references != null) {
          menu.addSeparator();
          fromArbreDocumentLocalMenuToReferencedElement = new Hashtable();
          ActionListener actionListener = new ActionListener()  {
            public void actionPerformed(ActionEvent event) {
              arbreDocument_localMenu_actionPerformed(event);
            }
          };
          try {
            for (int k = 0; k < references.length; k++) {
              String id = references[k];
              Element referencedElement =
                  fullyFledgedDocument.getIdManager().getElementById(id);
              if (referencedElement == null)
                throw new IdException(IdException.CANNOT_FIND_ID, id);
              DomTreeNode domTreeNode =
                  DomTreeNode.domTreeNodeFromNode(referencedElement,
                  this.fullyFledgedDocument.getNodeLinker(),
                  this.fullyFledgedDocument.getElementKeyProvider());
              String elementView = domTreeNode.toString();
              JMenuItem menuItem = new JMenuItem(
                  viewResource.getString(Label.GOTO) + ' ' + elementView);
              fromArbreDocumentLocalMenuToReferencedElement.put(menuItem,
                  referencedElement);
              menuItem.addActionListener(actionListener);
              menu.add(menuItem);
            }
          }
          catch (Exception exception) {
            ExceptionControler.printStackTrace(exception);
          }
        }
        if (!isMoveInProgress) {
          menu.addSeparator();
          menu.add(menuEdit.createMenuOperateElement());
        }
      }
      else if (!isMoveInProgress) {
        if (nodeType == Node.TEXT_NODE) {
          menu.addSeparator();
          menu.add(menuEdit.createMenuJoinNext());
          menu.add(menuEdit.createMenuSplitText());
        }
        else if (ComparableNode.areNodesElementsWithTheSameName(selectedNodes))
        {
          menu.addSeparator();
          menu.add(menuMove.createMenuReorderAscending());
          menu.add(menuMove.createMenuReorderDescending());
        }
      }
      menu.show(this, x, y);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void arbreDocument_localMenu_actionPerformed(ActionEvent event) {
    try {
      if (fromArbreDocumentLocalMenuToReferencedElement == null)
        return;
      Element referencedElement =
          (Element)fromArbreDocumentLocalMenuToReferencedElement.get(
          event.getSource());
      if (referencedElement != null)
        documentViewControl.gotoNode(referencedElement);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setViewport(JViewport viewport) {
    this.viewport = viewport;
  }

  public DomTree getDomTree() {
    return domTreeModel;
  }

  public void setRootNode(Node rootNode) {
    isSelectionChangeInhibited = true;
    try {
      domTreeModel.setRootNode(rootNode, false);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      isSelectionChangeInhibited = false;
    }
  }

  public void expandNode(Node node) {
    this.expandPath(domTreeModel.getTreePath(node));
  }

  public Node getSelectedNode() {
    Node[] selectedNodes = getSelectedNodes();
    return selectedNodes != null && selectedNodes.length == 1 ?
      selectedNodes[0] : null;
  }

  public Node[] getSelectedNodes() {
    TreePath[] treePaths = getSelectionPaths();
    if (treePaths == null || treePaths.length == 0)
      return null;
    DomTreeNode domTreeNode = (DomTreeNode)treePaths[0].getLastPathComponent();
    if (domTreeNode == null)
      return null;
    Node currentNode = domTreeNode.getNode();
    int count = 1;
    for (int k = 1; k < treePaths.length; k++) {
      domTreeNode = (DomTreeNode)treePaths[k].getLastPathComponent();
      if (domTreeNode == null)
        return null;
      Node nextNode = domTreeNode.getNode();
      if (nextNode.getPreviousSibling() == currentNode) {
        currentNode = nextNode;
        count++;
      }
      else if (isDescendant(nextNode, currentNode))
        treePaths[k] = null;
      else return null;
    }
    Node[] nodes = new Node[count];
    int index = 0;
    for (int k = 0; k < treePaths.length; k++) {
      TreePath treePath = treePaths[k];
      if (treePath != null) {
        domTreeNode = (DomTreeNode)treePath.getLastPathComponent();
        nodes[index] = domTreeNode.getNode();
        index++;
      }
    }
    return nodes;
  }

  protected boolean isDescendant(Node node, Node parentNode) {
    while (true) {
      if (node.getNodeType() == Node.ATTRIBUTE_NODE)
        node = ((Attr)node).getOwnerElement();
      else
        node = node.getParentNode();
      if (node == parentNode)
        return true;
      else if (node == null)
        return false;
    }
  }

  public void setSelectedNode(Node node, boolean withRecentring) {
    if (node == null) {
      clearSelection();
      return;
    }
    setSelectionPath(domTreeModel.getTreePath(node));
    recentreOnNode(node, withRecentring);
  }

  public void setSelectedNodeInterval(Node firstNode, Node lastNode,
    boolean withRecentring) {
    int intervalSize = getNodeIntervalSize(firstNode, lastNode);
    if (intervalSize == 0) {
      clearSelection();
      return;
    }
    TreePath[] treePaths = new TreePath[intervalSize];
    Node node = firstNode;
    for (int k = 0; k < intervalSize; k++) {
      treePaths[k] = domTreeModel.getTreePath(node);
      node = node.getNextSibling();
    }
    setSelectionPaths(treePaths);
    recentreOnNodeInterval(firstNode, lastNode, withRecentring);
  }

  protected static final int getNodeIntervalSize(Node firstNode, Node lastNode)
  {
    if (firstNode == null || lastNode == null)
      return 0;
    Node node = firstNode;
    int nb = 1;
    while (node != lastNode) {
      node = node.getNextSibling();
      if (node == null)
        return 0;
      nb++;
    }
    return nb;
  }

  protected void recentreOnNode(Node node, boolean evenIfNodeIsWithinViewport) {
    if (node == null)
      throw new IllegalArgumentException();
    if (viewport == null)
      return;
    TreePath treePath = domTreeModel.getTreePath(node);
    makeVisible(treePath);
    Rectangle pathBounds = getPathBounds(treePath);
    if (pathBounds == null)
      return;
    Rectangle viewRect = viewport.getViewRect();
    boolean isNodeWithinWiewport = pathBounds.y > viewRect.y
      && pathBounds.y + pathBounds.height < viewRect.y + viewRect.height;
    if (isNodeWithinWiewport && !evenIfNodeIsWithinViewport)
      return;
    int y = pathBounds.y - viewRect.height / 2;
    viewport.setViewPosition(new Point(0, y >= 0 ? y : 0));
  }

  protected void recentreOnNodeInterval(Node firstNode, Node lastNode,
    boolean evenIfNodeIsWithinViewport) {
    if (firstNode == null || lastNode == null)
      throw new IllegalArgumentException();
    if (viewport == null)
      return;
    TreePath firstTreePath = domTreeModel.getTreePath(firstNode);
    TreePath lastTreePath = domTreeModel.getTreePath(lastNode);
    makeVisible(firstTreePath);
    makeVisible(lastTreePath);
    Rectangle firstPathBounds = getPathBounds(firstTreePath);
    Rectangle lastPathBounds = getPathBounds(lastTreePath);
    if (firstPathBounds == null || lastPathBounds == null)
      return;
    Rectangle viewRect = viewport.getViewRect();
    boolean isNodeWithinWiewport = firstPathBounds.y > viewRect.y
      && lastPathBounds.y + lastPathBounds.height < viewRect.y + viewRect.height;
    if (isNodeWithinWiewport && !evenIfNodeIsWithinViewport)
      return;
    int y = (firstPathBounds.y + lastPathBounds.y) / 2 - viewRect.height / 2;
    viewport.setViewPosition(new Point(0, y >= 0 ? y : 0));
  }

  public void selectPreviousRow() {
    TreePath[] treePaths = getSelectionPaths();
    if (treePaths == null || treePaths.length == 0)
      return;
    int row = getRowForPath(treePaths[0]);
    if (row > 0)
      setSelectionRow(row - 1);
  }

  public void selectNextRow() {
    TreePath[] treePaths = getSelectionPaths();
    if (treePaths == null || treePaths.length == 0)
      return;
    int row = getRowForPath(treePaths[treePaths.length - 1]);
    if (row >= 0 && getPathForRow(row + 1) != null)
      setSelectionRow(row + 1);
  }
}
