/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import javax.swing.JMenu;

import org.w3c.dom.Document;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.global.DocumentTypeInfoView;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.view.main.menu.Menu;

public class MenuDocument extends Menu {
  protected GlobalControl globalControl;
  protected GlobalViewControl globalViewControl;
  protected FileControl fileControl;
  protected EditControl editControl;
  protected GlobalOptionsControl globalOptionsControl;
  protected FullyFledgedDocument fullyFledgedDocument;
  protected Options options;

  public MenuDocument(
      FullyFledgedDocument fullyFledgedDocument,
      Options options, GlobalControl globalControl,
      GlobalViewControl globalViewControl,
      FileControl fileControl, EditControl editControl,
      GlobalOptionsControl globalOptionsControl,
      Shortcuts shortcuts) {
    super(Label.DOCUMENT, shortcuts);
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.globalControl = globalControl;
    this.globalViewControl = globalViewControl;
    this.fileControl = fileControl;
    this.editControl = editControl;
    this.globalOptionsControl = globalOptionsControl;
    boolean isAdvancedUse =
      options.getThreeStateOptionValue(Options.ADVANCEDUSE)
        == ThreeStateOption.TRUE;
    this.add(this.createMenuItem(
        Label.NOUVEAU_DOCUMENT, Shortcuts.SHORTCUT_NEW_DOCUMENT, true,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        newDocument_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.NOUVELLE_INSTANCE, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        newInstance_execute();
      }
    }));
    if (isAdvancedUse) {
      this.addSeparator();
      this.add(this.createMenuItem(
          Label.DOCUMENT_TYPES, Shortcuts.SHORTCUT_DOCUMENT_TYPES,
          new Action()  {
        public void execute(ActionEvent event) throws Exception {
          documentTypes_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.DOCUMENT_TYPE, Shortcuts.SHORTCUT_DOCUMENT_TYPE, true,
          new Action()  {
        public void execute(ActionEvent event) throws Exception {
          documentType_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.TRANSFORMERS, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          transformers_execute();
        }
      }));
      this.addSeparator();
      this.add(this.createMenuItem(
          Label.ADD_NAMESPACE, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          addNamespace_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.REMOVE_EMPTY_TEXTS, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          removeEmptyTexts_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.INDENT_ELEMENTS, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          indent_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.EDIT_SOURCE, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          editSource_execute();
        }
      }));
    }
  }

  protected void newDocument_execute() {
    fileControl.newDocument(null, null);
  }

  protected void newInstance_execute() {
    globalControl.createInstance(false, null, null);
  }

  protected void documentTypes_execute() {
    DocumentTypeInfoView documentTypeInfoView =
        globalViewControl.createDocumentTypeInfoView();
    documentTypeInfoView.show();
    if (fullyFledgedDocument.getDocument() != null)
      documentTypeInfoView.select(
          fullyFledgedDocument.getDocumentTypeName(),
          fullyFledgedDocument.getDocumentTypeAdditionalKey());
  }

  protected void documentType_execute() {
    if (fullyFledgedDocument.getDocument() != null)
      globalOptionsControl.editDocumentTypeInfo(
          fullyFledgedDocument.getDocumentTypeName(),
          fullyFledgedDocument.getDocumentTypeAdditionalKey());
  }

  protected void transformers_execute() {
    globalViewControl.createTransformerInfoView().show();
  }

  protected void addNamespace_execute() {
    editControl.addNamespace(null);
  }

  protected void removeEmptyTexts_execute() {
    fileControl.removeEmptyTexts();
  }

  protected void indent_execute() {
    fileControl.indent();
  }

  protected void editSource_execute() {
    fileControl.editSource();
  }
}
