/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.view.main.menu.Menu;

public final class MenuFragment extends Menu {
  private EditControl editControl;
  private ToolControl toolControl;
  private ToolView toolView;

  public MenuFragment(
      ToolView toolView, EditControl editControl,
      ToolControl toolControl, Shortcuts shortcuts) {
    super(Label.FRAGMENT, shortcuts);
    this.toolView = toolView;
    this.editControl = editControl;
    this.toolControl = toolControl;
    // Menu
    this.add(this.createMenuCutNodes());
    this.add(this.createMenuCopyNodes());
    this.add(this.createMenuImportNodes());
    this.addSeparator();
    this.add(this.createMenuPasteBefore());
    this.add(this.createMenuPasteAfter());
    this.add(this.createMenuPasteChild());
    this.addSeparator();
    this.add(this.createMenuReplaceNodes());
    this.addSeparator();
    this.add(this.createMenuCopyNodesToSystemClipboard());
    this.add(this.createMenuCopyFragmentToSystemClipboard());
    this.add(this.createMenuCopySystemClipboardToFragment());
    this.addSeparator();
    this.add(this.createMenuPasteBeforeFromSystemClipboard());
    this.add(this.createMenuPasteAfterFromSystemClipboard());
    this.add(this.createMenuPasteChildFromSystemClipboard());
  }

  public JMenuItem createMenuCutNodes() {
    return this.createMenuItem(
        Label.CUT, Shortcuts.SHORTCUT_CUT,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            cutNodes_execute();
          }
      });
  }

  public JMenuItem createMenuCopyNodes() {
    return this.createMenuItem(
        Label.COPY, Shortcuts.SHORTCUT_COPY,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            copyNodes_execute(false);
          }
      });
  }

  public JMenuItem createMenuCopyNodesToSystemClipboard() {
    return this.createMenuItem(
        Label.COPY_TO_CLIPBOARD, Shortcuts.SHORTCUT_COPY_TO_CLIP,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            copyNodes_execute(true);
          }
      });
  }

  public JMenuItem createMenuImportNodes() {
    return this.createMenuItem(
        Label.IMPORT, Shortcuts.SHORTCUT_IMPORT,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            importNodes_execute();
          }
      });
  }

  public JMenuItem createMenuReplaceNodes() {
    return this.createMenuItem(
        Label.REPLACE, Shortcuts.SHORTCUT_REPLACE_BY_FRAGMENT,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            replaceNodes_execute(false);
          }
      });
  }

  public JMenuItem createMenuPasteBefore() {
    return this.createMenuItem(
        Label.PASTE_BEFORE, Shortcuts.SHORTCUT_PASTE_BEFORE,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_PREVIOUS, false);
          }
      });
  }

  public JMenuItem createMenuPasteAfter() {
    return this.createMenuItem(
        Label.PASTE_AFTER, Shortcuts.SHORTCUT_PASTE_AFTER,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_NEXT, false);
          }
      });
  }

  public JMenuItem createMenuPasteChild() {
    return this.createMenuItem(
        Label.PASTE_UNDER, Shortcuts.SHORTCUT_PASTE_UNDER,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_CHILD, false);
          }
      });
  }

  public JMenuItem createMenuPasteBeforeFromSystemClipboard() {
    return this.createMenuItem(
        Label.PASTE_CLIPBOARD_BEFORE, Shortcuts.SHORTCUT_PASTE_CLIP_BEFORE,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_PREVIOUS, true);
          }
      });
  }

  public JMenuItem createMenuPasteAfterFromSystemClipboard() {
    return this.createMenuItem(
        Label.PASTE_CLIPBOARD_AFTER, Shortcuts.SHORTCUT_PASTE_CLIP_AFTER,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_NEXT, true);
          }
      });
  }

  public JMenuItem createMenuPasteChildFromSystemClipboard() {
    return this.createMenuItem(
        Label.PASTE_CLIPBOARD_UNDER, Shortcuts.SHORTCUT_PASTE_CLIP_UNDER,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            pasteFragment_execute(EditControl.INSERTION_CHILD, true);
          }
      });
  }

  public JMenuItem createMenuCopyFragmentToSystemClipboard() {
    return this.createMenuItem(
        Label.COPY_FRAGMENT_TO_CLIPBOARD, Shortcuts.SHORTCUT_TO_CLIP,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            fragmentToClipboard_execute();
          }
      });
  }

  public JMenuItem createMenuCopySystemClipboardToFragment() {
    return this.createMenuItem(
        Label.COPY_CLIPBOARD_TO_FRAGMENT, Shortcuts.SHORTCUT_FROM_CLIP,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            clipboardToFragment_execute();
          }
      });
  }

  private void cutNodes_execute() {
    editControl.cutNodes(null, false);
  }

  private void copyNodes_execute(boolean copyToSystemClipoard) {
    editControl.copyNodes(null, copyToSystemClipoard);
  }

  private void importNodes_execute() {
    toolControl.importFragment();
  }

  private void replaceNodes_execute(final boolean copyToSystemClipoard) {
    editControl.replaceNodesByDocumentFragment(null, copyToSystemClipoard);
  }

  private void pasteFragment_execute(
      final int insertionType, final boolean copyFromSystemClipboard) {
    editControl.pasteDocumentFragment(
        null, insertionType, copyFromSystemClipboard);
  }

  private void fragmentToClipboard_execute() {
    editControl.copyDocumentFragmentToSystemClipboard();
  }

  private void clipboardToFragment_execute() {
    editControl.copySystemClipboardToDocumentFragment();
  }
}
