/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.model;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.view.model.DomTreeNode;
import org.xmloperator.view.model.DomTreeModel;

// DomTreeModel restricted to elements with ID.
public class DomIdTreeModel extends DomTreeModel {

  public DomIdTreeModel(FullyFledgedDocument fullyFledgedDocument) {
    super(fullyFledgedDocument);
  }

  public Object getChild(Object parent, int index) {
    if (parent == root)
      return index == 0 ? rootDomTreeNode : null;
    if (index < 0)
      return null;
    Node childNode = ((DomTreeNode)parent).getNode().getFirstChild();
    while (true) {
      while (true) {
        if (childNode == null)
          return null;
        if (isNodeDisplayable(childNode))
          break;
        childNode = childNode.getNextSibling();
      }
      if (index == 0)
        break;
      index--;
      childNode = childNode.getNextSibling();
    }
    return this.domTreeNode(childNode);
  }

  public int getChildCount(Object parent) {
    if (parent == root)
      return rootDomTreeNode != null ? 1 : 0;
    int count = 0;
    Node childNode = ((DomTreeNode)parent).getNode().getFirstChild();
    while (childNode != null) {
      if (isNodeDisplayable(childNode))
        count++;
      childNode = childNode.getNextSibling();
    }
    return count;
  }

  public int getIndexOfChild(Object parent, Object child) {
    if (parent == root)
      return child != null && child == rootDomTreeNode ? 0 : -1;
    int index = 0;
    Node childNode = ((DomTreeNode)parent).getNode().getFirstChild();
    while (childNode != null) {
      if (this.domTreeNode(childNode) == child)
        return index;
      if (isNodeDisplayable(childNode))
        index++;
      childNode = childNode.getNextSibling();
    }
    return -1;
  }

  // A node is displayable if it's an element with an ID
  //   or if it has a descendant with an ID.
  public final boolean isNodeDisplayable(Node node) {
    if (node.getNodeType() != Node.ELEMENT_NODE)
      return false;
    AttributesContent attributesContent = fullyFledgedDocument.
      getDocumentUtil().getAttributesContent((Element)node);
    if (attributesContent == null)
      return false;
    String idAttrName = attributesContent.getIdAttributeName();
    if (idAttrName != null
      && ((Element)node).getAttribute(idAttrName).length() != 0)
      return true;
    Node childNode = ((Element)node).getFirstChild();
    while (childNode != null) {
      if (isNodeDisplayable(childNode))
        return true;
      childNode = childNode.getNextSibling();
    }
    return false;
  }
}
