/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.model;

import org.w3c.dom.Node;

import org.xmloperator.framework.document.util.NodeLinker;
import org.xmloperator.framework.document.elementKey.ElementKey;
import org.xmloperator.framework.document.elementKey.ElementKeyProvider;
import org.xmloperator.document.elementKey.ElementKeyUtil;
import org.xmloperator.util.exception.ExceptionControler;

public final class DomTreeNode {
  private Node node;
  private ElementKeyProvider elementKeyProvider;

  public static final DomTreeNode domTreeNodeFromNode(final Node node,
    final NodeLinker nodeLinker, final ElementKeyProvider elementKeyProvider) {
    if (node == null)
      return null;
    if (nodeLinker == null)
      return new DomTreeNode(node, elementKeyProvider);
    DomTreeNode domTreeNode = (DomTreeNode)nodeLinker.getNodeUserData(node);
    if (domTreeNode == null) {
      domTreeNode = new DomTreeNode(node, elementKeyProvider);
      nodeLinker.setNodeUserData(node, domTreeNode);
    }
    return domTreeNode;
  }

  public DomTreeNode() {
    this(null, null);
  }

  public DomTreeNode(Node node, ElementKeyProvider elementKeyProvider) {
    this.node = node;
    this.elementKeyProvider = elementKeyProvider;
  }

  public Node getNode() {
    return this.node;
  }

  public String toString() {
    if (this.node == null)
      return "";
    try {
      ElementKey elementKey = this.node.getNodeType() == Node.ELEMENT_NODE
          && this.elementKeyProvider != null ?
          this.elementKeyProvider.getElementKey(this.node.getNodeName()) : null;
      return ElementKeyUtil.nodeToString(this.node, elementKey);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return "";
    }
  }
}
