/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.List;
import java.util.Arrays;
import java.util.Set;
import java.util.HashSet;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.ViewUtils;

public final class PanelDocument extends JPanel {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private FileControl fileControl;
  private EditControl editControl;
  private Options options;
  private final JPanel panelDocumentSystemId = new JPanel(new BorderLayout());
  private final JLabel labelDocumentSystemId = new JLabel("System Id : ");
  private final JTextField fieldDocumentSystemId = new JTextField();
  private final JTextField fieldDocumentTypeName = new JTextField();
  private final JComboBox comboBoxAdditionalKey = new JComboBox();
  private final JComboBox comboBoxEncoding =
      new JComboBox(Constants.encodingNames);
  private final JComboBox comboBoxStandalone =
    new JComboBox(new String[] {"no (default)", "yes"});
  private final JComboBox comboBoxXMLVersion =
    new JComboBox(Constants.xMLVersions);
  private final JLabel labelDTDNotOriginal = new JLabel();
  private boolean isDisplaying = false;

  public PanelDocument(FullyFledgedDocument fullyFledgedDocument,
    EditControl editControl, FileControl fileControl,
    Options options, boolean isLandscape) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.fileControl = fileControl;
    this.editControl = editControl;
    this.options = options;
    EmptyBorder thinEmptyBorder = new EmptyBorder(2, 4, 2, 4);
    JPanel panelNorth = new JPanel();
    panelNorth.setLayout(new BoxLayout(panelNorth, BoxLayout.Y_AXIS));
    this.add(panelNorth, BorderLayout.NORTH);
    Font font = options.getThreeStateOptionValue(Options.FONT)
      == ThreeStateOption.TRUE ? Constants.FONT_LARGE : Constants.FONT_NORMAL;
    // Document System Id
    panelDocumentSystemId.setBorder(thinEmptyBorder);
    panelDocumentSystemId.add(
        labelDocumentSystemId,
        isLandscape ? BorderLayout.WEST : BorderLayout.NORTH);
    fieldDocumentSystemId.setFont(font);
    fieldDocumentSystemId.setEnabled(false);
    panelDocumentSystemId.add(fieldDocumentSystemId);
    panelNorth.add(panelDocumentSystemId);
    // Document type name
    JPanel panelDocumentTypeName = new JPanel(new BorderLayout());
    panelDocumentTypeName.setBorder(thinEmptyBorder);
    panelDocumentTypeName.add(
      new JLabel(viewResource.getString(Label.TYPE) + " : "),
      BorderLayout.WEST);
    fieldDocumentTypeName.setFont(font);
    fieldDocumentTypeName.setEnabled(false);
    panelDocumentTypeName.add(fieldDocumentTypeName);
    panelNorth.add(panelDocumentTypeName);
    // Document type additional key
    JPanel panelDocumentTypeAdditionalKey = new JPanel(new BorderLayout());
    panelDocumentTypeAdditionalKey.setBorder(thinEmptyBorder);
    panelDocumentTypeAdditionalKey.add(
      new JLabel(viewResource.getString(Label.ADDITIONAL_KEY) + " : "),
      BorderLayout.WEST);
    comboBoxAdditionalKey.setFont(font);
    comboBoxAdditionalKey.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          additionalKey_actionPerformed();
        }
      });
    panelDocumentTypeAdditionalKey.add(comboBoxAdditionalKey);
    panelNorth.add(panelDocumentTypeAdditionalKey);
    // Encoding
    JPanel panelEncoding = new JPanel(new BorderLayout());
    panelEncoding.setBorder(thinEmptyBorder);
    panelEncoding.add(new JLabel("Encoding : "), BorderLayout.WEST);
    comboBoxEncoding.setFont(font);
    comboBoxEncoding.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          encoding_actionPerformed();
        }
      });
    panelEncoding.add(comboBoxEncoding);
    panelNorth.add(panelEncoding);
    // Standalone
    JPanel panelStandalone = new JPanel(new BorderLayout());
    panelStandalone.setBorder(thinEmptyBorder);
    panelStandalone.add(new JLabel("Standalone : "), BorderLayout.WEST);
    comboBoxStandalone.setFont(font);
    comboBoxStandalone.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          standalone_actionPerformed();
        }
      });
    panelStandalone.add(comboBoxStandalone);
    panelNorth.add(panelStandalone);
    // XML Version
    JPanel panelXMLVersion = new JPanel(new BorderLayout());
    panelXMLVersion.setBorder(thinEmptyBorder);
    panelXMLVersion.add(new JLabel("XML version : "), BorderLayout.WEST);
    comboBoxXMLVersion.setFont(font);
    comboBoxXMLVersion.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          xMLVersion_actionPerformed();
        }
      });
    panelXMLVersion.add(comboBoxXMLVersion);
    panelNorth.add(panelXMLVersion);
    // Space
    JPanel panelSpace = new JPanel();
    panelSpace.setBorder(thinEmptyBorder);
    panelNorth.add(panelSpace);
    // DTD not original
    JPanel panelDTDNotOriginal = new JPanel(new BorderLayout());
    panelDTDNotOriginal.setBorder(thinEmptyBorder);
    panelDTDNotOriginal.add(labelDTDNotOriginal);
    panelNorth.add(panelDTDNotOriginal);
  }


  public void setLayout(boolean isLandscape) {
    ViewUtils.setBorderLayout(
        this.panelDocumentSystemId, this.labelDocumentSystemId, isLandscape);
  }

  private void additionalKey_actionPerformed() {
    if (isDisplaying)
      return;
    String additionalKey = (String)comboBoxAdditionalKey.getSelectedItem();
    fileControl.setDocumentTypeAdditionalKey(additionalKey);
  }

  private void encoding_actionPerformed() {
    if (isDisplaying)
      return;
    String encoding = (String)comboBoxEncoding.getSelectedItem();
    editControl.setEncoding(encoding);
  }

  private void standalone_actionPerformed() {
    if (isDisplaying)
      return;
    boolean isStandalone = comboBoxStandalone.getSelectedIndex() == 1;
    editControl.setStandalone(isStandalone);
  }

  private void xMLVersion_actionPerformed() {
    if (isDisplaying)
      return;
    String xMLVersion = (String)comboBoxXMLVersion.getSelectedItem();
    editControl.setXMLVersion(xMLVersion);
  }

  public void display() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    isDisplaying = true;
    try {
      fieldDocumentSystemId.setText(fullyFledgedDocument.getDocumentSystemId());
      boolean isAdvancedUse =
        options.getThreeStateOptionValue(Options.ADVANCEDUSE)
          == ThreeStateOption.TRUE;
      {
        String rootElementName = document.getDocumentElement().getNodeName();
        fieldDocumentTypeName.setText(rootElementName);
        if (this.comboBoxAdditionalKey.getItemCount() != 0)
          this.comboBoxAdditionalKey.removeAllItems();
        Set keySet = new HashSet();
        List documentTypeInfoList = options.getOptionList(Options.DOCUMENTTYPE);
        for (int k = 0; k < documentTypeInfoList.size(); k++) {
          DocumentTypeInfo documentTypeInfo =
              (DocumentTypeInfo)documentTypeInfoList.get(k);
          String documentTypeName = documentTypeInfo.getDocumentTypeName();
          if (documentTypeName.equals(rootElementName)
              || documentTypeName.length() == 0) {
            String additionalKey =
                documentTypeInfo.getDocumentTypeAdditionalKey();
            this.comboBoxAdditionalKey.addItem(additionalKey);
            keySet.add(additionalKey);
          }
        }
        String additionalKey =
            fullyFledgedDocument.getDocumentTypeAdditionalKey();
        if (additionalKey == null)
          additionalKey = "";
        if (!keySet.contains(additionalKey))
          this.comboBoxAdditionalKey.addItem(additionalKey);
        comboBoxAdditionalKey.setSelectedItem(additionalKey);
      }
      {
        int encodingIndex = Arrays.asList(Constants.encodingNames).
                            indexOf(fullyFledgedDocument.getEncoding());
        if (encodingIndex < 0)
          encodingIndex = 0;
        comboBoxEncoding.setSelectedIndex(encodingIndex);
      }
      comboBoxEncoding.setEnabled(isAdvancedUse);
      comboBoxStandalone.setSelectedIndex(
        fullyFledgedDocument.isStandalone() ? 1 : 0);
      comboBoxStandalone.setEnabled(isAdvancedUse);
      comboBoxXMLVersion.setSelectedItem(fullyFledgedDocument.getXMLVersion());
      comboBoxXMLVersion.setEnabled(isAdvancedUse);
      {
        DTD dTD = fullyFledgedDocument.getDTD();
        labelDTDNotOriginal.setText(
            dTD != fullyFledgedDocument.getOriginalDTD() ?
            viewResource.getString(
            Label.THE_USED_DTD_IS_NOT_ATTACHED_TO_THE_DOCUMENT) + "." : null);
      }
    }
    finally {
      isDisplaying = false;
    }
  }
}
