/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.ProcessingInstruction;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.StringOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.util.exception.ExceptionControler;

public final class PanelPITarget extends JPanel {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private EditControl editControl;
  private Options options;
  private boolean isDisplaying = false;
  private final JComboBox comboBoxTarget = new JComboBox();

  public PanelPITarget(
    FullyFledgedDocument fullyFledgedDocument,
    EditControl editControl, Options options) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.editControl = editControl;
    this.options = options;
    Border emptyBorder = new EmptyBorder(4, 4, 4, 4);
    // Target
    JPanel voletTarget = new JPanel(new BorderLayout());
    voletTarget.setBorder(emptyBorder);
    comboBoxTarget.setFont(
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE ?
      Constants.FONT_LARGE : Constants.FONT_NORMAL);
    comboBoxTarget.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          target_actionPerformed();
        }
      });
    voletTarget.add(comboBoxTarget);
    this.add(voletTarget, BorderLayout.NORTH);
  }

  private void target_actionPerformed() {
    if (isDisplaying)
      return;
    editControl.replaceNode(null, Node.PROCESSING_INSTRUCTION_NODE,
      (String)comboBoxTarget.getSelectedItem());
  }

  public void display(ProcessingInstruction processingInstruction) {
    if (processingInstruction == null)
      throw new IllegalArgumentException();
    isDisplaying = true;
    try {
      if (this.comboBoxTarget.getItemCount() != 0)
        this.comboBoxTarget.removeAllItems();
      List targetList = new ArrayList();
      List targetOptionList = options.getOptionList(Options.TARGET);
      if (targetOptionList != null)
        for (int k = 0; k < targetOptionList.size(); k++)
          targetList.add(
              ((StringOption)targetOptionList.get(k)).getStringValue());
      String currentTarget = processingInstruction.getTarget();
      if (!targetList.contains(currentTarget))
        targetList.add(currentTarget);
      Collections.sort(targetList);
      for (int k = 0; k < targetList.size(); k++)
        this.comboBoxTarget.addItem(targetList.get(k));
      comboBoxTarget.setSelectedItem(currentTarget);
      comboBoxTarget.setEnabled(
        options.getThreeStateOptionValue(Options.ALLOWSPROCESSINGINSTRUCTION)
        == ThreeStateOption.TRUE && !fullyFledgedDocument.isReadOnly());
    }
    finally {
      isDisplaying = false;
    }
  }
}
