/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.util.ResourceBundle;
import java.awt.Font;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.node.TextSelection;
import org.xmloperator.framework.control.instance.EditControl;

public final class PanelTextNode extends JPanel implements TextSelection {
  private FullyFledgedDocument fullyFledgedDocument;
  private EditControl editControl;
  private Options options;
  private JTextArea champEdition;
  private boolean isDisplaying = false;
  private boolean isUpdatingField = false;
  private Node currentNode = null;

  public PanelTextNode(FullyFledgedDocument fullyFledgedDocument,
    EditControl editControl, Options options) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.editControl = editControl;
    this.options = options;
    Border emptyBorder = new EmptyBorder(4, 4, 4, 4);
    // Champ d'dition
    JPanel voletEdition = new JPanel(new BorderLayout());
    voletEdition.setBorder(emptyBorder);
    champEdition = new JTextArea();
    champEdition.setLineWrap(true);
    champEdition.setWrapStyleWord(true);
    {
      boolean isMonospace =
          options.getThreeStateOptionValue(Options.MONOSPACED)
          == ThreeStateOption.TRUE;
      boolean isLargeFont =
          options.getThreeStateOptionValue(Options.FONT)
          == ThreeStateOption.TRUE;
      champEdition.setFont(
          isMonospace ?
          isLargeFont ? Constants.FONT_MONOSPACE_LARGE :
          Constants.FONT_MONOSPACE_NORMAL :
          isLargeFont ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
    }
    champEdition.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          champEdition_update();
        };
        public void removeUpdate(DocumentEvent event) {
          champEdition_update();
        };
        public void changedUpdate(DocumentEvent event) {
          champEdition_update();
        };
      });
    voletEdition.add(new JScrollPane(champEdition));
    this.add(voletEdition);
  }

  private void champEdition_update() {
    if (isDisplaying || currentNode == null)
      return;
    isUpdatingField = true;
    try {
      editControl.setNodeValue(currentNode, champEdition.getText());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      isUpdatingField = false;
    }
  }

  public void display(Node node) {
    if (isUpdatingField)
      return;
    currentNode = node;
    isDisplaying = true;
    try {
      champEdition.setText(node.getNodeValue());
      champEdition.setEnabled(!fullyFledgedDocument.isReadOnly()
        && isNodeTypeManageable(node.getNodeType()));
    }
    catch (Exception exception) {
    }
    finally {
      isDisplaying = false;
    }
  }

  private boolean isNodeTypeManageable(int nodeType) {
    String optionId = null;
    switch (nodeType) {
      case Node.COMMENT_NODE:
        optionId = Options.ALLOWSCOMMENT;
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
        optionId = Options.ALLOWSPROCESSINGINSTRUCTION;
        break;
      case Node.CDATA_SECTION_NODE:
        optionId = Options.ALLOWSCDATASECTION;
        break;
    }
    return optionId == null
      || options.getThreeStateOptionValue(optionId) == ThreeStateOption.TRUE;
  }

  public void selectLastSearchNextInstance() {
    String text = champEdition.getText();
    String lastSearch = options.getStringOptionValue(Options.LASTSEARCH);
    boolean isCaseIgnored =
      options.getThreeStateOptionValue(Options.CASEIGNOREDFORSEARCH)
      == ThreeStateOption.TRUE;
    int searchLength = lastSearch.length();
    if (searchLength == 0)
      return;
    int position =
      search(text, lastSearch, isCaseIgnored, champEdition.getCaretPosition());
    if (position < 0) {
      position = search(text, lastSearch, isCaseIgnored, 0);
      if (position < 0)
        return;
    }
    select(position, searchLength);
  }

  private static final int search(String text,
    String searchedText, boolean isCaseIgnored, int fromPosition) {
    text = text.substring(fromPosition);
    if (isCaseIgnored) {
      text = text.toUpperCase();
      searchedText = searchedText.toUpperCase();
    }
    int index = text.indexOf(searchedText);
    return index >= 0 ? fromPosition + index : -1;
  }

  private void select(int position, int length) {
    try {
      champEdition.setCaretPosition(position);
      champEdition.moveCaretPosition(position + length);
    }
    catch (Exception exception) {
      isDisplaying = true;
      try {
        champEdition.setText(champEdition.getText());
      }
      catch (Exception exception2) {
      }
      finally {
        isDisplaying = false;
      }
    }
  }

  ///////////////////////////////
  // TextSelection implementation
  ///////////////////////////////

  public String getText() {
    return this.champEdition.getText();
  }

  public int getCaretPosition() {
    return champEdition.getCaretPosition();
  }

  public int getMarkPosition() {
    return champEdition.getCaret().getMark();
  }

  public String getSelectedText() {
    int p0 = this.getCaretPosition();
    int p1 = this.getMarkPosition();
    if (p0 == p1)
      return "";
    if (p0 > p1) {
      int p = p0;
      p0 = p1;
      p1 = p;
    }
    return this.getText().substring(p0, p1);
  }
}
