/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node.attribute;

import java.util.EventObject;
import java.util.Vector;
import java.awt.Font;
import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.SwingConstants;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.node.AttributeListener;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogSelectionIds;
import org.xmloperator.framework.view.instance.IdsDialogAccess;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.ViewUtils;

public final class AttributeTableCellEditor implements TableCellEditor {
  public static final int ATTRIBUTE_COLUMN_NAME = 0;
  public static final int ATTRIBUTE_COLUMN_VALUE = 1;
  private Frame frame;
  private JTable attributeTable;
  private AttributeListener attributeListener;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private IdsDialogAccess idsDialogAccess;
  private JTextField champValeur;
  private JComboBox comboBoxEnumeration;
  private JButton boutonReference;
  private JButton boutonId;
  private Element element = null;
  private int currentAttributeIndex = -1;
  private boolean isCurrentAttributeModifyable = false;
  private boolean isSettingEditor = false;

  public AttributeTableCellEditor(Frame frame, JTable attributeTable,
    AttributeListener attributeListener,
    FullyFledgedDocument fullyFledgedDocument, Options options,
    IdsDialogAccess idsDialogAccess) {
    this.frame = frame;
    this.attributeTable = attributeTable;
    this.attributeListener = attributeListener;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.idsDialogAccess = idsDialogAccess;
    Font font = options.getThreeStateOptionValue(Options.FONT)
      == ThreeStateOption.TRUE ? Constants.FONT_LARGE : Constants.FONT_NORMAL;
    champValeur = new JTextField();
    champValeur.setFont(font);
    champValeur.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          champValeur_update();
        };
        public void removeUpdate(DocumentEvent event) {
          champValeur_update();
        };
        public void changedUpdate(DocumentEvent event) {
          champValeur_update();
        };
      });
    comboBoxEnumeration = new JComboBox();
    comboBoxEnumeration.setFont(font);
    comboBoxEnumeration.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          enumeration_actionPerformed();
        }
      });
    boutonReference = new JButton();
    boutonReference.setHorizontalAlignment(SwingConstants.LEFT);
    boutonReference.setFont(font);
    boutonReference.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          boutonReference_actionPerformed();
        }
      });
    boutonId = new JButton();
    boutonId.setHorizontalAlignment(SwingConstants.LEFT);
    boutonId.setFont(new Font("dialog", Font.PLAIN,
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE ?
      14 : 12));
    boutonId.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          boutonId_actionPerformed();
        }
      });
  }

  public void setElement(Element element) {
    this.element = element;
    currentAttributeIndex = -1;
    isCurrentAttributeModifyable = false;
  }

  public void setAttributeIndex(int attributeIndex) {
    if (element == null)
      return;
    NamedNodeMap attrs = element.getAttributes();
    if (attributeIndex < 0 || attributeIndex >= attrs.getLength())
      attributeIndex = -1;
    if (attributeIndex == currentAttributeIndex)
      return;
    isSettingEditor = true;
    try {
      if (isCurrentAttributeModifyable)
        attributeTable.removeEditor();
      if (attributeIndex >= 0) {
        currentAttributeIndex = attributeIndex;
        Attr attr = (Attr)attrs.item(attributeIndex);
        isCurrentAttributeModifyable = !fullyFledgedDocument.isReadOnly();
        if (isCurrentAttributeModifyable)
          attributeTable.editCellAt(attributeIndex, ATTRIBUTE_COLUMN_VALUE);
      }
      else {
        currentAttributeIndex = -1;
        isCurrentAttributeModifyable = false;
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      isSettingEditor = false;
    }
  }

  public Attr getCurrentAttribute() {
    if (element == null)
      return null;
    NamedNodeMap attrs = element.getAttributes();
    return currentAttributeIndex >= 0
      && currentAttributeIndex < attrs.getLength() ?
      (Attr)attrs.item(currentAttributeIndex) : null;
  }

  private void champValeur_update() {
    attribute_update(champValeur.getText());
  }

  private void enumeration_actionPerformed() {
    attribute_update((String)comboBoxEnumeration.getSelectedItem());
  }

  private void boutonReference_actionPerformed() {
    try {
      Attr attr = getCurrentAttribute();
      if (attr == null)
        return;
      AttributesContent attributesContent = fullyFledgedDocument.
        getElementDefinitionMapper().getAttributesContent(element);
      if (attributesContent == null)
        return;
      String attrName = attr.getName();
      String attrType = attributesContent.getType(attrName);
      String attrValue = attr.getValue();
      if (attrType.equals(AttributesContent.TYPE_IDREF)) {
        DialogSelectionIds dialogueSelectionId =
            this.idsDialogAccess.getDialogSelectionId(true);
        dialogueSelectionId.setTitle("IDREF \"" + attrName + '"');
        if (attrValue.length() != 0)
          dialogueSelectionId.setSelectedIds(attrValue, true);
        else {
          String contextId = this.contextId();
          dialogueSelectionId.setSelectedIds(contextId, false);
        }
        dialogueSelectionId.show();
        String selectedId = dialogueSelectionId.getSelectedIds();
        if (selectedId == null)
          return;
        attribute_update(selectedId);
        boutonReference.setText(selectedId);
      }
      else if (attrType.equals(AttributesContent.TYPE_IDREFS)) {
        DialogSelectionIds dialogueSelectionIds =
            this.idsDialogAccess.getDialogSelectionIds();
        dialogueSelectionIds.setTitle("IDREFS \"" + attrName + '"');
        if (attrValue.length() != 0)
          dialogueSelectionIds.setSelectedIds(attrValue, true);
        else {
          String contextId = this.contextId();
          dialogueSelectionIds.setSelectedIds(contextId, false);
        }
        dialogueSelectionIds.show();
        String selectedIds = dialogueSelectionIds.getSelectedIds();
        if (selectedIds == null)
          return;
        attribute_update(selectedIds);
        boutonReference.setText(selectedIds);
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private String contextId() {
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return null;
    Node contextNode = element;
    while (contextNode != null
      && contextNode.getNodeType() == Node.ELEMENT_NODE) {
      String id = documentUtil.getId((Element)contextNode);
      if (id != null && id.length() != 0)
        return id;
      contextNode = contextNode.getParentNode();
    }
    return null;
  }

  private void boutonId_actionPerformed() {
    try {
      Attr attr = getCurrentAttribute();
      if (attr == null)
        return;
      AttributesContent attributesContent = fullyFledgedDocument.
        getElementDefinitionMapper().getAttributesContent(element);
      if (attributesContent == null)
        return;
      String attrName = attr.getName();
      String attrType = attributesContent.getType(attrName);
      if (attrType.equals(AttributesContent.TYPE_ID)) {
        IdManagerRead idManager = fullyFledgedDocument.getIdManager();
        String id = attr.getValue();
        if (id.length() == 0)
          id = idManager.getNewVerifiedId();
        boolean isValueRequired = idManager.isIdReferenced(id);
        DialogText dialogReplaceId =
            this.idsDialogAccess.getDialogReplaceId(isValueRequired);
        dialogReplaceId.setTitle("ID \"" + attrName + '"');
        dialogReplaceId.setText(id);
        dialogReplaceId.show();
        if (!dialogReplaceId.isOK())
          return;
        id = dialogReplaceId.getText();
        attribute_update(id);
        boutonId.setText(id);
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void attribute_update(String newValue) {
    Attr attr = getCurrentAttribute();
    if (attr != null && !isSettingEditor)
      attributeListener.setAttribute(attr.getName(), newValue);
  }

  ////////////////////////////
  // CellEditor implementation
  ////////////////////////////

  public Object getCellEditorValue() {
    return null; // Editing is never validated
  }

  public boolean isCellEditable(EventObject anEvent) {
    return isCurrentAttributeModifyable;
  }

  public boolean shouldSelectCell(EventObject anEvent) {
    return true;
  }

  public boolean stopCellEditing() {
    return true;
  }

  public void cancelCellEditing() {
  }

  public void addCellEditorListener(CellEditorListener listener) {
    // Not used
  }

  public void removeCellEditorListener(CellEditorListener listener) {
    // Not used
  }

  /////////////////////////////////
  // TableCellEditor implementation
  /////////////////////////////////

  public Component getTableCellEditorComponent(JTable table, Object value,
    boolean isSelected, int row, int column) {
    Attr currentAttr = getCurrentAttribute();
    if (currentAttr == null)
      return champValeur;
    boolean isCurrentAttribute = row == currentAttributeIndex;
    String attrValue = currentAttr.getValue();
    if (isCurrentAttribute)
      champValeur.setText(attrValue);
    AttributesContent attributesContent = fullyFledgedDocument.
      getElementDefinitionMapper().getAttributesContent(element);
    if (attributesContent == null)
      return champValeur;
    String attrName = currentAttr.getName();
    String attrType = attributesContent.getType(attrName);
    if (attrType.equals(AttributesContent.TYPE_ENUMERATION)) {
      if (isCurrentAttribute) {
        if (this.comboBoxEnumeration.getItemCount() != 0)
          this.comboBoxEnumeration.removeAllItems();
        String[] enumeration = attributesContent.getEnumeration(attrName);
        for (int k = 0; k < enumeration.length; k++)
          this.comboBoxEnumeration.addItem(enumeration[k]);
        comboBoxEnumeration.setSelectedItem(attrValue);
      }
      return comboBoxEnumeration;
    }
    else if (fullyFledgedDocument.getIdManager().isEnabled()
      && attrType.indexOf(AttributesContent.TYPE_ID) == 0) {
      if (attrType.equals(AttributesContent.TYPE_ID)) {
        if (isCurrentAttribute)
          boutonId.setText(attrValue);
        return boutonId;
      }
      else {
        if (isCurrentAttribute)
          boutonReference.setText(attrValue);
        return boutonReference;
      }
    }
    else
      return champValeur;
  }
}
