/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.stat;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Event;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;

import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.view.stat.ElementCount;
import org.xmloperator.view.stat.ElementCountComparator;
import org.xmloperator.view.stat.ElementsCounter;

public class DialogElementsStatistic extends JDialog {
  protected static final int COLUMN_COUNT = 0;
  protected static final int COLUMN_NAME = 1;
  protected static final String[] enteteColonne = {
    Label.NOMBRE,
    Label.ELEMENT};
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected TableModel tableModel;
  protected JTable table;
  protected List stats;

  public DialogElementsStatistic(Frame parent, Element rootElement,
    Options options) {
    // construction d'ensemble
    super(parent, true);
    if (rootElement == null)
      throw new IllegalArgumentException();
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    stats = ElementsCounter.elementStats(rootElement);
    Collections.sort(stats, new ElementCountComparator(false));
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(400, 300));
    final ActionListener disposeActionListener =
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          dispose();
        }
      };
    voletContenu.registerKeyboardAction(disposeActionListener, "ESCAPE",
      KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    // Volet du bouton Ajouter
    JPanel voletBouton = new JPanel(new FlowLayout());
    JButton boutonValider =
      new JButton(viewResource.getString(Label.VALIDER));
    boutonValider.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.VALIDER).charAt(0));
    boutonValider.addActionListener(disposeActionListener);
    voletBouton.add(boutonValider);
    voletContenu.add(voletBouton, BorderLayout.SOUTH);
    // volet statistique
    tableModel = new AbstractTableModel() {
      public String getColumnName(int col) {
        return table_getColumnName(col);
      }
      public Class getColumnClass(int col) {
        return (col == COLUMN_COUNT) ? Integer.class : String.class;
      }
      public int getRowCount() {
        return table_getRowCount();
      }
      public int getColumnCount() {
        return enteteColonne.length;
      }
      public Object getValueAt(int row, int col) {
        return table_getValueAt(row, col);
      }
    };
    table = new JTable(tableModel);
    table.setShowVerticalLines(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    boolean isFontLarge =
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE;
    table.setRowHeight(isFontLarge ? 20 : 18);
    table.setFont(isFontLarge ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
    JTableHeader tableHeader = table.getTableHeader();
    tableHeader.setResizingAllowed(true);
    tableHeader.setReorderingAllowed(false);
    tableHeader.addMouseListener(
      new MouseAdapter() {
        public void mousePressed(MouseEvent event) {
          tableHeader_mousePressed(event);
        }
      });
    TableColumn nameColumn =
      tableHeader.getColumnModel().getColumn(COLUMN_NAME);
    nameColumn.setPreferredWidth(2 * nameColumn.getPreferredWidth());
    JScrollPane scrollPane = new JScrollPane(table);
    voletContenu.add(scrollPane);
    // Mise au dimensions
    pack();
  }

  protected void processWindowEvent(WindowEvent event) {
    if(event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  protected void tableHeader_mousePressed(MouseEvent event) {
    JTableHeader tableHeader = table.getTableHeader();
    Point clickPoint = event.getPoint();
    boolean isSortedByCount;
    if (tableHeader.getHeaderRect(COLUMN_NAME).contains(clickPoint))
      isSortedByCount = false;
    else if (tableHeader.getHeaderRect(COLUMN_COUNT).contains(clickPoint))
      isSortedByCount = true;
    else
      return;
    Collections.sort(stats, new ElementCountComparator(isSortedByCount));
    table.repaint();
  }

  protected String table_getColumnName(int col) {
    return (col >= 0 && col < enteteColonne.length) ?
      viewResource.getString(enteteColonne[col]) : "";
  }

  protected int table_getRowCount() {
    return stats.size();
  }

  protected Object table_getValueAt(int row, int col) {
    if (row >= 0 && row < stats.size()) {
      ElementCount stat = (ElementCount)stats.get(row);
      switch (col) {
        case COLUMN_COUNT:
          return new Integer(stat.getCount());
        case COLUMN_NAME:
          return stat.getName();
      }
    }
    return "";
  }
}
